@@ -2,13 +2,35 @@
 #
 # This file creates a w32 binary release
 #
-VERSION=0.92
+VERSION=1.00
 DDIR="`pwd`/Enigma-$VERSION"
-SDIR=../..
+SDIR=..
 BDIR=../
+DLLDEFAULTS="SDL SDL_image SDL_mixer SDL_ttf jpeg libpng libpng1 zlib zlib-1"
 
+if [ "$ENIGMADEVDLLS" != "" ]; then
+  DLLS=$ENIGMADEVDLLS
+else
+  DLLS=$DLLDEFAULTS
+fi
+
+STRIP=
+if [ "$STRIP" = "" ]; then
+  STRIP=strip
+fi
+
+if [ "$ENIGMADEVCROSSPREFIX" != "" ]; then
+  PATH="$ENIGMADEVCROSSPREFIX/bin:$PATH"
+fi
+
+# copy text file with newline convertion if necessary
 function cptext {
-    $NLCVT unix2dos $SDIR/$1 >$DDIR/$2
+    $NLCVT dos2unix $SDIR/$1 >$DDIR/$2 2> /dev/null
+    if test $? -ne 0; then
+        $NLCVT unix2dos $SDIR/$1 >$DDIR/$2
+    else
+        cp $SDIR/$1 $DDIR/$2
+    fi
 }
 
 function cpfile {
@@ -19,6 +41,16 @@ function cpbuiltfile {
 	cp $BDIR/$1 $DDIR/$2
 }
 
+# copy files of an extention with newline convertion to lf if necessary
+function cplffiles {
+    for f in `(cd $SDIR/$1 && ls *.$3)`; do
+        $NLCVT dos2unix $SDIR/$1/$f >$DDIR/$2/$f 2> /dev/null
+        if test $? -ne 0; then
+            cp $SDIR/$1/$f $DDIR/$2/$f
+        fi
+    done
+}
+
 #
 # Copy levels
 #
@@ -26,10 +58,16 @@ function copy_levels
 {
     echo "... copying levels"
     mkdir $DDIR/data/levels
-    cp -p $SDIR/data/levels/*.{png,txt,lua,xml} $DDIR/data/levels
+    cp -p $SDIR/data/levels/*.png $DDIR/data/levels
+    cplffiles data/levels data/levels lua
+    cplffiles data/levels data/levels txt
+    cplffiles data/levels data/levels xml
     for folder in `(cd $SDIR/data/levels && ls -F)| grep / | grep -v CVS`; do 
 	mkdir $DDIR/data/levels/$folder; 
-	cp -p $SDIR/data/levels/$folder/*.{png,lua,txt} $DDIR/data/levels/$folder
+	cp -p $SDIR/data/levels/$folder/*.png $DDIR/data/levels/$folder
+        cplffiles data/levels/$folder data/levels/$folder lua
+        cplffiles data/levels/$folder data/levels/$folder txt
+        cplffiles data/levels/$folder data/levels/$folder xml
     done
 }
 
@@ -37,11 +75,15 @@ function copy_data
 {
     echo "... copying data files"
     mkdir $DDIR/data
-    cp -p $SDIR/data/*.lua $DDIR/data/
+    cplffiles data data lua
+    cplffiles data data xml
     for folder in `(cd $SDIR && ls -F data)| grep / | grep gfx`; do 
 	mkdir $DDIR/data/$folder; 
 	cp -p $SDIR/data/$folder/*.{png,jpg} $DDIR/data/$folder
     done
+    mkdir $DDIR/data/schemas
+    cplffiles data/schemas data/schemas xsd
+    cplffiles data/schemas data/schemas xml
     copy_levels
     mkdir $DDIR/data/fonts
     cp -p $SDIR/data/fonts/*.{bmf,png,txt,ttf} $DDIR/data/fonts
@@ -52,7 +94,7 @@ function copy_data
     
     echo "... copying locales:"
     mkdir $DDIR/data/locale
-    for lang in de fr nl it es; do
+    for lang in de fr nl it es sv ru hu pt; do
     	echo "$lang.gmo"
 	mkdir "$DDIR/data/locale/$lang"
 	mkdir "$DDIR/data/locale/$lang/LC_MESSAGES"
@@ -66,20 +108,24 @@ function copy_data
 function copy_doc
 {
     echo "... copying documentation"
-    mkdir $DDIR/doc
-    cptext README 		README.txt
-    cptext NEWS 		NEWS.txt
-    cptext COPYING 		doc/COPYING.txt
-    cptext CHANGES 		doc/CHANGES.txt
-    cptext etc/README-SDL.txt 	doc/README-SDL.txt
-    cptext doc/ant_lua.txt 	doc/ant_lua.txt
-    cptext doc/sounds.txt	doc/sounds.txt
-    cptext doc/objects.txt	doc/objects.txt
-    cptext doc/README.mingw32   doc/README-mingw32.txt
-    cptext doc/sound.lua        doc/sound.lua
-    cpbuiltfile doc/refman/enigma-ref.html doc/
-    cpfile doc/functions.html	doc/
-    cpfile doc/functions.css	doc/
+    mkdir $DDIR/images
+    cptext README                       README
+    cptext COPYING                      COPYING
+    cptext CHANGES                      CHANGES
+    cptext ACKNOWLEDGEMENTS 		ACKNOWLEDGEMENTS
+    cptext doc/index.html 		index.html
+    cptext doc/gpl.txt   		gpl.txt
+    cptext doc/lgpl.txt 		lgpl.txt
+    cptext doc/images/enigma2.css	images/enigma2.css
+    cpfile doc/images/navbar.png	images/
+    cpfile doc/images/favicon.png	images/
+    cpfile doc/images/menu_bg.jpg	images/
+    cptext etc/README-SDL.txt 	        README-SDL.txt
+    cptext doc/reference/ant_lua.txt 	reference/ant_lua.txt
+    cptext doc/reference/sounds.txt	reference/sounds.txt
+    cptext doc/reference/sound.lua      reference/sound.lua
+    cpfile doc/reference/lua2xml	reference/
+    cpfile doc/reference/xml2lua	reference/
 }
 
 #
@@ -93,13 +139,24 @@ function copy_manual
     cpbuiltfile doc/manual/*.html manual
 }
 
+#
+# Copy reference manual
+#
+function copy_refman
+{
+    echo "... copying reference manual"
+    mkdir $DDIR/reference $DDIR/reference/images
+    cpfile doc/reference/images/*.png reference/images
+    cpbuiltfile doc/reference/*.html reference
+}
+
 #
 # Copy windows DLLs
 #
 function copy_dlls
 {
-    echo "... copying DLLs"
-    for f in SDL SDL_image SDL_mixer SDL_ttf jpeg libpng libpng1 zlib zlib-1; do
+    echo "... copying DLLs: $DLLS"
+    for f in $DLLS; do
         ff="$f.dll"
         if [ -f $ff ]; then
             cp $ff $DDIR/
@@ -118,12 +175,13 @@ echo "----------------------------------------------------------------"
 echo "                 Building Enigma-$VERSION.zip"
 echo "----------------------------------------------------------------"
 
+
 ###############
 # Find programs
 ###############
 
 UPX=`which upx`
-STRIP=`which i586-mingw32msvc-strip`
+STRIPFULLPATH=`which $STRIP`
 NLCVT="perl $SDIR/etc/nlcvt.pl"
 
 if [ ! ${UPX} ]; then
@@ -131,7 +189,7 @@ if [ ! ${UPX} ]; then
   UPX="echo"
 fi
 
-if [ ! ${STRIP} ]; then
+if [ ! ${STRIPFULLPATH} ]; then
   echo "*** Program not found: strip"
   STRIP="echo"
 fi
@@ -144,6 +202,7 @@ mkdir $DDIR
 
 copy_data
 copy_manual
+copy_refman
 copy_doc
 
 cp $BDIR/etc/enigma.nsi $DDIR/enigma.nsi
