@@ -13,24 +13,24 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: actors.cc,v 1.59 2004/03/21 17:31:47 dheck Exp $
  */
+
+#include "errors.hh"
 #include "enigma.hh"
 #include "player.hh"
 #include "sound.hh"
 #include "server.hh"
 #include "world.hh"
 
-#include <cassert>
 #include <iostream>
 #include <set>
 
 using namespace std;
 using namespace enigma;
 using namespace world;
-using px::V2;
+using ecl::V2;
 
 #include "actors_internal.hh"
 
@@ -53,7 +53,7 @@ Actor::Actor (const ActorTraits &tr)
   m_sprite(),
   startingpos(), 
   respawnpos(), use_respawnpos(false),
-  spikes(false)
+  spikes(false), controllers (0)
 {
     set_attrib("mouseforce", 0.0);
 
@@ -61,7 +61,7 @@ Actor::Actor (const ActorTraits &tr)
     m_actorinfo.mass = tr.default_mass;
     m_actorinfo.radius = tr.radius;
 
-    assert(m_actorinfo.radius <= get_max_radius());
+    ASSERT(m_actorinfo.radius <= get_max_radius(), XLevelRuntime, "Actor: radius of actor too large");
 }
 
 void Actor::on_collision (Actor *) 
@@ -76,7 +76,7 @@ const ActorInfo &Actor::get_actorinfo() const {
     return m_actorinfo; 
 }
 
-const px::V2 &Actor::get_pos() const
+const ecl::V2 &Actor::get_pos() const
 {
     return m_actorinfo.pos;
 }
@@ -91,7 +91,7 @@ void Actor::think(double /*dtime*/) {
     if (f) {
         Floor *fl = f->floor;
         Item *it = f->item;
-        bool item_covers_floor = (it && it->covers_floor());
+        bool item_covers_floor = (it && it->covers_floor(m_actorinfo.pos));
         if (!item_covers_floor && fl && this->is_on_floor())
             fl->actor_contact(this);
     }
@@ -127,7 +127,7 @@ void Actor::respawn() {
     on_respawn(p);
 }
 
-void Actor::add_force (const px::V2 &f) {
+void Actor::add_force (const ecl::V2 &f) {
     m_actorinfo.forceacc += f;
 }
 
@@ -135,17 +135,17 @@ void Actor::init() {
     m_sprite = display::AddSprite(get_pos());
 }
 
-void Actor::on_creation(const px::V2 &p) 
+void Actor::on_creation(const ecl::V2 &p) 
 {
     set_model(get_kind());
     m_sprite.move (p);
     move ();
 }
 
-void Actor::on_respawn (const px::V2 &/*pos*/) {
+void Actor::on_respawn (const ecl::V2 &/*pos*/) {
 }
 
-void Actor::warp(const px::V2 &newpos) {
+void Actor::warp(const ecl::V2 &newpos) {
     m_actorinfo.pos = newpos;
     m_actorinfo.vel = V2();
     m_sprite.move (newpos);
@@ -217,10 +217,11 @@ bool Actor::can_move() const {
     return true;
 }
 
-void Actor::message(const string &m, const Value &) {
+Value Actor::message(const string &m, const Value &) {
     if (m == "init") {
         startingpos = get_pos();
     }
+    return Value();
 }
 
 bool Actor::sound_event (const char *name) {
@@ -258,16 +259,20 @@ namespace
 	void on_collision (Actor *a) {
 	    SendMessage(a, "shatter");
 	}
+
+        bool attackCurrentOnly;
+        double timeKeepAttackStrategy;
     };
 }
 
 RotorBase::RotorBase(const ActorTraits &tr)
-: Actor(tr)
+: Actor(tr), attackCurrentOnly(false), timeKeepAttackStrategy(0)
 {
     set_attrib ("range", 5.0);
     set_attrib ("force", 10.0);
     set_attrib ("gohome", 1.0);
     set_attrib ("attacknearest", 1.0);
+    set_attrib ("prefercurrent", 0.0);
 }
 
 void RotorBase::think (double dtime)
@@ -280,7 +285,15 @@ void RotorBase::think (double dtime)
 
     Actor *target = 0;
     V2     target_vec;
-    bool attack_nearest = int_attrib ("attacknearest");
+    bool attack_nearest = (int_attrib ("attacknearest") != 0);
+
+    double preferCurrent;
+    double_attrib("prefercurrent", &preferCurrent);
+    timeKeepAttackStrategy  -= dtime;
+    if (timeKeepAttackStrategy < 0) {
+        timeKeepAttackStrategy = enigma::DoubleRand(0.8, 1.6);
+        attackCurrentOnly = (enigma::DoubleRand(0.0, 1.0) < preferCurrent);
+    }
 
     vector<Actor *> actors;
     GetActorsInRange (get_pos(), range, actors);
@@ -291,13 +304,14 @@ void RotorBase::think (double dtime)
             && a->is_movable() && !a->is_invisible())
         {
             V2 v = a->get_pos() - get_pos();
-            if (attack_nearest) {
+            if (attack_nearest && !attackCurrentOnly ||
+                    attackCurrentOnly && a == player::GetMainActor(
+                    player::CurrentPlayer())) {
                 if (!target || (length(v) < length(target_vec))) {
                     target = a;
                     target_vec = v;
                 }
-            }
-            else {
+            } else if (!attackCurrentOnly) {            
                 target = a;
                 target_vec += normalize(v);
             }
@@ -335,8 +349,8 @@ namespace
 ActorTraits Rotor::traits = {
     "ac-rotor",                 // name
     ac_rotor,                   // id
-    22.0/64,                    // radius
-    0.8                         // mass
+    22.0f/64,                   // radius
+    0.8f                        // mass
 };
 
 
@@ -356,8 +370,8 @@ namespace
 ActorTraits Top::traits = {
     "ac-top",                   // name
     ac_top,                     // id
-    16.0/64,                    // radius
-    0.8                         // mass
+    16.0f/64,                   // radius
+    0.8f                        // mass
 };
 
 
@@ -379,8 +393,8 @@ namespace
 ActorTraits Bug::traits = {
     "ac-bug",                   // name
     ac_bug,                     // id
-    12.0/64,                    // radius
-    0.7                         // mass
+    12.0f/64,                   // radius
+    0.7f                        // mass
 };
 
 
@@ -414,8 +428,8 @@ namespace
 ActorTraits Horse::traits = {
     "ac-horse",                 // name
     ac_horse,                   // id
-    24.0/64,                    // radius
-    1.2                         // mass
+    24.0f/64,                   // radius
+    1.2f                        // mass
 };
 
 Horse::Horse() 
@@ -487,7 +501,7 @@ namespace
         bool is_flying() const { return true; }
         bool is_dead() const { return false; }
         bool is_on_floor() const { return false; }
-        void on_creation(const px::V2 &p);
+        void on_creation(const ecl::V2 &p);
         bool can_move() const { return true; }
 
         void animcb();
@@ -525,7 +539,7 @@ void CannonBall::animcb()
     KillActor (this);
 }
 
-void CannonBall::on_creation(const px::V2 &p) 
+void CannonBall::on_creation(const ecl::V2 &p) 
 {
     Actor::on_creation(p);
 
@@ -576,14 +590,15 @@ namespace
         void set_sink_model(const string &m);
         void set_shine_model (bool shinep);
         void update_halo();
+        virtual void hide();
 
         /* ---------- Actor interface ---------- */
         
         virtual void think (double dtime);
         virtual void move_screen ();
 
-        void on_creation(const px::V2 &p);
-        void on_respawn (const px::V2 &/*pos*/)
+        void on_creation(const ecl::V2 &p);
+        void on_respawn (const ecl::V2 &/*pos*/)
         {
             change_state(APPEARING);
         }
@@ -602,7 +617,7 @@ namespace
         bool can_be_warped() const { return state==NORMAL; }
 
         // Object interface.
-        void message(const string &m, const Value &);
+        virtual Value message(const string &m, const Value &);
 
         // ModelCallback interface.
         void animcb();
@@ -628,7 +643,7 @@ namespace
         double m_invisible_rest_time;
     };
 
-    const double BasicBall::SHIELD_TIME = 5.0;
+    const double BasicBall::SHIELD_TIME = 10.0;
 }
 
 BasicBall::BasicBall(const ActorTraits &tr)
@@ -647,7 +662,7 @@ BasicBall::BasicBall(const ActorTraits &tr)
 {
 }
 
-void BasicBall::on_creation(const px::V2 &p) 
+void BasicBall::on_creation(const ecl::V2 &p) 
 {
     Actor::on_creation(p);
     if (server::CreatingPreview)
@@ -691,7 +706,7 @@ void BasicBall::change_state_noshield (State newstate)
         change_state(newstate);
 }
 
-void BasicBall::message(const string &m, const Value &v) 
+Value BasicBall::message(const string &m, const Value &v) 
 {
     switch (state) {
     case NORMAL:
@@ -744,14 +759,16 @@ void BasicBall::message(const string &m, const Value &v)
             m_drunk_rest_time += 5.0; // Drunken for 5 more seconds
         }
     }
+    return Value();
 }
 
 void BasicBall::set_sink_model(const string &m)
 {
-    int modelnum = px::round_down<int>(sinkDepth);
+    int modelnum = ecl::round_down<int>(sinkDepth);
 
     if (!has_shield() && modelnum != sinkModel) {
-        assert(modelnum >= minSinkDepth && modelnum < maxSinkDepth);
+        ASSERT(modelnum >= minSinkDepth && modelnum < maxSinkDepth, XLevelRuntime,
+            "BasicBall: set_sink_model called though modelnum incorrect");
 
         string img = m+"-sink";
         img.append(1, static_cast<char>('0'+modelnum));
@@ -787,10 +804,10 @@ void BasicBall::update_model()
         }
         else {
             ActorInfo *ai = get_actorinfo();
-            int xpos = px::round_nearest<int> (ai->pos[0] * 32.0);
-            int ypos = px::round_nearest<int> (ai->pos[1] * 32.0);
+            int xpos = ecl::round_nearest<int> (ai->pos[0] * 32.0);
+            int ypos = ecl::round_nearest<int> (ai->pos[1] * 32.0);
 
-            bool shinep = (xpos + ypos) % 2;
+            bool shinep = ((xpos + ypos) % 2) != 0;
             set_shine_model (shinep);
         }
         break;
@@ -804,7 +821,9 @@ void BasicBall::sink (double dtime)
     double sink_speed  = 0.0;
     double raise_speed = 0.0;   // at this velocity don't sink; above: raise
 
-    if (Floor *fl = GetFloor (get_gridpos()))
+    Item *it = GetItem(get_gridpos());
+    Floor *fl = GetFloor (get_gridpos());
+    if (!(it != NULL && it->covers_floor(get_pos())) && fl != NULL)
         fl->get_sink_speed (sink_speed, raise_speed);
     
     if (sink_speed == 0.0 || has_shield()) {
@@ -969,7 +988,8 @@ void BasicBall::change_state(State newstate) {
         world::GrabActor(this);
         break;
     case JUMP_VORTEX:
-        assert(oldstate == RISING_VORTEX);
+        ASSERT(oldstate == RISING_VORTEX, XLevelRuntime,
+            "BasicBall: change to state JUMP_VORTEX but not RISING_VORTEX");
         vortex_normal_time = 0;
         set_model(kind);
         world::ReleaseActor(this);
@@ -1002,6 +1022,16 @@ bool BasicBall::has_shield() const {
 void BasicBall::update_halo() {
     HaloState newstate = m_halostate;
 
+    double radius = get_actorinfo()->radius;
+    string halokind;
+
+    // Determine which halomodel has to be used:
+    if (radius == 19.0/64) { // Halo for normal balls
+        halokind="halo";
+    }else if (radius == 13.0f/64) { // Halo for small balls
+        halokind="halo-small";
+    }
+
     if (m_shield_rest_time <= 0)
         newstate = NOHALO;
     else if (m_shield_rest_time <= 3.0)
@@ -1010,8 +1040,9 @@ void BasicBall::update_halo() {
         newstate = HALONORMAL;
 
     if (newstate != m_halostate) {
-	if (m_halostate == NOHALO)
-	    m_halosprite = display::AddSprite (get_pos(), "halo");
+	if (m_halostate == NOHALO){
+	   m_halosprite = display::AddSprite (get_pos(), halokind.c_str());
+    }
         switch (newstate) {
         case NOHALO:
             // remove halo
@@ -1020,10 +1051,10 @@ void BasicBall::update_halo() {
             break;
         case HALOBLINK:
             // blink for the last 3 seconds
-            m_halosprite.replace_model (display::MakeModel ("halo-blink"));
+            m_halosprite.replace_model (display::MakeModel (halokind+"-blink"));
             break;
         case HALONORMAL:
-	    m_halosprite.replace_model (display::MakeModel ("halo"));
+            m_halosprite.replace_model (display::MakeModel (halokind));
             break;
         }
         m_halostate = newstate;
@@ -1033,6 +1064,11 @@ void BasicBall::update_halo() {
     }
 }
 
+void BasicBall::hide() {
+    Actor::hide();
+    disable_shield();
+}
+
 //----------------------------------------
 // Balls of different sorts
 //----------------------------------------
@@ -1118,15 +1154,15 @@ ActorTraits WhiteBall::traits = {
 ActorTraits WhiteBall_Small::traits = {
     "ac-whiteball-small",       // name
     ac_meditation,              // id
-    13.0/64,                    // radius
-    0.7                         // mass
+    13.0f/64,                   // radius
+    0.7f                        // mass
 };
 
 ActorTraits Killerball::traits = {
     "ac-killerball",            // name
     ac_killerball,              // id
-    13.0/64,                    // radius
-    0.7                         // mass
+    13.0f/64,                   // radius
+    0.7f                        // mass
 };
 
 
