@@ -13,24 +13,32 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: client.cc,v 1.6 2004/05/22 13:04:32 dheck Exp $
  */
 
 #include "client.hh"
+#include "game.hh"
 #include "display.hh"
 #include "options.hh"
 #include "server.hh"
-#include "help.hh"
+#include "gui/HelpMenu.hh"
 #include "main.hh"
-#include "menus.hh"
+#include "gui/GameMenu.hh"
 #include "sound.hh"
 #include "player.hh"
 #include "world.hh"
 #include "nls.hh"
+#include "StateManager.hh"
+#include "lev/Index.hh"
+#include "lev/PersistentIndex.hh"
+#include "lev/Proxy.hh"
+#include "lev/RatingManager.hh"
+#include "lev/ScoreManager.hh"
 
-#include "px/sdl.hh"
+#include "ecl_sdl.hh"
+
+#include "enet/enet.h"
 
 #include <cctype>
 #include <cstring>
@@ -39,14 +47,10 @@
 #include <iostream>
 
 using namespace enigma::client;
-using namespace px;
+using namespace ecl;
 using namespace std;
 
 #include "client_internal.hh"
-
-using levels::LevelInfo;
-using levels::LevelPack;
-
 
 /* -------------------- Auxiliary functions -------------------- */
 
@@ -62,12 +66,27 @@ namespace
     }
 
     /*! Generate the message that is displayed when the level starts. */
-    string displayedLevelInfo (const LevelInfo &info)
+    string displayedLevelInfo (lev::Proxy *level)
     {
-        string text = (info.name.empty()) ?
-            _("Another nameless level") : string("\"")+info.name+"\"";
-        if (!info.author.empty())
-            text += _(" by ") + info.author;
+        std::string text;
+        std::string tmp;
+
+        tmp  = level->getLocalizedString("title");
+        if (tmp.empty())
+            tmp = _("Another nameless level");
+        text = string("\"")+ tmp +"\"";
+        tmp = level->getAuthor();
+        if (!tmp.empty())
+            text += _(" by ") + tmp;
+        tmp = level->getLocalizedString("subtitle");
+        if (!tmp.empty() && tmp != "subtitle")
+           text += string(" - \"")+ tmp + "\""; 
+        tmp = level->getCredits(false);
+        if (!tmp.empty())
+            text += string(" - Credits: ")+ tmp; 
+        tmp = level->getDedication(false);
+        if (!tmp.empty())
+            text += string(" - Dedication: ")+ tmp; 
         return text;
     }
 }
@@ -83,134 +102,6 @@ namespace
 
 #define CLIENT client_instance
 
-
-/* -------------------- GameMenu -------------------- */
-
-GameMenu::GameMenu (int zoomxpos_, int zoomypos_)
-: resume(new gui::StaticTextButton(_("Resume Level"), this)),
-  restart(new gui::StaticTextButton(_("Restart Level"), this)),
-  options(new gui::StaticTextButton(_("Options"), this)),
-  abort(new gui::StaticTextButton(_("Abort Level"), this)),
-  zoomed(0),
-  zoomxpos(zoomxpos_),
-  zoomypos(zoomypos_)
-{
-    add(resume,     Rect(0,0,150,40));
-    add(restart,    Rect(0,45,150,40));
-#if 1
-    add(options,    Rect(0,90,150,40));
-    add(abort,      Rect(0,135,150,40));
-#else
-    add(abort,    Rect(0,90,150,40));
-#endif
-    center();
-}
-
-GameMenu::~GameMenu() {
-    delete(zoomed);
-}
-
-void GameMenu::draw_background(px::GC &gc) 
-{
-    const video::VMInfo *vminfo = video::GetInfo();
-
-    if (!zoomed) {
-        Rect game_area   = display::GetGameArea();
-        int  part_width  = game_area.w/3;
-        int  part_height = (part_width*vminfo->height)/vminfo->width;
-
-        if (part_height > game_area.h) {
-            part_height = game_area.h/3;
-            part_width  = (part_height*vminfo->width)/vminfo->height;
-            assert(part_width <= game_area.w);
-        }
-
-        // randomly choose ball offset
-        int x, y;
-        for (int trials = 5; trials; --trials) {
-            x = IntegerRand(0, 5);
-            y = IntegerRand(0, 3);
-
-            // try to avoid menu-ball overlap:
-            if (x<2 || x>3 || y<1 || y>2 || (trials == 1)) {
-                int ax = zoomxpos-game_area.x;
-                int ay = zoomypos-game_area.y;
-
-                // upper left corner of part
-                x = ax/32-1-x;
-                y = ay/32-1-y;
-
-                // ensure part is inside game_area
-                x = max(0, min(x, (game_area.w-part_width)/32-1));
-                y = max(0, min(y, (game_area.h-part_height)/32-1));
-
-                // adjust to game fields
-                x = x*32+24;
-                y = y*32+16;
-
-                break;
-            }
-        }
-
-        // Be sure to redraw everything, or actors may appear on top
-        // of the stones (actors are drawn in one pass and only
-        // clipped to the screen boundary).
-        display::RedrawAll(video::GetScreen());
-
-        // get the selected part from screen
-//         SDL_Surface *back = video::GetScreen()->get_surface();
-        Rect     src_area(game_area.x+x, game_area.y+y, part_width, part_height);
-        Surface *src = Grab(video::GetScreen()->get_surface(), src_area);
-
-        // zoom multiple times for softer image
-//         const double stepsize = 0.3;
-//         for (double zoom = 0.4; zoom < 0.9; zoom += stepsize) {
-//             int      sx  = round_down<int>(zoom * vminfo->width);
-//             int      sy  = round_down<int>(zoom * vminfo->height);
-//             Surface *tmp = src->zoom(sx, sy);
-
-//             delete src;
-//             src = tmp;
-//         }
-        zoomed = src->zoom(vminfo->width, vminfo->height);
-        delete src;
-    }
-
-    px::blit(gc, 0,0, zoomed);
-}
-
-bool GameMenu::on_event (const SDL_Event &e) 
-{
-    if (e.type == SDL_MOUSEBUTTONDOWN
-        && e.button.button == SDL_BUTTON_RIGHT)
-    {
-        Menu::quit();
-        return true;
-    }
-    return false;
-}
-
-void GameMenu::on_action(gui::Widget *w) {
-    if (w == resume) {
-        Menu::quit();
-    }
-    else if (w == abort) {
-        client::Msg_Command("abort");
-        Menu::quit();
-    }
-    else if (w == restart) {
-        client::Stop ();
-        server::Msg_LoadLevel (server::CurrentLevel);
-        Menu::quit();
-    }
-    else if (w == options) {
-        enigma::ShowOptionsMenu (0);
-        Menu::quit();
-    }
-}
-
-
-
 
 /* -------------------- Client class -------------------- */
 
@@ -221,12 +112,79 @@ Client::Client()
   m_cheater(false), 
   m_user_input()
 {
+    m_network_host = 0;
 }
 
-Client::~Client() {
-
+Client::~Client() 
+{
+    network_stop();
 }
 
+bool Client::network_start()
+{
+    if (m_network_host)
+        return true;
+
+    m_network_host = enet_host_create (NULL,
+                                       1 /* only allow 1 outgoing connection */,
+                                       57600 / 8 /* 56K modem with 56 Kbps downstream bandwidth */,
+                                       14400 / 8 /* 56K modem with 14 Kbps upstream bandwidth */);
+
+    if (m_network_host == NULL) {
+        fprintf (stderr, 
+                 "An error occurred while trying to create an ENet client host.\n");
+        return false;
+    }
+
+
+    // ----- Connect to server
+
+    ENetAddress sv_address;
+    ENetPeer *m_server;
+
+    /* Connect to some.server.net:1234. */
+    enet_address_set_host (&sv_address, "localhost");
+    sv_address.port = 12345;
+
+    /* Initiate the connection, allocating the two channels 0 and 1. */
+    m_server = enet_host_connect (m_network_host, &sv_address, 2);    
+    
+    if (m_server == NULL) {
+       fprintf (stderr, 
+                "No available peers for initiating an ENet connection.\n");
+       return false;
+    }
+    
+    // Wait up to 5 seconds for the connection attempt to succeed.
+    ENetEvent event;
+    if (enet_host_service (m_network_host, &event, 5000) > 0 &&
+        event.type == ENET_EVENT_TYPE_CONNECT)
+    {
+        fprintf (stderr, "Connection to some.server.net:1234 succeeded.");
+        return true;
+    }
+    else
+    {
+        /* Either the 5 seconds are up or a disconnect event was */
+        /* received. Reset the peer in the event the 5 seconds   */
+        /* had run out without any significant event.            */
+        enet_peer_reset (m_server);
+        m_server = 0;
+
+        fprintf (stderr, "Connection to localhost:12345 failed.");
+        return false;
+    }
+}
+
+void Client::network_stop ()
+{
+    if (m_network_host)
+        enet_host_destroy (m_network_host);
+    if (m_server)
+        enet_peer_reset (m_server);
+}
+
+
 /* ---------- Event handling ---------- */
 
 void Client::handle_events() 
@@ -284,11 +242,11 @@ void Client::on_mousebutton(SDL_Event &e)
         else if (e.button.button == 3|| e.button.button == 4) {
             // right mousebutton, wheel down -> rotate inventory
             rotate_inventory(+1);
-	}
-	else if (e.button.button == 5) {
-	    // wheel down -> inverse rotate inventory
+        }
+        else if (e.button.button == 5) {
+            // wheel down -> inverse rotate inventory
             rotate_inventory(-1);
-	}
+        }
     }
     update_mouse_button_state();
 }
@@ -310,7 +268,7 @@ public:
 
     HistoryProxy();
     ~HistoryProxy() {
-        if (!--instances) options::SetOption("History", content);
+        if (!--instances) app.state->setProperty("CommandHistory", content);
     }
 };
 
@@ -319,7 +277,7 @@ int    HistoryProxy::instances = 0;
 
 HistoryProxy::HistoryProxy() {
     if (!instances++) {
-        content = options::GetString("History");
+        content = app.state->getString("CommandHistory");
         if (content.find(HSEP) == string::npos) content = string(1, HSEP);
     }
 }
@@ -419,6 +377,14 @@ void Client::on_keydown(SDL_Event &e)
         case SDLK_a:
             server::Msg_Command ("restart");
             break;
+        case SDLK_F3: 
+            if (keymod & KMOD_SHIFT) {
+                // force a reload from file
+                lev::Proxy * curProxy = lev::Proxy::loadedLevel();
+                if (curProxy != NULL)
+                    curProxy->release();
+                server::Msg_Command ("restart");
+            }
         default:
             break;
         };
@@ -429,7 +395,7 @@ void Client::on_keydown(SDL_Event &e)
         case SDLK_t:
             if (enigma::WizardMode) {
                 Screen *scr = video::GetScreen();
-                px::TintRect(scr->get_surface (), display::GetGameArea(), 
+                ecl::TintRect(scr->get_surface (), display::GetGameArea(), 
                              100, 100, 100, 0);
                 scr->update_all();
             }
@@ -467,14 +433,21 @@ void Client::on_keydown(SDL_Event &e)
                 server::Msg_Command ("suicide"); 
             break;
 
-        case SDLK_F4: Msg_AdvanceLevel(levels::advance_strictly_next); break;
-        case SDLK_F5: Msg_AdvanceLevel(levels::advance_unsolved); break;
+        case SDLK_F4: Msg_AdvanceLevel(lev::ADVANCE_STRICTLY); break;
+        case SDLK_F5: Msg_AdvanceLevel(lev::ADVANCE_UNSOLVED); break;
 
         case SDLK_F10: {
-            string fname = string("screenshots/");
-            fname += server::CurrentLevelPack->get_info(server::CurrentLevel).filename;
-            fname += ".png";
-            video::Screenshot(fname);
+            lev::Proxy *level = lev::Proxy::loadedLevel();
+            std::string basename = std::string("screenshots/") +
+                    level->getLocalSubstitutionLevelPath();
+            std::string fname = basename + ".png";
+            std::string fullPath;
+            int i = 1;
+            while (app.resourceFS->findFile(fname, fullPath)) {
+                fname = basename + ecl::strf("#%d", i++) + ".png";
+            }
+            std::string savePath = app.userImagePath + "/" + fname;
+            video::Screenshot(savePath);
             break;
         }
         case SDLK_RETURN: process_userinput(); break;
@@ -517,12 +490,12 @@ void Client::show_help()
     server::Msg_Pause (true);
     video::TempInputGrab grab(false);
 
-    helptext_ingame[15] = options::GetBool("TimeHunting")
-        ? _("Skip to next non-par level")
+    helptext_ingame[15] = app.state->getInt("NextLevelMode") == lev::NEXT_LEVEL_NOT_BEST
+        ? _("Skip to next level for best score hunt")
         : _("Skip to next unsolved level");
 
     video::ShowMouse();
-    displayHelp(helptext_ingame, 200);
+    gui::displayHelp(helptext_ingame, 200);
     video::HideMouse();
 
     update_mouse_button_state();
@@ -532,7 +505,7 @@ void Client::show_help()
     server::Msg_Pause (false);
     game::ResetGameTimer();
 
-    if (options::GetBool("TimeHunting")) 
+    if (app.state->getInt("NextLevelMode") == lev::NEXT_LEVEL_NOT_BEST) 
         server::Msg_Command ("restart"); // inhibit cheating
 
 }
@@ -542,7 +515,7 @@ void Client::show_menu()
 {
     server::Msg_Pause (true);
 
-    px::Screen *screen = video::GetScreen();
+    ecl::Screen *screen = video::GetScreen();
 
     video::TempInputGrab grab (false);
 
@@ -550,7 +523,7 @@ void Client::show_menu()
     {
         int x, y;
         display::GetReferencePointCoordinates(&x, &y);
-        GameMenu(x, y).manage();
+        enigma::gui::GameMenu(x, y).manage();
     }
     video::HideMouse();
     update_mouse_button_state();
@@ -560,7 +533,7 @@ void Client::show_menu()
     server::Msg_Pause (false);
     game::ResetGameTimer();
 
-    if (options::GetBool("TimeHunting")) 
+    if (app.state->getInt("NextLevelMode") == lev::NEXT_LEVEL_NOT_BEST) 
         server::Msg_Command ("restart"); // inhibit cheating
 
 }
@@ -576,13 +549,24 @@ void Client::draw_screen()
 
         vector<string> lines;
 
-        px::split_copy (m_error_message, '\n', back_inserter(lines));
+        ecl::split_copy (m_error_message, '\n', back_inserter(lines));
         int x     = 60;
         int y     = 60;
         int yskip = 25;
-        for (unsigned i=0; i<lines.size(); ++i) {
-            f->render(gc, x,  y, lines[i].c_str());
+        const video::VMInfo *vminfo = video::GetInfo();
+        int width = vminfo->width - 120;
+        for (unsigned i=0; i<lines.size(); ) {
+            std::string::size_type breakPos = ecl::breakString (f, lines[i], 
+                                                                " ", width);
+            f->render(gc, x,  y, lines[i].substr(0,breakPos).c_str());
             y += yskip;
+            if (breakPos != lines[i].size()) {
+                // process rest of line
+                lines[i] = lines[i].substr(breakPos);
+            } else {
+                // process next line
+                i++;
+            }
         }
         scr->update_all();
         scr->flush_updates();
@@ -594,26 +578,32 @@ void Client::draw_screen()
 }
 
 
-void Client::init_hunted_time(int ilevel, string& hunted) 
+std::string Client::init_hunted_time() 
 {
+    std::string hunted;
     m_hunt_against_time = 0;
-    if (options::GetBool("TimeHunting")) {
-        Level level(server::CurrentLevelPack, ilevel);
-        int   difficulty     = options::GetDifficulty();
-        int   par_time       = level.get_par_time(difficulty);
-        int   best_user_time = level.get_best_user_time(difficulty);
+    if (app.state->getInt("NextLevelMode") == lev::NEXT_LEVEL_NOT_BEST) {
+        lev::Index *ind = lev::Index::getCurrentIndex();
+        lev::ScoreManager *scm = lev::ScoreManager::instance();
+        lev::Proxy *curProxy = ind->getCurrent();
+        lev::RatingManager *ratingMgr = lev::RatingManager::instance();
+        
+        int   difficulty     = app.state->getInt("Difficulty");
+        int   wr_time       = ratingMgr->getBestScore(curProxy, difficulty);
+        int   best_user_time = scm->getBestUserScore(curProxy, difficulty);
 
-        if (best_user_time>0 && (par_time == -1 || best_user_time<par_time)) {
+        if (best_user_time>0 && (wr_time == -1 || best_user_time<wr_time)) {
             m_hunt_against_time = best_user_time;
             hunted              = "you";
         }
-        else if (par_time>0) {
-            m_hunt_against_time = par_time;
-            hunted              = level.get_par_holder(difficulty);
+        else if (wr_time>0) {
+            m_hunt_against_time = wr_time;
+            hunted              = ratingMgr->getBestScoreHolder(curProxy, difficulty);
         }
 
         // STATUSBAR->set_timerstart(-m_hunt_against_time);
     }
+    return hunted;
 }
 
 void Client::tick (double dtime) 
@@ -642,20 +632,25 @@ void Client::tick (double dtime)
     }
 
     case cls_game:
-        if (options::GetBool("TimeHunting")) {
+        if (app.state->getInt("NextLevelMode") == lev::NEXT_LEVEL_NOT_BEST) {
             int old_second = round_nearest<int> (m_total_game_time);
             int second     = round_nearest<int> (m_total_game_time + dtime);
 
             if (m_hunt_against_time && old_second <= m_hunt_against_time) {
                 if (second > m_hunt_against_time) { // happens exactly once when par has passed by
-                    Level  level(server::CurrentLevelPack, server::CurrentLevel);
-                    int    difficulty     = options::GetDifficulty();
-                    int    par_time       = level.get_par_time (difficulty);
-                    int    best_user_time = level.get_best_user_time (difficulty);
+                    lev::Index *ind = lev::Index::getCurrentIndex();
+                    lev::ScoreManager *scm = lev::ScoreManager::instance();
+                    lev::Proxy *curProxy = ind->getCurrent();
+                    lev::RatingManager *ratingMgr = lev::RatingManager::instance();
+                    int    difficulty     = app.state->getInt("Difficulty");
+                    int    wr_time       = ratingMgr->getBestScore(curProxy, difficulty);
+                    int    best_user_time = scm->getBestUserScore(curProxy, difficulty);
                     string message;
 
-                    if (par_time>0 && (best_user_time<0 || best_user_time>par_time)) {
-                        message = string(_("Too slow for "))+level.get_par_holder(difficulty)+".. [Ctrl-A]";
+                    if (wr_time>0 && (best_user_time<0 || best_user_time>wr_time)) {
+                        message = string(_("Too slow for ")) + 
+                            ratingMgr->getBestScoreHolder(curProxy, difficulty) +
+                            ".. [Ctrl-A]";
                     }
                     else {
                         message = string(_("You are slow today.. [Ctrl-A]"));
@@ -712,72 +707,86 @@ void Client::tick (double dtime)
 
 void Client::level_finished() 
 {
-    Level level (server::CurrentLevelPack, server::CurrentLevel);
-
-    int difficulty = options::GetDifficulty();
+    lev::Index *ind = lev::Index::getCurrentIndex();
+    lev::ScoreManager *scm = lev::ScoreManager::instance();
+    lev::Proxy *curProxy = ind->getCurrent();
+    lev::RatingManager *ratingMgr = lev::RatingManager::instance();
+    int    difficulty     = app.state->getInt("Difficulty");
+    int    wr_time       = ratingMgr->getBestScore(curProxy, difficulty);
+    int    best_user_time = scm->getBestUserScore(curProxy, difficulty);
+    string par_name       = ratingMgr->getBestScoreHolder(curProxy, difficulty);
+    int par_time   = ratingMgr->getParScore(curProxy, difficulty);
 
     int    level_time     = round_nearest<int> (m_total_game_time);
-    int    par_time       = level.get_par_time (difficulty);
-    string par_name       = level.get_par_holder (difficulty);
-    int    best_user_time = level.get_best_user_time (difficulty);
 
     string      text;
     bool        timehunt_restart = false;
 
-    if (par_time > 0) {
-        if (best_user_time<0 || best_user_time>par_time) {
-            if (level_time == par_time)
-                text = string(_("Exactly the par time by "))+par_name+"!";
-            else if (level_time<par_time)
-                text = _("Great! A new par time!");
+    if (wr_time > 0) {
+        if (best_user_time<0 || best_user_time>wr_time) {
+            if (level_time == wr_time)
+                text = string(_("Exactly the world record of "))+par_name+"!";
+            else if (level_time<wr_time)
+                text = _("Great! A new world record!");
         }
     }
     if (text.length() == 0 && best_user_time>0) {
         if (level_time == best_user_time) {
-            text = _("Again your personal best time...");
-            timehunt_restart = true; // when hunting yourself: Equal is too slow
+            text = _("Again your personal record...");
+            if (app.state->getInt("NextLevelMode") == lev::NEXT_LEVEL_NOT_BEST)
+                timehunt_restart = true; // when hunting yourself: Equal is too slow
         }
         else if (level_time<best_user_time)
-            text = _("New personal best time!");
+            if (par_time >= 0 && level_time <= par_time)
+                text = _("New personal record - better than par!");
+            else if (par_time >= 0)
+                text = _("New personal record, but over par!");
+            else
+                text = _("New personal record!");
     }
 
-    if (options::GetBool("TimeHunting") &&
-        (par_time>0 || best_user_time>0))
+    if (app.state->getInt("NextLevelMode") == lev::NEXT_LEVEL_NOT_BEST &&
+        (wr_time>0 || best_user_time>0))
     {
-        bool with_par = best_user_time == -1 || (par_time >0 && par_time<best_user_time);
-        int  behind   = level_time - (with_par ? par_time : best_user_time);
+        bool with_par = best_user_time == -1 || (wr_time >0 && wr_time<best_user_time);
+        int  behind   = level_time - (with_par ? wr_time : best_user_time);
 
         if (behind>0) {
             if (best_user_time>0 && level_time<best_user_time && with_par) {
-                text = _("Your best, ");
+                text = _("Your record, ");
             }
             else {
                 text = "";
             }
             text += strf("%d:%02d ", static_cast<int> (behind/60)%100, behind%60);
             if (with_par) 
-                text += _("behind par time.");
+                text += _("behind world record.");
             else
-                text += _("behind best time.");
+                text += _("behind your record.");
 
             timehunt_restart = true; // time hunt failed -> repeat level
         }
     }
 
-    if (text.length() == 0)
-        text = _("Level finished!");
+    if (text.length() == 0) {
+        if (par_time >= 0 && level_time <= par_time)
+            text = _("Level finished - better than par!");
+        else if (par_time >= 0)
+            text = _("Level finished, but over par!");
+        else
+            text = _("Level finished!");
+    }
     if (m_cheater)
         text += _(" Cheater!");
 
     Msg_ShowText (text, false);
 
     if (!m_cheater) {
-        level.set_level_time (options::GetDifficulty(), level_time);
+        scm->updateUserScore(curProxy, difficulty, level_time);
+
+        // save score (just in case Enigma crashes when loading next level)
+        lev::ScoreManager::instance()->save();
 
-        if (options::LevelStatusChanged) {
-            // save options (just in case Enigma crashes when loading next level)
-            options::Save();
-        }
     }
 
     if (timehunt_restart)
@@ -786,36 +795,40 @@ void Client::level_finished()
         m_state = cls_finished;
 }
 
-void Client::level_loaded (unsigned ilevel)
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
+
+void Client::level_loaded(bool isRestart)
 {
-    LevelPack *lp = server::CurrentLevelPack;
+    lev::Index *ind = lev::Index::getCurrentIndex();
+    lev::ScoreManager *scm = lev::ScoreManager::instance();
+    lev::Proxy *curProxy = ind->getCurrent();
 
     // update window title
-    const LevelInfo &info = lp->get_info(ilevel);
+    video::SetCaption(ecl::strf(_("Enigma pack %s - level #%d: %s"), ind->getName().c_str(),
+            ind->getCurrentLevel(), curProxy->getTitle().c_str()).c_str());
 
-    char       buffer[100];
-    snprintf (buffer, sizeof(buffer), _("Enigma level #%d: %s"), 
-              ilevel+1, info.name.c_str());
-    video::SetCaption(buffer);
-
-    string hunted;
-    init_hunted_time(ilevel, hunted);   // sets m_hunt_against_time (used below)
+    string hunted = init_hunted_time();   // sets m_hunt_against_time (used below)
 
     // show level information (name, author, etc.)
     {
         string displayed_info = "";
         if (m_hunt_against_time>0) {
             if (hunted == "you")
-                displayed_info = _("Your best time: ");
+                displayed_info = _("Your record: ");
             else
-                displayed_info = _("Par to beat: ");
-            displayed_info += strf("%d:%02d", (m_hunt_against_time/60)%100, m_hunt_against_time%60);
+                displayed_info = _("World record to beat: ");
+            displayed_info += ecl::strf("%d:%02d", (m_hunt_against_time/60)%100, 
+                    m_hunt_against_time%60);
 //+ _(" by ") +hunted;
 // makes the string too long in many levels
+            Msg_ShowText (displayed_info, false, 4.0);
+        }
+        else {
+            displayed_info = displayedLevelInfo(curProxy);
+            Msg_ShowText (displayed_info, true, 2.0);
         }
-        else
-            displayed_info = displayedLevelInfo(info);
-        Msg_ShowText (displayed_info, false, 4.0);
     }
 
     sound::FadeoutMusic();
@@ -826,11 +839,11 @@ void Client::level_loaded (unsigned ilevel)
     }
 
     // start screen transition
-
     GC gc(video::BackBuffer());
     display::DrawAll(gc);
 
-    m_effect.reset (video::MakeEffect (video::TM_PUSH_RANDOM, video::BackBuffer()));
+    m_effect.reset (video::MakeEffect ((isRestart ? video::TM_SQUARES :
+            video::TM_PUSH_RANDOM), video::BackBuffer()));
     m_cheater = false;
     m_state   = cls_preparing_game;
 }
@@ -838,7 +851,7 @@ void Client::level_loaded (unsigned ilevel)
 
 void Client::handle_message (Message *m) { // @@@ unused
     switch (m->type) {
-    case cl_level_loaded:
+    case CLMSG_LEVEL_LOADED:
 
         break;
     default:
@@ -857,8 +870,15 @@ void Client::error (const string &text)
 
 /* -------------------- Functions -------------------- */
 
-void client::Msg_LevelLoaded (unsigned levelidx) {
-    CLIENT.level_loaded(levelidx);
+
+bool client::NetworkStart() 
+{
+    return CLIENT.network_start();
+}
+
+void client::Msg_LevelLoaded(bool isRestart)
+{
+    CLIENT.level_loaded(isRestart);
 }
 
 void client::Tick (double dtime) {
@@ -870,11 +890,16 @@ void client::Stop() {
     CLIENT.stop ();
 }
 
-void client::Msg_AdvanceLevel (levels::LevelAdvanceMode mode) {
-    Level level (server::CurrentLevelPack, server::CurrentLevel);
+void client::Msg_AdvanceLevel (lev::LevelAdvanceMode mode) {
 
-    if (advance_level (level, mode))
-        server::Msg_LoadLevel (level.get_index());
+    lev::Index *ind = lev::Index::getCurrentIndex();
+    // log last played level
+    lev::PersistentIndex::addCurrentToHistory();
+
+    if (ind->advanceLevel(mode)) {
+        // now we may advance
+        server::Msg_LoadLevel(ind->getCurrent(), false);
+    }
     else
         client::Msg_Command("abort");
 }
@@ -911,7 +936,7 @@ void client::Msg_PlayerPosition (unsigned iplayer, const V2 &pos)
 }
 
 void client::Msg_PlaySound (const std::string &wavfile, 
-                            const px::V2 &pos,
+                            const ecl::V2 &pos,
                             double relative_volume)
 {
     sound::SoundEvent (wavfile.c_str(), pos, relative_volume);
@@ -922,7 +947,7 @@ void client::Msg_PlaySound (const std::string &wavfile, double relative_volume)
     sound::SoundEvent (wavfile.c_str(), V2(), relative_volume);
 }
 
-void client::Msg_Sparkle (const px::V2 &pos) {
+void client::Msg_Sparkle (const ecl::V2 &pos) {
     display::AddEffect (pos, "ring-anim");
 }
 
