@@ -13,9 +13,8 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: display.cc,v 1.45 2004/05/22 13:04:32 dheck Exp $
  */
 
 /*
@@ -28,8 +27,8 @@
 #include "video.hh"
 #include "main.hh"
 
-#include "px/sdl.hh"
-#include "px/px.hh"
+#include "ecl_sdl.hh"
+#include "ecl.hh"
 
 #include <algorithm>
 #include <functional>
@@ -37,7 +36,7 @@
 #include <iostream>
 
 using namespace std;
-using namespace px;
+using namespace ecl;
 using namespace display;
 using namespace enigma;
 
@@ -104,7 +103,7 @@ StatusBarImpl::StatusBarImpl (const ScreenArea &area)
 }
 
 StatusBarImpl::~StatusBarImpl() {
-    px::delete_sequence(m_models.begin(), m_models.end());
+    ecl::delete_sequence(m_models.begin(), m_models.end());
     m_models.clear();
 }
 
@@ -155,8 +154,12 @@ void StatusBarImpl::show_odometer (bool active)
     }
 }
 
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
 
-void StatusBarImpl::redraw (px::GC &gc, const ScreenArea &r) {
+
+void StatusBarImpl::redraw (ecl::GC &gc, const ScreenArea &r) {
     const video::VMInfo *vminfo = video::GetInfo();
     ScreenArea a = get_area();
     clip(gc, intersect(a, r));
@@ -259,7 +262,7 @@ void StatusBarImpl::set_inventory (const std::vector<std::string> &modelnames)
         hide_text();
     }
 
-    px::delete_sequence(m_models.begin(), m_models.end());
+    ecl::delete_sequence(m_models.begin(), m_models.end());
     m_models.clear();
 
     for (size_t i=0; i<modelnames.size(); ++i) {
@@ -304,6 +307,7 @@ TextDisplay::TextDisplay (Font &f)
   text(), 
   changedp(false), finishedp(true),
   pingpong (false),
+  showscroll(false),
   xoff(0), scrollspeed(200),
   textsurface(0), font(f)
 {
@@ -321,16 +325,30 @@ void TextDisplay::set_text (const string &t, bool scrolling, double duration)
 
     time = 0;
 
+    if (scrolling) {
+        if (duration <= 0) {
+            xoff = -area.w;
+            scrollspeed = 160;
+        } else {
+            // Showscroll mode: first show string then scoll it out
+            showscroll = true;
+            scrollspeed = 0;
+            if (area.w < textsurface->width()) {
+                // start left adjusted for long strings
+                xoff = 0;
+            } else {
+                // start centered for short strings
+                xoff = -(area.w - textsurface->width())/2;
+            }
+        }
+    }
+
     if (duration > 0)
         maxtime = duration;
     else
         maxtime = 1e20;       // "infinite" for all practical purposes
 
-    if (scrolling) {
-        xoff = -area.w;
-        scrollspeed = 160;
-    }
-    else {// centered text string
+    if (!scrolling) {// centered text string
         if (area.w < textsurface->width()) {
             pingpong = true;
             scrollspeed = 4 * (textsurface->width() - area.w) / duration;
@@ -351,8 +369,14 @@ void TextDisplay::tick (double dtime)
 {
     time += dtime;
     if (time > maxtime) {
-        finishedp = true;
-        changedp = true;
+        if (showscroll) {
+            showscroll = false;
+            scrollspeed = 160;            
+            maxtime = 1e20;       // "infinite" for all practical purposes
+        } else {
+            finishedp = true;
+            changedp = true;
+        }
     }
     else {
         int oldxoff = round_nearest<int>(xoff);
@@ -374,7 +398,7 @@ void TextDisplay::tick (double dtime)
     }
 }
 
-void TextDisplay::draw (px::GC &gc, const ScreenArea &r) {
+void TextDisplay::draw (ecl::GC &gc, const ScreenArea &r) {
     clip(gc, intersect(area, r));
     set_color(gc, 0,0,0);
     box(gc, area);
@@ -401,7 +425,8 @@ DisplayEngine::DisplayEngine (int tilew, int tileh)
     m_screenoffset[0] = m_screenoffset[1] = 0;
 }
 
-DisplayEngine::~DisplayEngine() {
+DisplayEngine::~DisplayEngine() 
+{
     delete_sequence (m_layers.begin(), m_layers.end());
 }
 
@@ -422,7 +447,7 @@ void DisplayEngine::set_offset (const V2 &off) {
     world_to_video (off, &m_screenoffset[0], &m_screenoffset[1]);
 }
 
-void DisplayEngine::move_offset (const px::V2 &off) {
+void DisplayEngine::move_offset (const ecl::V2 &off) {
     m_new_offset = off;
 }
 
@@ -431,7 +456,7 @@ void DisplayEngine::move_offset (const px::V2 &off) {
   modified externally since the last call to update_offset(). */
 void DisplayEngine::update_offset () 
 {
-    px::Screen *screen = video::GetScreen();
+    ecl::Screen *screen = video::GetScreen();
 
     int oldx = m_screenoffset[0];
     int oldy = m_screenoffset[1];
@@ -466,7 +491,7 @@ void DisplayEngine::update_offset ()
     }
 }
 
-void DisplayEngine::set_screen_area (const px::Rect & r) {
+void DisplayEngine::set_screen_area (const ecl::Rect & r) {
     m_area = r;
 }
 
@@ -495,7 +520,7 @@ void DisplayEngine::world_to_screen (const V2 & pos, int *x, int *y)
 }
 
 
-void DisplayEngine::world_to_video (const px::V2 &pos, int *x, int *y)
+void DisplayEngine::world_to_video (const ecl::V2 &pos, int *x, int *y)
 {
     *x = round_nearest<int>(pos[0]*m_tilew);
     *y = round_nearest<int>(pos[1]*m_tileh);
@@ -511,7 +536,7 @@ void DisplayEngine::video_to_screen (int x, int y, int *xx, int *yy)
    tiles that contains a certain rectangle `r' in video space.  This
    function is used for calculating the region that needs to be
    updated when a sprite with extension `r' is moved on the screen. */
-void DisplayEngine::video_to_world (const px::Rect &r, Rect &s) {
+void DisplayEngine::video_to_world (const ecl::Rect &r, Rect &s) {
     dRect dr (r.x, r.y, r.w, r.h);
     s = round_grid (dr, get_tilew(), get_tileh());
 }
@@ -560,7 +585,7 @@ void DisplayEngine::mark_redraw_screen() {
     mark_redraw_area(screen_to_world(m_area));
 }
 
-void DisplayEngine::draw_all (px::GC &gc) 
+void DisplayEngine::draw_all (ecl::GC &gc) 
 {
     WorldArea wa = screen_to_world (get_area());
 
@@ -609,7 +634,7 @@ void DisplayEngine::update_layer (DisplayLayer *l, WorldArea wa)
 
 void DisplayEngine::update_screen() 
 {
-    px::Screen *screen = video::GetScreen();
+    ecl::Screen *screen = video::GetScreen();
     GC gc(screen->get_surface());
 
     if (m_new_offset != m_offset) {
@@ -755,7 +780,7 @@ Model *DL_Grid::yield_model (int x, int y) {
 }
 
 
-void DL_Grid::draw (px::GC &gc, const WorldArea &a, int destx, int desty) {
+void DL_Grid::draw (ecl::GC &gc, const WorldArea &a, int destx, int desty) {
     int x2 = a.x+a.w;
     int y2 = a.y+a.h;
     int tilew = get_engine()->get_tilew();
@@ -793,7 +818,7 @@ void SpriteHandle::kill() {
     }
 }
 
-void SpriteHandle::move (const px::V2 &newpos) const {
+void SpriteHandle::move (const ecl::V2 &newpos) const {
     if (layer)
         layer->move_sprite (id, newpos);
 }
@@ -816,10 +841,12 @@ void SpriteHandle::set_callback (ModelCallback *cb) const {
 
 void SpriteHandle::hide() const {
     layer->get_sprite(id)->visible = false;
+    layer->get_sprite(id)->mayNeedRedraw = false;
 }
 
 void SpriteHandle::show() const {
     layer->get_sprite(id)->visible = true;
+    layer->get_sprite(id)->mayNeedRedraw = true;
 }
 
 
@@ -844,14 +871,15 @@ void DL_Sprites::new_world (int w, int h) {
     numsprites = 0;
 }
 
-void DL_Sprites::move_sprite (SpriteId id, const px::V2& newpos) 
+void DL_Sprites::move_sprite (SpriteId id, const ecl::V2& newpos) 
 {
     Sprite *sprite = sprites[id];
 
     int newx, newy;
     get_engine()->world_to_video (newpos, &newx, &newy);
 
-    if (newx != sprite->screenpos[0] || newy != sprite->screenpos[1]) {
+    if (newx != sprite->screenpos[0] || newy != sprite->screenpos[1] ||
+            sprite->mayNeedRedraw ) {
         redraw_sprite_region(id); // make sure old sprite is removed
         sprite->pos = newpos;
         sprite->screenpos[0] = newx;
@@ -916,7 +944,7 @@ void DL_Sprites::kill_sprite (SpriteId id) {
     }
 }
 
-void DL_Sprites::draw (px::GC &gc, const WorldArea &a, int /*x*/, int /*y*/)
+void DL_Sprites::draw (ecl::GC &gc, const WorldArea &a, int /*x*/, int /*y*/)
 {
     DisplayEngine *engine = get_engine();
     clip (gc, intersect (engine->get_area(), engine->world_to_screen(a)));
@@ -940,7 +968,7 @@ void DL_Sprites::draw_sprites (bool drawshadowp, GC &gc) {
     }
 }
 
-void DL_Sprites::draw_onepass (px::GC &gc)
+void DL_Sprites::draw_onepass (ecl::GC &gc)
 {
 //     draw_sprites (false, gc);
 }
@@ -982,7 +1010,7 @@ void DL_Sprites::tick (double dtime)
 // RUBBER BANDS
 //----------------------------------------------------------------------
 
-void DL_Lines::draw_onepass (px::GC &gc)
+void DL_Lines::draw_onepass (ecl::GC &gc)
 {
     DisplayEngine *engine = get_engine();
 
@@ -1199,7 +1227,7 @@ namespace display
         {}
     };
 
-    class StoneShadowCache : public px::Nocopy {
+    class StoneShadowCache : public ecl::Nocopy {
     public:
         StoneShadowCache(int tilew, int tileh);
         ~StoneShadowCache();
@@ -1398,7 +1426,7 @@ void DL_Shadows::new_world(int w, int h)
     buffer = Surface::make_surface(ss);
 }
 
-void DL_Shadows::draw (px::GC &gc, const WorldArea &a, int destx, int desty) {
+void DL_Shadows::draw (ecl::GC &gc, const WorldArea &a, int destx, int desty) {
     int x2 = a.x+a.w;
     int y2 = a.y+a.h;
     int tilew = get_engine()->get_tilew();
@@ -1509,22 +1537,26 @@ void DL_Shadows::draw(GC &gc, int xpos, int ypos, int x, int y) {
 //----------------------------------------------------------------------
 
 Follower::Follower (DisplayEngine *e)
-: m_engine(e)
+: m_boundary (0.5),
+  m_engine(e)
 {}
 
-int Follower::get_hoff() const { 
+double Follower::get_hoff() const 
+{ 
     ScreenArea gamearea = m_engine->get_area();
-    return gamearea.w / m_engine->get_tilew() - 1;
+    return gamearea.w / m_engine->get_tilew() -m_boundary*2; 
 }
 
-int Follower::get_voff() const { 
+double Follower::get_voff() const 
+{
     ScreenArea gamearea = m_engine->get_area();
-    return gamearea.h / m_engine->get_tileh() - 1;
+    return gamearea.h / m_engine->get_tileh() -m_boundary*2;
 }
 
-void Follower::center(const px::V2 &point) {
-    double borderh = 0.5; 
-    double borderv = 0.5;
+void Follower::center(const ecl::V2 &point) 
+{
+    double borderh = m_boundary; 
+    double borderv = m_boundary;
     double hoff = get_hoff();
     double voff = get_voff();
 
@@ -1535,7 +1567,8 @@ void Follower::center(const px::V2 &point) {
     set_offset(off);
 }
 
-bool Follower::set_offset (V2 offs) {
+bool Follower::set_offset (V2 offs) 
+{
     DisplayEngine *e = get_engine();
     offs[0] = max (offs[0], 0.0);
     offs[1] = max (offs[1], 0.0);
@@ -1557,7 +1590,7 @@ Follower_Screen::Follower_Screen(DisplayEngine *e)
 
 /*! Determine whether the screen must be scrolled or not, and change
   the coordinate origin of the screen accordingly. */
-void Follower_Screen::tick(double, const px::V2 &point) {
+void Follower_Screen::tick(double, const ecl::V2 &point) {
     DisplayEngine *engine = get_engine();
     V2 oldoff = engine->get_offset();
     Follower::center(point);
@@ -1576,12 +1609,14 @@ Follower_Scrolling::Follower_Scrolling(DisplayEngine *e, bool screenwise_)
   screenwise (screenwise_)
 {}
 
-void Follower_Scrolling::center(const px::V2 &point) {
+void Follower_Scrolling::center(const ecl::V2 &point) 
+{
     Follower::center(point);
     curpos = destpos = get_engine()->get_offset();
 }
 
-void Follower_Scrolling::tick(double dtime, const px::V2 &point) {
+void Follower_Scrolling::tick(double dtime, const ecl::V2 &point) 
+{
     DisplayEngine *engine   = get_engine();
 
     if (!currently_scrolling) {
@@ -1612,8 +1647,8 @@ void Follower_Scrolling::tick(double dtime, const px::V2 &point) {
             if (screenwise) {
                 double hoff = get_hoff();
                 double voff = get_voff();
-                destpos[0] = floor((point[0]-0.5) / hoff) * hoff;
-                destpos[1] = floor((point[1]-0.5) / voff) * voff;
+                destpos[0] = floor((point[0]-m_boundary) / hoff) * hoff;
+                destpos[1] = floor((point[1]-m_boundary) / voff) * voff;
             } else {
                 destpos = point - V2(gamearea.w/tilew, gamearea.h/tileh)/2;
             }
@@ -1657,7 +1692,7 @@ Follower_Smooth::Follower_Smooth (DisplayEngine *e)
 {
 }
 
-px::V2 Follower_Smooth::calc_offset (const px::V2 &point)
+ecl::V2 Follower_Smooth::calc_offset (const ecl::V2 &point)
 {
     DisplayEngine *engine   = get_engine();
     ScreenArea     gamearea = engine->get_area();
@@ -1675,13 +1710,13 @@ px::V2 Follower_Smooth::calc_offset (const px::V2 &point)
     return destpos;
 }
 
-void Follower_Smooth::tick (double /*time*/, const px::V2 &point)
+void Follower_Smooth::tick (double /*time*/, const ecl::V2 &point)
 {
     DisplayEngine *engine   = get_engine();
     engine->move_offset (calc_offset (point));
 }
 
-void Follower_Smooth::center (const px::V2 &point)
+void Follower_Smooth::center (const ecl::V2 &point)
 {
     set_offset(calc_offset (point));
 }
@@ -1897,6 +1932,12 @@ void GameDisplay::get_reference_point_coordinates(int *x, int *y) {
     get_engine()->world_to_screen(m_reference_point, x, y);
 }
 
+void GameDisplay::set_scroll_boundary (double boundary) 
+{
+    if (m_follower)
+        m_follower->set_boundary (boundary);
+}
+
 /* ---------- Screen updates ---------- */
 
 void GameDisplay::redraw_all (Screen *scr) {
@@ -1906,7 +1947,7 @@ void GameDisplay::redraw_all (Screen *scr) {
     redraw (scr);
 }
 
-void GameDisplay::redraw (px::Screen *screen) {
+void GameDisplay::redraw (ecl::Screen *screen) {
     GC gc(screen->get_surface());
     if (SDL_GetTicks() - last_frame_time > 10) {
         CommonDisplay::redraw();
@@ -2007,7 +2048,7 @@ void display::FocusReferencePoint() {
     gamedpy->follow_center();
 }
 
-void display::SetReferencePoint (const px::V2 &point) {
+void display::SetReferencePoint (const ecl::V2 &point) {
     gamedpy->set_reference_point (point);
 }
 
@@ -2015,6 +2056,11 @@ void display::SetFollowMode(FollowMode m) {
     gamedpy->set_follow_mode(m);
 }
 
+void display::SetScrollBoundary (double boundary) {
+    gamedpy->set_scroll_boundary (boundary);
+}
+
+
 void display::GetReferencePointCoordinates(int *x, int *y) {
     gamedpy->get_reference_point_coordinates(x, y);
 }
