@@ -13,10 +13,11 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: game.cc,v 1.63 2004/05/22 13:04:31 dheck Exp $
  */
+#include "game.hh"
+#include "errors.hh"
 #include "main.hh"
 #include "options.hh"
 #include "video.hh"
@@ -24,8 +25,9 @@
 #include "server.hh"
 #include "world.hh"
 #include "sound.hh"
+#include "lev/PersistentIndex.hh"
 
-#include "px/sdl.hh"
+#include "ecl_sdl.hh"
 #include <cassert>
 
 using namespace enigma;
@@ -42,24 +44,22 @@ namespace
 
 /* -------------------- Level previews -------------------- */
 
-bool game::DrawLevelPreview (px::GC &gc, const Level &l)
+bool game::DrawLevelPreview (ecl::GC &gc, lev::Proxy *levelProxy)
 {
-    LevelPack   *lp       = l.get_levelpack();
-    unsigned     levelidx = l.get_index();
     bool success = false;
 
-    server::CreatingPreview = true;
     sound::TempDisableSound();
     try {
-        server::Msg_SetLevelPack(lp->get_name());
-        server::Msg_LoadLevel (levelidx);
+        server::Msg_LoadLevel (levelProxy, true);
+
         display::DrawAll(gc);
         success = true;
     }
-    catch (levels::XLevelLoading &) {
-        ;
+    catch (XLevelLoading &err) {
+        // log the message as we cannot display it on the screen
+        Log << "DrawLevelPreview load error:\n" << err.what();
+        success = false;
     }
-    server::CreatingPreview = false;
     sound::TempReEnableSound();
     return success;
 }
@@ -68,26 +68,24 @@ bool game::DrawLevelPreview (px::GC &gc, const Level &l)
 
 /* -------------------- Functions -------------------- */
 
-void game::StartGame (levels::LevelPack *lp, unsigned ilevel)
+void game::StartGame ()
 {
+    lev::Index *ind = lev::Index::getCurrentIndex();
+    server::InitNewGame();
+
     video::HideMouse();
     sdl::TempInputGrab grab(enigma::Nograb ? SDL_GRAB_OFF : SDL_GRAB_ON);
 
-    server::CurrentLevelPack = lp;
-    server::CurrentLevel = ilevel; // XXX
-
-    server::Msg_SetLevelPack (lp->get_name());
-    server::Msg_LoadLevel (ilevel);
+    server::Msg_LoadLevel(ind->getCurrent(), false);
 
     double dtime = 0;
+    last_tick_time=SDL_GetTicks();
     while (!client::AbortGameP()) {
-        last_tick_time=SDL_GetTicks();
-
         try {
             client::Tick (dtime);
             server::Tick (dtime);
         }
-        catch (enigma_levels::XLevelRuntime& err) {        
+        catch (XLevelRuntime& err) {        
             client::Msg_Error (string("Server Error: level runtime error:\n")
                                + err.what());
             server::Msg_Panic(true);
@@ -97,11 +95,15 @@ void game::StartGame (levels::LevelPack *lp, unsigned ilevel)
         if (sleeptime >= 3)      // only sleep if relatively idle
             SDL_Delay(sleeptime);
 
-        dtime = (SDL_GetTicks()-last_tick_time)/1000.0;
+        Uint32 current_tick_time = SDL_GetTicks();
+        dtime = (current_tick_time - last_tick_time)/1000.0;
 
         if (abs(1-dtime/0.01) < 0.2) {
             // less than 20% deviation from desired frame time?
             dtime = 0.01;
+            last_tick_time += 10;
+        } else {
+            last_tick_time = current_tick_time;
         }
 
 	if (dtime > 500.0) /* Time has done something strange, perhaps
@@ -110,6 +112,9 @@ void game::StartGame (levels::LevelPack *lp, unsigned ilevel)
 // 	else if (dtime > 0.5)
 //             dtime = 0.5;
     }
+    // add last played level
+    lev::PersistentIndex::addCurrentToHistory();
+
     video::ShowMouse();
 }
 
