@@ -13,9 +13,8 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: laser.cc,v 1.26 2004/03/15 20:15:35 dheck Exp $
  */
 #include "laser.hh"
 #include "sound.hh"
@@ -30,7 +29,7 @@ using namespace world;
 using namespace lasers;
 
 using stones::maybe_push_stone;
-using px::V2;
+using ecl::V2;
 
 namespace
 {
@@ -147,6 +146,17 @@ an incoming beam).
 
 vector<void*> PhotoCell::instances;
 
+PhotoCell::~PhotoCell() 
+{
+    photo_deactivate();
+}
+
+
+/**
+ * This function notifies all instances of PhotoCell that a
+ * recalculation of the laser beams is about to begin by calling
+ * on_recalc_start() for each instance.
+ */
 void PhotoCell::notify_start()
 {
     for(unsigned i=0; i<instances.size(); ++i)
@@ -156,6 +166,11 @@ void PhotoCell::notify_start()
     }
 }
 
+/**
+ * This function notifies all instances of PhotoCell that the engine
+ * has finished recalculating the laser beams by calling
+ * on_recalc_finish() for each instance.
+ */
 void PhotoCell::notify_finish()
 {
     for(unsigned i=0; i<instances.size(); ++i)
@@ -167,18 +182,17 @@ void PhotoCell::notify_finish()
 
 void PhotoCell::photo_activate()
 {
-    vector<void*>::iterator i;
-    i = std::find(instances.begin(), instances.end(), this);
+    vector<void*>::iterator i = std::find(instances.begin(), instances.end(), this);
     if (i != instances.end())
-        assert(0 || "Photocell activated twice\n");
+        assert (0 || "Photocell activated twice\n");
     else
-        instances.push_back(this);
+        instances.push_back (this);
 }
 
 void PhotoCell::photo_deactivate()
 {
     vector<void*>::iterator i;
-    i = std::find(instances.begin(), instances.end(), this);
+    i = std::find (instances.begin(), instances.end(), this);
     if (i != instances.end())
         instances.erase(i);
 }
@@ -186,8 +200,16 @@ void PhotoCell::photo_deactivate()
 
 /* -------------------- PhotoStone -------------------- */
 
-void PhotoStone::check_state()
+PhotoStone::PhotoStone(const char *kind) : Stone(kind) 
 {
+    illuminated = false;
+}
+
+void PhotoStone::on_recalc_start() 
+{}
+
+void PhotoStone::on_recalc_finish() 
+{ 
     GridPos p = get_pos();
     bool illu = (LightFrom(p, NORTH) || LightFrom(p, EAST)
                  || LightFrom(p, WEST) || LightFrom(p, SOUTH));
@@ -223,9 +245,6 @@ void PhotoStone::check_state()
 //   the beam is recalculated.  For objects that need to be notified
 //   when the laser goes on or off, use the `PhotoStone'
 //   mixin.
-//
-// More bits of information can be found in the description of laser
-// stones and mirrors.
 
 vector<LaserBeam*> LaserBeam::instances;
 map<GridPos, int>  LaserBeam::old_laser_positions;
@@ -349,7 +368,7 @@ void LaserBeam::all_emitted()
     }
 
     if (count) {
-        sound::SoundEvent ("laseron", px::V2(x/count+.5, y/count+.5));
+        sound::SoundEvent ("laseron", ecl::V2(x/count+.5, y/count+.5));
     }
 
     old_laser_positions.clear();
@@ -427,8 +446,8 @@ namespace
     protected:
         MirrorStone(const char *name, bool movable=false, bool transparent=false);
 
-        bool is_transparent() const { return int_attrib("transparent"); }
-        bool is_movable() const { return int_attrib("movable"); }
+        bool is_transparent() const { return int_attrib("transparent") != 0; }
+        bool is_movable() const { return int_attrib("movable") != 0; }
 
         void set_orientation(int o) { set_attrib("orientation", o); }
         int get_orientation() { return int_attrib("orientation"); }
@@ -444,7 +463,7 @@ namespace
         void init_model();
     private:
 	// Object interface.
-        void message(const string &m, const Value &);
+        virtual Value message(const string &m, const Value &);
 
         // LaserEmitter interface
         DirectionBits emission_directions() const {
@@ -485,7 +504,7 @@ void MirrorStone::init_model() {
     set_model(mname);
 }
 
-void MirrorStone::message(const string &m, const Value &val) {
+Value MirrorStone::message(const string &m, const Value &val) {
     if (m == "trigger" || m=="turn") {
         rotate_right();
     }
@@ -514,6 +533,7 @@ void MirrorStone::message(const string &m, const Value &val) {
         init_model();
         MaybeRecalcLight(get_pos());
     }
+    return Value();
 }
 
 void MirrorStone::actor_hit(const world::StoneContact &sc)
@@ -559,7 +579,7 @@ namespace
     private:
         void SetOrientation(char o) {
             const char *a = " -\\|/";
-            MirrorStone::set_orientation(strchr(a,o)-a);
+            MirrorStone::set_orientation(int (strchr(a,o)-a));
         }
         char GetOrientation() {
             const char *a = " -\\|/";
@@ -650,7 +670,7 @@ namespace
 
         void SetOrientation(char o) {
             const char *a = " v<^>";
-            MirrorStone::set_orientation(strchr(a,o)-a);
+            MirrorStone::set_orientation( int (strchr(a,o)-a));
         }
 
         Direction GetOrientation() // orientation of the flat side of the mirror
