@@ -13,35 +13,47 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: lua.cc,v 1.22 2004/05/22 13:04:30 dheck Exp $
  */
 #include "lua.hh"
+#include "errors.hh"
+#include "main.hh"
 #include "world.hh"
 #include "config.h"
 #include "video.hh"
 #include "server.hh"
 #include "sound.hh"
 #include "options.hh"
+#include "lev/Index.hh"
+#include "lev/Proxy.hh"
 
-#include "display-lua.hh"
-#include "enigma-lua.hh"
-#include "px-lua.hh"
-
+#ifndef CXXLUA
 extern "C" {
 #include "lualib.h"
-#include "tolua.h"
+#include "tolua++.h"
 }
-#include "px/px.hh"
+#else
+#include "lualib.h"
+#include "tolua++.h"
+#endif 
+
+
+#include "lua-display.hh"
+#include "lua-enigma.hh"
+#include "lua-ecl.hh"
+
+#include "ecl.hh"
 #include <cassert>
 
+#include "nls.hh"
+
 using namespace std;
 using namespace enigma;
 using namespace lua;
 
-using px::round_down;
-using px::strf;
+using ecl::round_down;
+using ecl::strf;
 
 using enigma::GridPos;
 using world::Object;
@@ -59,11 +71,56 @@ namespace lua
 namespace 
 {
     lua_State *level_state = 0;  // level-local state
-    int object_tag;             // Lua tag for `Object's
     lua_State *global_state = 0; // global Lua state
+
+    lua::Error _lua_err_code (int i)
+    {
+        switch (i) {
+	case 0: return NO_LUAERROR;
+	case LUA_ERRRUN: return ERRRUN;
+	case LUA_ERRFILE: return ERRFILE;
+	case LUA_ERRSYNTAX: return ERRSYNTAX;
+	case LUA_ERRMEM: return ERRMEM;
+	case LUA_ERRERR: return ERRERR;
+	}
+	assert (!"Should never get there!");
+    }
+
+    void throwLuaError(lua_State * L, const char * message) {
+        std::string backtrace = message;
+        backtrace += "\nBacktrace:\n";
+        lua_Debug dbgInfo;
+        int frame = 0;
+        while (lua_getstack(L, frame, &dbgInfo)) {
+            lua_getinfo(L, "Sl", &dbgInfo);
+            if (dbgInfo.source[0] == '@')
+                // lua code loaded from file
+                backtrace += ecl::strf("#%d %s: %d\n", frame, dbgInfo.source,
+                                dbgInfo.currentline);
+            else if (dbgInfo.source[0] == '-' && dbgInfo.source[1] == '-' &&
+                dbgInfo.source[2] == '@') {
+                // lua code loaded from string
+                std::string code = dbgInfo.source;
+                std::string::size_type slashPosFilenameEnd = code.find('\n');
+                std::string::size_type slashPosLineStart;
+                std::string::size_type slashPosLineEnd = slashPosFilenameEnd;
+                for (int i = 1; i < dbgInfo.currentline; i++) {
+                    slashPosLineStart = slashPosLineEnd;
+                    slashPosLineEnd = code.find('\n', slashPosLineStart + 1);
+                }
+                backtrace += ecl::strf("#%d %s: %d\n  (%s)\n", frame, 
+                        code.substr(2, slashPosFilenameEnd - 2).c_str(),
+                        dbgInfo.currentline - 1,
+                        code.substr(slashPosLineStart + 1, slashPosLineEnd - 
+                                slashPosLineStart - 1).c_str());
+            }
+            frame++;
+        }
+        luaL_error(L, backtrace.c_str());
+        
+    }
 }
 
-
 /* -------------------- Helper routines -------------------- */
 
 using enigma::Value;
@@ -82,7 +139,7 @@ void lua::SetTableVar (lua_State *L,
 
 void lua::SetSoundTable (const char *name)
 {
-    CallFunc (global_state, "ActivateSoundTable", name);
+    CallFunc (global_state, "ActivateSoundTable", name, NULL);
 }
 
 
@@ -103,7 +160,8 @@ to_value(lua_State *L, int idx)
     case LUA_TNIL: return Value();
     case LUA_TNUMBER: return Value(lua_tonumber(L,idx));
     case LUA_TSTRING: return Value(lua_tostring(L,idx));
-    default: lua_error(L,"Cannot convert type to Value.");
+    case LUA_TBOOLEAN: return (lua_toboolean(L,idx)) ? Value(1) : Value();
+    default: throwLuaError(L,"Cannot convert type to Value.");
     }
     return Value();
 }
@@ -111,7 +169,7 @@ to_value(lua_State *L, int idx)
 static bool
 is_object(lua_State *L, int idx)
 {
-    return lua_isuserdata(L,idx) && lua_tag(L,idx)==object_tag;
+    return lua_isuserdata(L,idx) && luaL_checkudata(L,idx,"_ENIGMAOBJECT");
 }
 
 static Object *
@@ -121,21 +179,27 @@ to_object(lua_State *L, int idx)
         return 0;
 
     if (!is_object(L,idx)) {
-        lua_error(L, "Cannot convert type to an Object");
+        throwLuaError(L, "Cannot convert type to an Object");
         return 0;
     }
-    return static_cast<Object*>(lua_touserdata(L,idx));
+    return static_cast<Object*>(*(static_cast<void**>(lua_touserdata(L,idx))));
 }
 
 static void
 pushobject (lua_State *L, Object *obj)
 {
+    Object **udata;
     /* Lua does not allow NULL pointers in userdata variables, so
        convert them manually to `nil' values. */
     if (obj == 0)
         lua_pushnil(L);
     else
-        lua_pushusertag(L, obj, object_tag);
+      {
+        udata=(Object**)lua_newuserdata(L,sizeof(int));
+	*udata=obj;
+        luaL_getmetatable(L, "_ENIGMAOBJECT");
+        lua_setmetatable(L, -2);
+      }
 }
 
 
@@ -145,9 +209,11 @@ int lua::MakeObject (lua_State *L)
 {
     const char *name = lua_tostring(L, 1);
     if (!name) {
-        lua_error(L, "MakeObject: string expected as argument");
+        throwLuaError(L, "MakeObject: string expected as argument");
     }
     Object *obj = world::MakeObject(name);
+    if (obj == NULL)
+        throwLuaError(L, ecl::strf("MakeObject: unknown object name '%s'", name).c_str());
     pushobject(L, obj);
     return 1;
 }
@@ -168,7 +234,7 @@ en_set_attrib(lua_State *L)
     if (obj && key)
         obj->set_attrib(key, to_value(L, 3));
     else
-        lua_error(L, strf("SetAttrib: invalid object or attribute name '%s'", key).c_str());
+        throwLuaError(L, strf("SetAttrib: invalid object or attribute name '%s'", key).c_str());
     return 0;
 }
 
@@ -179,22 +245,22 @@ en_get_attrib(lua_State *L)
     const char *key = lua_tostring(L,2);
 
     if (!obj) {
-        lua_error(L, "GetAttrib: invalid object");
+        throwLuaError(L, "GetAttrib: invalid object");
         return 0;
     }
     if (!key) {
-        lua_error(L, "GetAttrib: invalid key");
+        throwLuaError(L, "GetAttrib: invalid key");
         return 0;
     }
 
     if (0 == strcmp(key, "kind")) {
-        lua_error(L, "GetAttrib: illegal attribute, use GetKind()");
+        throwLuaError(L, "GetAttrib: illegal attribute, use GetKind()");
         return 0;
     }
 
     const Value *v =  obj->get_attrib(key);
     if (!v) {
-        lua_error(L, strf("GetAttrib: unknown attribute '%s'", key).c_str());
+        // unknown attribute
         lua_pushnil(L);
     }
     else
@@ -208,7 +274,7 @@ en_get_kind(lua_State *L)
     Object *obj = to_object(L,1);
 
     if (!obj) {
-        lua_error(L, "GetKind: invalid object");
+        throwLuaError(L, "GetKind: invalid object");
         return 0;
     }
 
@@ -216,6 +282,16 @@ en_get_kind(lua_State *L)
     return 1;
 }
 
+static int
+en_is_same_object(lua_State *L)
+{
+    Object *obj1 = to_object(L,1);
+    Object *obj2 = to_object(L,2);
+
+    lua_pushboolean(L, obj1 == obj2);
+    return 1;
+}
+
 static int
 en_set_floor(lua_State *L)
 {
@@ -226,11 +302,11 @@ en_set_floor(lua_State *L)
     if (lua_isnil(L, 3))
         fl = 0;
     else if (is_object(L,3)) {
-        fl = static_cast<Floor*>(lua_touserdata(L,3));
+        fl = static_cast<Floor*>(*(static_cast<void**> (lua_touserdata(L,3))));
     	if( ! fl)
-	    lua_error(L, "object is no valid floor");
+	    throwLuaError(L, "object is no valid floor");
     } else
-        lua_error(L, "argument 3 must be an Object or nil");
+        throwLuaError(L, "argument 3 must be an Object or nil");
     world::SetFloor(GridPos(x,y), fl);
     return 0;
 }
@@ -241,8 +317,9 @@ en_set_item(lua_State *L)
     int x = round_down<int>(lua_tonumber(L, 1));
     int y = round_down<int>(lua_tonumber(L, 2));
     Item *it = dynamic_cast<Item*>(to_object(L, 3));
-    if( ! it)
-        lua_error(L, "object is no valid item");
+    if( ! it) {
+        throwLuaError(L, "object is no valid item");
+    }
     world::SetItem(GridPos(x,y), it);
     return 0;
 }
@@ -254,7 +331,7 @@ en_set_stone(lua_State *L)
     int y = round_down<int>(lua_tonumber(L, 2));
     Stone *st = dynamic_cast<Stone*>(to_object(L, 3));
     if( ! st)
-        lua_error(L, "object is no valid stone");
+        throwLuaError(L, "object is no valid stone");
     world::SetStone(GridPos(x,y), st);
     return 0;
 }
@@ -282,8 +359,11 @@ en_set_actor(lua_State *L)
     double y = lua_tonumber(L,2);
     Actor *ac = dynamic_cast<Actor*>(to_object(L, 3));
     if( ! ac)
-        lua_error(L, "object is no valid actor");
-    world::AddActor(x, y, ac);
+        throwLuaError(L, "object is no valid actor");
+    if (world::IsInsideLevel(GridPos(round_down<int>(x), round_down<int>(y))))
+        world::AddActor(x, y, ac);
+    else
+        throwLuaError(L, "position is outside of world");
     return 0;
 }
 
@@ -293,19 +373,21 @@ en_send_message(lua_State *L)
 {
     Object     *obj = to_object(L, 1);
     const char *msg = lua_tostring(L, 2);
+    Value v;
     if (!msg)
-        lua_error(L,"Illegal message");
+        throwLuaError(L,"Illegal message");
     else if (obj) {
         try {
-            world::SendMessage (obj, msg, to_value(L, 3));
+            v = world::SendMessage (obj, msg, to_value(L, 3));
         }
-        catch (const enigma_levels::XLevelRuntime &e) {
-            lua_error (L, e.what());
+        catch (const XLevelRuntime &e) {
+            throwLuaError (L, e.what());
         }
         catch (...) {
-            lua_error (L, "uncaught exception");
+            throwLuaError (L, "uncaught exception");
         }
     }
+    push_value(L, v);
     return 0;
 }
 
@@ -316,7 +398,7 @@ int lua::PlaySound (lua_State *L)
     double      y         = lua_tonumber (L, 3);
     double      volume    = lua_tonumber (L, 4);
 
-    sound::PlaySound (soundname, px::V2 (x, y), volume);
+    sound::PlaySound (soundname, ecl::V2 (x, y), volume);
 
     return 0;
 }
@@ -339,16 +421,16 @@ int lua::EmitSound (lua_State *L)
     const char *soundname = lua_tostring(L, 2);
 
     if (!soundname)
-        lua_error(L,"Illegal sound");
+        throwLuaError(L,"Illegal sound");
     else if (obj) {
         GridObject *gobj = dynamic_cast<GridObject*>(obj);
         if (gobj) {
             if (!gobj->sound_event (soundname)) 
-                lua_error(L, strf("Can't find sound '%s'", soundname).c_str());
+                throwLuaError(L, strf("Can't find sound '%s'", soundname).c_str());
         }
     }
     else
-        lua_error(L, "EmitSound: invalid object");
+        throwLuaError(L, "EmitSound: invalid object");
 
     return 0;
 }
@@ -360,9 +442,9 @@ en_name_object(lua_State *L)
     const char *name = lua_tostring(L,2);
 
     if (!obj) 
-        lua_error(L, "NameObject: Illegal object");
+        throwLuaError(L, "NameObject: Illegal object");
     else if (!name) 
-        lua_error(L, "NameObject: Illegal name");
+        throwLuaError(L, "NameObject: Illegal name");
     else
         world::NameObject(obj, name);
 
@@ -412,7 +494,7 @@ en_get_pos(lua_State *L)
     GridPos  p;
 
     if (!obj) {
-        lua_error(L, "GetPos: Illegal object");
+        throwLuaError(L, "GetPos: Illegal object");
         return 0;
     }
 
@@ -430,7 +512,7 @@ en_get_pos(lua_State *L)
 }
 
 static int en_add_constant_force(lua_State *L) {
-    px::V2 v;
+    ecl::V2 v;
     v[0] = lua_tonumber(L, 1);
     v[1] = lua_tonumber(L, 2);
     world::SetConstantForce (v);
@@ -450,14 +532,14 @@ en_add_rubber_band (lua_State *L)
     d.minlength = lua_tonumber (L, 5);
 
     if (!a1)
-        lua_error(L, "AddRubberBand: First argument must be an actor\n");
+        throwLuaError(L, "AddRubberBand: First argument must be an actor\n");
     else {
         if (a2)
             world::AddRubberBand (a1, a2, d);
         else if (st)
             world::AddRubberBand (a1, st, d);
         else
-            lua_error(L, "AddRubberBand: Second argument must be actor or stone\n");
+            throwLuaError(L, "AddRubberBand: Second argument must be actor or stone\n");
     }
     return 0;
 }
@@ -472,15 +554,23 @@ en_get_ticks(lua_State *L)
 static int
 en_is_solved(lua_State *L)
 {
-    const char  *levelname = lua_tostring(L,1);
-    int          solved    = 0;
+    // Function depreceated
+    // - filename is no longer a useful reference for levels
+    // - levels should not depend on external data for reasons of
+    //   network compatibility and level journaling
+    throwLuaError(L, "Usage of depreceated function \"IsSolved()\"");
 
-    levels::Level level (0, 0);
-    if (levels::FindLevel (levelname, level))
-        solved = level.is_solved(options::GetInt("Difficulty"));
-    else 
-        lua_error(L, strf("IsSolved: Unknown level '%s'", levelname).c_str());
-    lua_pushnumber(L, solved);
+//    const char  *levelname = lua_tostring(L,1);
+//    int          solved    = 0;
+//    
+//    levels::Level level (0, 0);
+//    if (levels::FindLevel (levelname, level)) {
+//        solved = level.is_solved(options::GetInt("Difficulty"));
+//    } else 
+//    if (solved)
+//        lua_pushnumber(L, solved);
+//    else
+//        lua_pushnil(L);
     return 1;
 }
 
@@ -496,7 +586,7 @@ en_add_scramble(lua_State *L)
     if (found && found[0]) 
         world::AddScramble(GridPos(x,y), enigma::Direction(found-allowed));
     else 
-        lua_error(L, "AddScramble: Third argument must be one character of \"wsen\"");
+        throwLuaError(L, "AddScramble: Third argument must be one character of \"wsen\"");
 
     return 0;
 }
@@ -518,9 +608,9 @@ en_add_signal (lua_State *L) {
     GridLoc source, target;
 
     if (sourcestr == 0 || !to_gridloc(sourcestr, source))
-        lua_error (L, "AddSignal: invalid source descriptor");
+        throwLuaError (L, "AddSignal: invalid source descriptor");
     if (targetstr == 0 || !to_gridloc(targetstr, target)) 
-        lua_error (L, "AddSignal: invalid target descriptor");
+        throwLuaError (L, "AddSignal: invalid target descriptor");
     if (msg == 0)
         msg = "signal";
 
@@ -528,6 +618,18 @@ en_add_signal (lua_State *L) {
     return 0;
 }
 
+int loadLib(lua_State *L) 
+{
+    const char *id = lua_tostring(L, 1);
+    lev::Proxy * curProxy = lev::Index::getCurrentProxy();
+    try {
+        curProxy->loadDependency(id);
+    } catch (XLevelLoading &err) {
+        throwLuaError(L, err.what());
+    }
+    return 1;
+}
+
 static CFunction globalfuncs[] = {
     {FindDataFile,          "FindDataFile"},
     {lua::PlaySoundGlobal,  "PlaySoundGlobal"},
@@ -541,6 +643,7 @@ static CFunction levelfuncs[] = {
     // internal functions
 
     {FindDataFile,          "FindDataFile"},
+    {loadLib,               "LoadLib"},
     {en_get_object_template,"GetObjectTemplate"},
     {lua::MakeObject,       "MakeObject"},
     {en_set_actor,          "SetActor"},
@@ -558,6 +661,7 @@ static CFunction levelfuncs[] = {
     {en_get_pos,            "GetPos"},
     {en_get_attrib,         "GetAttrib"},
     {en_get_kind,           "GetKind"},
+    {en_is_same_object,     "IsSameObject"},
 
     // manipulating objects
 
@@ -604,8 +708,7 @@ int lua::FindDataFile (lua_State *L)
 {
     const char *filename = lua_tostring(L, 1);
     string absfile;
-    if (enigma::FindFile(filename, absfile)) 
-//        = enigma::FindDataFile(filename);
+    if (app.resourceFS->findFile(filename, absfile)) 
         lua_pushstring(L, absfile.c_str());
     else
         lua_pushnil (L);
@@ -623,64 +726,156 @@ void lua::RegisterFuncs(lua_State *L, CFunction *funcs)
     lua_pop(L, 1);
 }
 
-int lua::CallFunc(lua_State *L, const char *funcname, const Value& arg) {
+Error lua::CallFunc(lua_State *L, const char *funcname, const Value& arg, world::Object *obj) {
+    int retval;
     lua_getglobal(L, funcname);
     push_value(L, arg);
-    return lua_call(L, 1, 0);
+    pushobject(L, obj);
+    retval=lua_pcall(L,2,0,0);
+    if (retval!=0) // error
+      {
+	lua_setglobal (L, "_LASTERROR") ; //Set _LASTERROR to returned error message
+      }
+    return _lua_err_code(retval);
 }
 
-int lua::CallFunc(lua_State *L, const char *funcname, const file::ByteVec& arg) {
+Error lua::CallFunc(lua_State *L, const char *funcname, const ByteVec& arg) {
+  int retval;
     lua_getglobal(L, funcname);
     lua_pushlstring (L, &arg[0], arg.size());
-    return lua_call(L, 1, 0);
+    retval=lua_pcall(L,1,0,0);
+    if (retval!=0) // error
+      {
+	lua_setglobal (L, "_LASTERROR") ; //Set _LASTERROR to returned error message
+      }
+    return _lua_err_code(retval);
 }
 
-int lua::Dofile(lua_State *L, const string &filename) 
+Error lua::DoAbsoluteFile(lua_State *L, const string &filename)
 {
-    string fname;
-    if (enigma::FindFile(filename, fname)) {
-        int oldtop = lua_gettop(L);
-        int retval = lua_dofile(L, fname.c_str());
-        lua_settop(L, oldtop);
-        return retval;
+    int oldtop = lua_gettop(L);
+    int retval = luaL_loadfile(L, filename.c_str());
+    if (retval!=0) // error
+    {
+      lua_setglobal (L, "_LASTERROR") ; //Set _LASTERROR to returned error message
+    }
+    else 
+    {
+      retval= lua_pcall(L, 0, 0, 0);
+      if (retval!=0) // error
+      {
+        lua_setglobal (L, "_LASTERROR") ; //Set _LASTERROR to returned error message
+      }
+    }
+    lua_settop(L, oldtop);
+    return _lua_err_code(retval);
+}
+    
+Error lua::DoGeneralFile(lua_State *L, GameFS * fs, const string &filename)
+{
+    string completefn;
+    if (fs->findFile(filename, completefn)) {
+        return lua::DoAbsoluteFile(L, completefn);
+    }
+    else {
+        return _lua_err_code(LUA_ERRFILE);
     }
-    return LUA_ERRFILE;
 }
 
-int lua::Dobuffer (lua_State *L, const file::ByteVec &luacode) {
+Error lua::Dofile(lua_State *L, const string &filename) 
+{
+    return lua::DoGeneralFile(L, app.resourceFS, filename);
+}
+
+Error lua::DoSysFile(lua_State *L, const string &filename) 
+{
+    return lua::DoGeneralFile(L, app.systemFS, filename);
+}
+
+void lua::CheckedDoFile (lua_State *L, GameFS * fs, std::string const& fname)
+{
+    string completefn;
+    if (!fs->findFile(fname, completefn))
+    {
+        fprintf(stderr, _("Cannot find '%s'.\n"), fname.c_str());
+        fprintf(stderr, _("Your installation may be incomplete or invalid.\n"));
+        exit (1);
+    }
+
+    lua::Error status = lua::DoAbsoluteFile(L, completefn);
+    if (status != lua::NO_LUAERROR) {
+        fprintf(stderr, _("There was an error loading '%s'.\n"), completefn.c_str());
+        fprintf(stderr, _("Your installation may be incomplete or invalid.\n"));
+	fprintf(stderr, _("Error: '%s'\n"), lua::LastError(L).c_str());
+        exit (1);
+    }
+}
+
+
+Error lua::Dobuffer (lua_State *L, const ByteVec &luacode) {
+  int retval;
     const char *buffer = reinterpret_cast<const char *>(&luacode[0]);
-    return lua_dobuffer (L, buffer, luacode.size(), "buffer");
+    
+    retval=luaL_loadbuffer(L, buffer, luacode.size(), "buffer");
+    if (retval!=0) // error
+      {
+	lua_setglobal (L, "_LASTERROR") ; //Set _LASTERROR to returned error message
+      }
+      else 
+	{
+	  retval= lua_pcall(L, 0, 0, 0);
+	  if (retval!=0) // error
+          {
+	    lua_setglobal (L, "_LASTERROR") ; //Set _LASTERROR to returned error message
+          }
+	}
+    return _lua_err_code(retval);
 }
 
 string lua::LastError (lua_State *L)
 {
     lua_getglobal (L, "_LASTERROR");
-    return string (lua_tostring (L, -1));
+    if (lua_isstring(L,-1)){
+      return string (lua_tostring (L, -1));
+    }
+    else {
+      return "Lua Error. No error message available.";
+    }
 }
 
 
-int lua::DoSubfolderfile(lua_State *L, const string &basefolder, const string &filename) {
-    std::list <string> fnames = enigma::FindDataFiles(basefolder, filename);
+Error lua::DoSubfolderfile(lua_State *L, const string &basefolder, const string &filename) {
+    std::list <string> fnames = app.resourceFS->findSubfolderFiles(basefolder, filename);
     int retval = 0;
     while (fnames.size() > 0) {
         int oldtop = lua_gettop(L);
 	string fname = fnames.front();
-        retval = lua_dofile(L, fname.c_str());
+        retval = luaL_loadfile(L, fname.c_str());
+        if (retval!=0) // error
+        {
+	  lua_setglobal (L, "_LASTERROR") ; //Set _LASTERROR to returned error message
+        }
+	else 
+	{
+	  retval= lua_pcall(L, 0, 0, 0);
+	  if (retval!=0) // error
+          {
+	    lua_setglobal (L, "_LASTERROR") ; //Set _LASTERROR to returned error message
+          }
+	}
 	fnames.pop_front();
         lua_settop(L, oldtop);
     }
-    return retval;
+    return _lua_err_code(retval);
 }
 
 lua_State *lua::GlobalState() 
 {
     if (global_state == 0) {
-        lua_State *L = global_state = lua_open(0);
+        lua_State *L = global_state = lua_open();
 
-        lua_baselibopen(L);
-        lua_strlibopen(L);
-        lua_mathlibopen(L);
-        lua_iolibopen(L);
+        luaL_openlibs(L);
+        CheckedDoFile(L, app.systemFS, "compat.lua");
 
         tolua_open(L);
         tolua_enigma_open(L);
@@ -697,27 +892,23 @@ void lua::ShutdownGlobal()
     global_state = 0;
 }
 
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
+
 
 
 lua_State *lua::InitLevel() 
 {
     char buffer[255];
 
-    lua_State *L = level_state = lua_open(0);
-
-    lua_dostring (L, "options={}");
+    lua_State *L = level_state = lua_open();
+    luaL_dostring (L, "options={}");
     snprintf (buffer, sizeof(buffer), "options.Difficulty = %d", 
               server::GetDifficulty());
-    lua_dostring (L, buffer);
-
-    lua_baselibopen(L);
-    lua_strlibopen(L);
-    lua_mathlibopen(L);
-    lua_iolibopen(L);
-
-    // Override _ALERT function so we can catch error messages
-    lua_dostring (L, "function _ALERT(msg) _LASTERROR = msg end");
+    luaL_dostring (L, buffer);
 
+    luaL_openlibs(L);
 
     tolua_open(L);
     tolua_enigma_open(L);
@@ -726,8 +917,8 @@ lua_State *lua::InitLevel()
 
     RegisterFuncs(L, levelfuncs);
 
-    // Create a new tag for world::Object objects
-    object_tag = lua_newtag(L);
+    // Create a new metatable for world::Object objects
+    luaL_newmetatable(L,"_ENIGMAOBJECT");
     return L;
 }
 
@@ -742,3 +933,4 @@ void lua::ShutdownLevel() {
         level_state = 0;
     }
 }
+
