@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002,2003,2004 Daniel Heck
+ * Copyright (C) 2002,2003,2004,2006 Daniel Heck
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -13,26 +13,29 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: player.cc,v 1.42 2004/05/22 13:04:30 dheck Exp $
  */
 #include "player.hh"
+#include "Inventory.hh"
 #include "display.hh"
+#include "errors.hh"
 #include "sound.hh"
 #include "client.hh"
 #include "server.hh"
 #include "world.hh"
 #include "main.hh"
 
-#include "px/tools.hh"
+#include "ecl_util.hh"
 
 #include <algorithm>
 #include <iostream>
 #include <cassert>
 
 using namespace std;
-using namespace enigma::player;
+using namespace enigma;
+using world::Actor;
+using enigma::Inventory;
 
 namespace
 {
@@ -40,12 +43,12 @@ namespace
     public:
         PlayerInfo();
 
-        string          name;
-        Inventory       inventory;
-        vector<Actor*>  actors;
-        bool            out_of_lives;
-        double          dead_dtime; // number of seconds the player is already dead
-        bool            inhibit_pickup;
+        string         name;
+        Inventory      inventory;
+        vector<Actor*> actors;
+        bool           out_of_lives;
+        double         dead_dtime; // number of seconds the player is already dead
+        bool           inhibit_pickup;
     };
 
     typedef vector<PlayerInfo> PlayerList;
@@ -73,22 +76,6 @@ namespace
     };
 }
 
-/* -------------------- Auxiliary functions -------------------- */
-namespace
-{
-    void UpdateInventory (Inventory *inv)
-    {
-        if (inv == GetInventory (CurrentPlayer())) {
-            std::vector<std::string> modelnames;
-            for (int i=0; i<inv->size(); ++i) {
-                world::Item *it = inv->get_item(i);
-                modelnames.push_back(it->get_inventory_model());
-            }
-            STATUSBAR->set_inventory (modelnames);
-        }
-    }
-}
-
 /* -------------------- PlayerInfo -------------------- */
 
 PlayerInfo::PlayerInfo ()
@@ -110,6 +97,9 @@ void LevelLocalData::respawn_dead_actors(double dtime)
 
         info.time_left -= dtime;
         if (info.time_left < 0) {
+//             if (lua::CallFunc (lua::LevelState(), "Respawn", Value()) != 0) {
+//                 throw enigma_levels::XLevelRuntime(string("Calling 'Respawn' failed:\n")+lua::LastError(lua::LevelState()));
+                
             info.actor->respawn();
             respawn_list.erase(respawn_list.begin()+i);
             continue;           // don't increment i
@@ -135,9 +125,11 @@ bool LevelLocalData::remove_extralife (Actor *a)
 
     if (idx == -1) // no extralife found
         return false;
-
-    delete inv->yield_item(idx);
-    return true;
+    else {
+        delete inv->yield_item(idx);
+        player::RedrawInventory (inv);
+        return true;
+    }
 }
 
 
@@ -146,144 +138,33 @@ bool LevelLocalData::remove_extralife (Actor *a)
 namespace
 {
     LevelLocalData leveldat;
-    PlayerList     players;     // this currently has always size 2
+    PlayerList     players(2);     // this currently has always size 2
     unsigned       icurrent_player = 0;
-}
-
-
-
-
-/* -------------------- Inventory -------------------- */
-
-unsigned const Inventory::max_items = 12;
-
-Inventory::Inventory() : m_items()
-{}
-
-Inventory::~Inventory() {
-    clear();
-}
-
-void Inventory::clear() {
-    px::delete_sequence(m_items.begin(), m_items.end());
-    m_items.clear();
-}
-
-Item * Inventory::get_item(int idx) const {
-    return idx >= size() ? 0 : m_items[idx];
-}
-
-Item * Inventory::yield_item(int idx) {
-    if (0 <= idx && idx < size()) {
-        Item *it = m_items[idx];
-        m_items.erase(m_items.begin()+ idx);
-        redraw();
-        return it;
-    }
-    return 0;
-}
-
-Item * Inventory::yield_first() {
-    return yield_item(0);
-}
-
-Item * Inventory::remove_item(Item *wanted) {
-    vector<Item*>::iterator e = m_items.end();
-    for (vector<Item*>::iterator i = m_items.begin(); i != e; ++i) {
-        if (*i == wanted) {
-            m_items.erase(i);
-            return wanted;
-        }
-    }
-    return 0;
-}
-
-bool Inventory::is_full() const 
-{
-    ItemHolder *holder = dynamic_cast<ItemHolder*>(get_item(0));
-    if (holder)
-        return holder->is_full(); 
-    return m_items.size() == max_items; 
-}
-
-void Inventory::add_item(Item *i) 
-{
-    ItemHolder *holder = dynamic_cast<ItemHolder*>(get_item(0));
-    if (holder && !holder->is_full()) {
-        // first item is a bag and not full
-        holder->add_item(i);
-    }
-    else {
-        m_items.insert(m_items.begin(), i);
-        redraw();
-    }
-}
-
-void Inventory::redraw() 
-{
-    UpdateInventory (this);
-}
-
-void Inventory::rotate(int dir) 
-{
-    if (!m_items.empty()) {
-	if (dir==1)
-	    std::rotate(m_items.begin(), m_items.begin()+1, m_items.end());
-	else
-	    std::rotate(m_items.begin(), m_items.end()-1, m_items.end());
-
-        redraw();
-    }
-}
-
-void Inventory::activate_first() 
-{
-    if (!m_items.empty()) {
-        Item *it = m_items[0];
-        Actor *ac = 0;
-        GridPos p;
-        bool can_drop_item = false;
-        if (!players[icurrent_player].actors.empty()) {
-            ac = players[icurrent_player].actors[0];
-            p = GridPos(ac->get_pos());
-            can_drop_item = ac->can_drop_items();
-        }
-
-        switch (it->activate(ac, p)) {
-        case world::ITEM_DROP:
-            // only drop if no item underneath and actor allows it
-            if (it->can_drop_at(p) && can_drop_item) {
-                remove_item(it);
-                redraw();
-                it->drop(ac, p);
-            }
-            break;
-        case world::ITEM_KILL:
-            remove_item(it);
-            redraw();
-            break;
-        case world::ITEM_KEEP:
-            break;
-        }
-    }
-}
-
-int Inventory::find(const string& kind, int start_idx) const 
-{
-    int size_ = size();
-    for (int i = start_idx; i<size_; ++i) {
-        if (get_item(i)->is_kind(kind))
-            return i;
-    }
-    return -1;
+    std::vector<Actor *> unassignedActors;
 }
 
 
 /* -------------------- Functions -------------------- */
 
-void player::NewGame () 
-{
+void player::NewGame (bool isRestart) {
     int nplayers = 2;           // Always prepare for two players
+    std::vector<int> extralives(2);
+
+    // calculate number of extralives
+    for (int i=0; i<nplayers; ++i) {
+        if (isRestart) {
+            // count existing number of extralives
+            int idxLife = -1;
+            extralives[i] = -1;
+            do {
+                ++extralives[i];
+                idxLife = players[i].inventory.find("it-extralife", ++idxLife);
+            } while (idxLife != -1);
+        } else {
+            // new game provides 2 extralives
+            extralives[i] = 2;
+        }
+    }
 
     players.clear();
     players.resize(nplayers);
@@ -291,9 +172,11 @@ void player::NewGame ()
     for (int i=0; i<nplayers; ++i) {
         Inventory *inv = GetInventory(i);
 
-        inv->add_item (world::MakeItem (world::it_extralife));
-        inv->add_item (world::MakeItem (world::it_extralife));
+        for (int j = 0 ; j < extralives[i]; j++)
+            inv->add_item (MakeItem (world::it_extralife));
     }
+    
+    unassignedActors.clear();
 }
 
 void player::AddYinYang ()
@@ -305,23 +188,21 @@ void player::AddYinYang ()
     }
 }
 
-void player::LevelLoaded()
+void player::LevelLoaded(bool isRestart)
 {
     if (server::TwoPlayerGame && server::SingleComputerGame)
         AddYinYang();
-    Inventory *inv = GetInventory (icurrent_player);
-    assert (inv);
-    inv->redraw();
+    RedrawInventory ();
 }
 
-void player::NewWorld() 
+void player::PrepareLevel() 
 {
     // Clear up the inventories of all players: keep only extra lifes.
     for (unsigned iplayer=0; iplayer<players.size(); ++iplayer)
     {
         Inventory *inv = GetInventory(iplayer);
         int nextralifes=0;
-        for (int i=0; i<inv->size(); ++i)
+        for (size_t i=0; i<inv->size(); ++i)
             if (get_id (inv->get_item(i)) == world::it_extralife)
                 nextralifes += 1;
         inv->clear();
@@ -341,28 +222,28 @@ void player::LevelFinished()
         for (unsigned j=0; j<players[i].actors.size(); ++j) {
             Actor *a = players[i].actors[j];
             world::SendMessage(a, "disappear");
+            world::KillRubberBands (a);
         }
     }
 }
 
-Inventory * player::GetInventory (int iplayer) {
+Inventory * player::GetInventory (int iplayer) 
+{
     return &players[iplayer].inventory;
 }
 
-Inventory * player::GetInventory (Actor *a) {
+
+Inventory * player::GetInventory (Actor *a) 
+{
     if (const Value *v = a->get_attrib("player"))
         return GetInventory(to_int(*v));
     return 0;
 }
 
-Item * player::wielded_item (Actor *a) {
-    if (player::Inventory *inv = player::GetInventory(a))
-        return inv->get_item(0);
-    return 0;
-}
 
-bool player::wielded_item_is(Actor *a, const string &kind) {
-    if (player::Inventory *inv = GetInventory(a))
+bool player::WieldedItemIs (Actor *a, const string &kind) 
+{
+    if (Inventory *inv = GetInventory(a))
         if (Item *it = inv->get_item(0))
             return it->is_kind(kind);
     return false;
@@ -373,17 +254,13 @@ int player::CurrentPlayer() {
     return icurrent_player;
 }
 
-bool player::IsCurrentPlayer(Actor *a) {
-    return static_cast<int>(icurrent_player) == a->int_attrib("player");
-}
-
 void player::SetCurrentPlayer(unsigned iplayer) 
 {
     if (iplayer >= players.size())
-        Log << px::strf("SetCurrentPlayer: no such player %d\n", iplayer);
+        Log << ecl::strf("SetCurrentPlayer: no such player %d\n", iplayer);
     else {
         icurrent_player = iplayer;
-        UpdateInventory (GetInventory(iplayer));
+        RedrawInventory (GetInventory(iplayer));
     }
 }
 
@@ -400,8 +277,9 @@ unsigned player::NumberOfRealPlayers() {
 }
 
 /*! Sets respawn positions for black or white actors. */
-void player::SetRespawnPositions(GridPos pos, bool black) {
-    px::V2 center = pos.center();
+void player::SetRespawnPositions(GridPos pos, bool black) 
+{
+    ecl::V2 center = pos.center();
 
     for (unsigned i=0; i<players.size(); ++i) {
         vector<Actor *> &al = players[i].actors;
@@ -469,21 +347,40 @@ void player::AddActor (unsigned iplayer, Actor *a)
     }
 }
 
-void player::SwapPlayers() {
+bool player::HasActor(unsigned iplayer, Actor *a) {
+    if (iplayer >= 0 && iplayer < players.size()) {
+        for (int i = 0; i < players[iplayer].actors.size(); i++) {
+            if (players[iplayer].actors[i] == a)
+                return true;
+        }
+    }
+    return false;
+}
+
+void player::SwapPlayers() 
+{
     if (NumberOfRealPlayers() >= 2) {
         SetCurrentPlayer(1-icurrent_player);
     }
 }
 
-static bool has_extralive(unsigned pl) {
-    return players[pl].inventory.find("it-extralife") != -1;
+static bool has_extralive(unsigned pl, unsigned num) {
+    size_t idx = 0;
+    for (int i = 0; i < num; i++) {
+        int idxLife = players[pl].inventory.find("it-extralife", idx);
+        if (idxLife == -1)
+            return false;
+        else
+            idx = idxLife + 1;
+    }
+    return true;
 }
 
 static bool resurrect_actor (unsigned pl, Actor *a) 
 {
     assert(server::ConserveLevel); // no resurrection otherwise!
 
-    bool has_life = has_extralive(pl);
+    bool has_life = has_extralive(pl, 1);
     if (has_life)
         leveldat.resurrect_actor(a); // = resurrect with delay
     else
@@ -491,95 +388,117 @@ static bool resurrect_actor (unsigned pl, Actor *a)
     return has_life;
 }
 
+void player::AddUnassignedActor (Actor *a) {
+    unassignedActors.push_back(a);
+}
+
 static void CheckDeadActors() 
 {
     bool           toggle_player    = false;
     const unsigned NO_PLAYER        = UINT_MAX;
     unsigned       toggle_to_player = NO_PLAYER;
     bool           new_game         = false; // complete restart (new lives)
-    bool           reset_level      = false; // reset items etc.
 
-    if (server::ConserveLevel) {
-        // to live means to be not dead and to be able to move
-        for (unsigned pl = 0; pl<players.size(); ++pl) {
-            vector<Actor*>& actors = players[pl].actors;
-            bool                     has_living_actor = false;
-            int                      dead_essentials = 0;
+    // to live means to be not dead and to be able to move
+    for (int pl = -1; pl<(int)players.size(); ++pl) {  // -1 are unassigned actors
+        vector<Actor*>& actors  = (pl == -1) ? unassignedActors : players[pl].actors;
+        bool            has_living_actor = false;  // controllable and living
+        std::map<std::string, int> essMap; 
+        std::map<std::string, int>::iterator itEss; 
 
-            for (size_t i=0; i<actors.size(); ++i) {
-                Actor *a = actors[i];
+        for (size_t i=0; i<actors.size(); ++i) {
+            Actor *a = actors[i];
+            std::string essId;
+            if (!a->string_attrib ("essential_id", &essId))
+                essId = a->get_traits().name;
+            int essential = a->int_attrib("essential");
+            // count number of necessary actors per kind
+            if (essential == 1)
+                --essMap[essId];
 
-                if (!a->is_dead() || resurrect_actor(pl, a)) {
-                    if (a->get_mouseforce() != 0.0)
-                        has_living_actor = true;
+            if (!a->is_dead() ||
+                    (pl >= 0 && server::ConserveLevel && resurrect_actor(pl, a))) {
+                // actor is still alive
+                if (pl >= 0 && a->controlled_by(pl) && a->get_mouseforce() != 0.0) {
+                    has_living_actor = true;
                 }
-                else {
-                    // player is dead and could not resurrect
-                    int essential = a->int_attrib("essential");
-                    if (essential>0) {
-                        dead_essentials++;
-                        if (dead_essentials >= essential)
-                            new_game = true;
-                    }
-                }
-            }
-
-            if (has_living_actor) {
-                if (toggle_to_player == NO_PLAYER)
-                    toggle_to_player = pl;
+                // count number of alive actors per kind
+                if (essential >= 0)
+                    ++essMap[essId];
             }
             else {
-                if (pl == icurrent_player)
-                    toggle_player = true;
+                // player is dead and could not resurrect
+                if (essential == -1) {
+                    // actor is personnally essential but dead
+                    new_game = true;
+                }
             }
         }
+        // check if for any kind we have less living actors as required
+        for (itEss = essMap.begin(); itEss != essMap.end(); itEss++) {
+            if ((*itEss).second < 0)
+                new_game = true;
+        }
 
-        // if no_living_player -> toggle_player is true and toggle_to_player is NO_PLAYER
-        // => new_game is set to true below
+        if (has_living_actor) {
+            if (toggle_to_player == NO_PLAYER)
+                toggle_to_player = pl;
+        }
+        else {
+            if (pl == icurrent_player)
+                // check if player has yinyang for single gamer mode
+                if (player::GetInventory(pl)->find("it-yinyang",0) >= 0)
+                    toggle_player = true;
+                else
+                    new_game = true;
+        }
     }
-    else {                  // ConserveLevel == false
+
+    // if no_living_player -> toggle_player is true and toggle_to_player is NO_PLAYER
+    // => new_game is set to true below
+
+    if ((server::ConserveLevel == false) && !server::IsRestartingLevel() && 
+            (new_game || (toggle_player && toggle_to_player == NO_PLAYER))) {
+        // check if we have enough extralives for a restart instead of new game
+        std::vector<int> numDead;
+        bool reset_level = true;
         for (unsigned pl = 0; pl<players.size(); ++pl) {
-            vector<Actor*>& actors         = players[pl].actors;
-            bool            has_living_actor = false;
+            numDead.push_back(0);
+            vector<Actor*>& actors = players[pl].actors;
             for (size_t i = 0; i<actors.size(); ++i) {
                 Actor *a = actors[i];
                 if (a->is_dead()) {
-                    if (has_extralive(pl)) {
-                        leveldat.remove_extralife(a);
-                        reset_level = true;
-                    }
-                    else {
-                        players[pl].out_of_lives = true;
-                        if (pl == icurrent_player)
-                            toggle_player = true;
-                    }
-                    break;
+                    numDead[pl]++;
                 }
-                else
-                    has_living_actor = true;
-
             }
-
-            if (has_living_actor) {
-                if (toggle_to_player == NO_PLAYER)
-                    toggle_to_player = pl;
+            if (!has_extralive(pl, numDead[pl])) {
+                reset_level = false;
+                break;
             }
         }
+        if (reset_level) {
+            for (unsigned pl = 0; pl<players.size(); ++pl) {
+                Inventory *inv = player::GetInventory(pl);
+                for (int i=0; i<numDead[pl]; i++) {
+                    int idx = inv->find("it-extralife");
+                    ASSERT (idx != -1, XLevelRuntime, "Missing extralife for restart of level");
+                    delete inv->yield_item(idx);
+                }
+            }
+            server::RestartLevel(); // should restart w/o scrolling
+            return;
+        }
     }
 
-    if (reset_level) {
-        server::RestartLevel(); // should restart w/o scrolling
-    }
-    else if (new_game) {
+    if (new_game) {
         server::Msg_RestartGame();
     }
     else if (toggle_player) {
         if (toggle_to_player == NO_PLAYER)
             server::Msg_RestartGame();
         else
-            SetCurrentPlayer(toggle_to_player);
+            player::SetCurrentPlayer(toggle_to_player);
     }
-
 }
 
 Actor *player::GetMainActor (unsigned iplayer) 
@@ -631,12 +550,13 @@ Inventory *player::MayPickup(Actor *a)
     return dont_pickup ? 0 : inv;
 }
 
-void player::PickupItem(Actor *a, GridPos p) 
+void player::PickupItem (Actor *a, GridPos p) 
 {
     if (Inventory *inv = MayPickup(a)) {
         if (Item *item = world::YieldItem(p)) {
             item->on_pickup(a);
             inv->add_item(item);
+            RedrawInventory (inv);
             sound::SoundEvent ("pickup", p.center());
         }
     }
@@ -653,23 +573,81 @@ void player::PickupStoneAsItem (Actor *a, enigma::GridPos p)
                 kind[0] = 'i';
 
             if (Item *item = world::MakeItem(kind.c_str())) {
+                KillRubberBands(stone);
                 world::DisposeObject (stone);
                 inv->add_item(item);
+                player::RedrawInventory(inv);
                 sound::SoundEvent ("pickup", p.center());
             }
         }
     }
 }
 
-void player::ActivateItem() {
+void player::ActivateFirstItem() 
+{
     Inventory &inv = players[icurrent_player].inventory;
-    inv.activate_first();
+
+
+    if (inv.size() > 0) {
+        Item *it = inv.get_item (0);
+        world::Actor *ac = 0;
+        GridPos p;
+        bool can_drop_item = false;
+        if (!players[icurrent_player].actors.empty()) {
+            ac = players[icurrent_player].actors[0];
+            p = GridPos(ac->get_pos());
+            can_drop_item = ac->can_drop_items();
+        }
+
+        switch (it->activate(ac, p)) {
+        case world::ITEM_DROP:
+            // only drop if no item underneath and actor allows it
+            if (it->can_drop_at(p) && can_drop_item) {
+                it = inv.yield_first ();
+                RedrawInventory (&inv);
+                it->drop(ac, p);
+            }
+            break;
+        case world::ITEM_KILL:
+            DisposeObject (inv.yield_first ());
+            RedrawInventory (&inv);
+            break;
+        case world::ITEM_KEEP:
+            break;
+        }
+    }
 }
 
 void player::RotateInventory(int dir) 
 {
-    sound::SoundEvent ("invrotate", px::V2());
+    sound::SoundEvent ("invrotate", ecl::V2());
     Inventory &inv = players[icurrent_player].inventory;
-    inv.rotate(dir);
+    if (dir == 1)
+        inv.rotate_left ();
+    else
+        inv.rotate_right ();
+    RedrawInventory (&inv);
+}
+
+
+/** Update the specified inventory on the screen, provided it is the
+    inventory of the current player.  For all other inventories, this
+    function does nothing. */
+void player::RedrawInventory (Inventory *inv)
+{
+    if (inv == GetInventory (CurrentPlayer())) 
+        RedrawInventory();
+}
+
+
+void player::RedrawInventory()
+{
+    Inventory *inv = GetInventory (CurrentPlayer());
+    std::vector<std::string> modelnames;
+    for (size_t i=0; i<inv->size(); ++i) {
+        world::Item *it = inv->get_item(i);
+        modelnames.push_back(it->get_inventory_model());
+    }
+    STATUSBAR->set_inventory (modelnames);
 }
 
