@@ -1,11 +1,11 @@
 /*
- * Copyright (C) 2004 Daniel Heck
+ * Copyright (C) 2004,2005 Daniel Heck
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -13,20 +13,31 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: server.cc,v 1.7 2004/04/24 11:46:51 dheck Exp $
  */
 #include "server.hh"
-#include "player.hh"
-#include "world.hh"
+
+#include "errors.hh"
+#include "game.hh"
 #include "actors.hh"
-#include "player.hh"
-#include "lua.hh"
 #include "client.hh"
-#include "options.hh"
-#include "nls.hh"
+#include "lua.hh"
 #include "main.hh"
+#include "nls.hh"
+#include "options.hh"
+#include "player.hh"
+#include "player.hh"
+#include "StateManager.hh"
+#include "world.hh"
+
+#include "enet/enet.h"
+
+#ifdef WIN32
+// SendMessage is a Syscall on Windows, so we simply undefine it to use this
+// name for one of the methods
+#undef SendMessage
+#endif
 
 #include <cctype>
 
@@ -34,7 +45,7 @@ using namespace enigma::server;
 using namespace world;
 using namespace std;
 
-namespace
+namespace enigma_server
 {
     enum ServerState {
         sv_idle,
@@ -42,6 +53,7 @@ namespace
         sv_running,
         sv_paused,
         sv_restart_level,
+        sv_restart_game,
         sv_finished,
     };
 
@@ -53,19 +65,20 @@ namespace
 
         static Server *instance;
     };
+
+    void PrepareLevel();
+
 }
 
 
 /* -------------------- Global variables -------------------- */
 
-levels::LevelPack   *server::CurrentLevelPack = 0;
-unsigned             server::CurrentLevel     = 0;
 
 bool           server::CreatingPreview = false;
 
 bool server::NoCollisions = false;
 
-
+double   server::LevelTime;
 bool     server::ConserveLevel;
 bool     server::TwoPlayerGame;
 bool     server::SingleComputerGame;
@@ -95,55 +108,79 @@ namespace
     ServerState        state               = sv_idle;
     double             time_accu           = 0;
     double             current_state_dtime = 0;
-    levels::LevelPack *levelpack           = 0;
     int                 move_counter; // counts movements of stones
-
+    ENetAddress        network_address;
+    ENetHost           *network_host       = 0;
 }
 
-void load_level (unsigned ilevel)
+void load_level(lev::Proxy *levelProxy, bool isRestart)
 {
-    if (levelpack) {
-        const levels::LevelInfo &info = levelpack->get_info(ilevel);
+    server::PrepareLevel();
 
+    try {
         // first set default compatibility mode
-        // (may be overidden by load_level (from lua))
-        server::GameCompatibility = info.type;
+        // (may be overidden by load_level (from Lua))
+        server::GameCompatibility = levelProxy->getEngineCompatibility();
+        if (server::GameCompatibility == GAMET_UNKNOWN)
+            // use default Comopatibility
+            // server::GameCompatibility = GAMET_ENIGMA;
+            throw XLevelLoading("unknown engine compatibility");
+        
+        // clear inventory before level load and give us 2 extralives
+        player::NewGame(isRestart);
 
-        player::NewGame();
-//2, levelpack->needs_twoplayers()); // two virtual players
-        try {
-            levelpack->load_level (ilevel);
-            server::CurrentLevel = ilevel;
-            state = sv_waiting_for_clients;
-            game::ResetGameTimer();
+        levelProxy->loadLevel();
+//            server::CurrentLevel = static_cast<unsigned> (ilevel);
+        game::ResetGameTimer();
 
-            if (!CreatingPreview) {
-                player::LevelLoaded();
-                client::Msg_LevelLoaded(ilevel);
-            }
-        }
-        catch (levels::XLevelLoading &err) {
-            client::Msg_Error (_("Server Error: could not load level '")
-                               + info.filename + "'\n"
-                               + err.what());
-            state = sv_idle;
+        world::InitWorld();
+        if (!CreatingPreview) {
+                player::LevelLoaded(isRestart);
+                client::Msg_LevelLoaded(isRestart);
         }
     }
-    else {
-        // XXX server error: no level pack selected
-        state = sv_idle;
+    catch (XLevelLoading &err) {
+        std::string levelPathString = 
+            (levelProxy->getNormPathType() == lev::Proxy::pt_resource) ?
+            levelProxy->getAbsLevelPath() : levelProxy->getNormLevelPath();
+        std::string msg = _("Server Error: could not load level '")
+                               + levelPathString + "'\n"
+                               + err.what();
+        if (!CreatingPreview) {
+            client::Msg_Error(msg);
+            state = sv_idle;
+        } else {
+            throw XLevelLoading(msg);
+        }
+    }
+    catch (XLevelRuntime &err) {
+        std::string levelPathString = 
+            (levelProxy->getNormPathType() == lev::Proxy::pt_resource) ?
+            levelProxy->getAbsLevelPath() : levelProxy->getNormLevelPath();
+        std::string msg = _("Server Error: could not load level '")
+                               + levelPathString + "'\n"
+                               + err.what();
+        if (!CreatingPreview) {
+            client::Msg_Error(msg);
+            state = sv_idle;
+        } else {
+            throw XLevelLoading(msg);
+        }
     }
 }
 
+
 void server::RaiseError (const std::string &msg)
 {
-    throw levels::XLevelLoading (msg);
+    throw XLevelLoading (msg);
 }
 
 void gametick(double dtime)
 {
     const double timestep = 0.01; // 10ms
 
+    server::LevelTime += dtime;
+
     time_accu += dtime;
     if (time_accu > 1.0) {
         fprintf (stderr, "Whoa, system overload!\n");
@@ -152,13 +189,12 @@ void gametick(double dtime)
     player::Tick (time_accu);
     for (;time_accu >= timestep; time_accu -= timestep) {
         world::Tick (timestep);
-//         player::Tick (timestep);
-        if (lua::CallFunc (lua::LevelState(), "Tick", timestep) != 0) {
-            throw enigma_levels::XLevelRuntime(string("Calling 'Tick' failed:\n")+lua::LastError(lua::LevelState()));
+        if (lua::CallFunc (lua::LevelState(), "Tick", timestep, NULL) != 0) {
+            throw XLevelRuntime (string("Calling 'Tick' failed:\n")
+                                                + lua::LastError(lua::LevelState()));
         }
     }
     world::TickFinished ();
-    
 }
 
 
@@ -171,16 +207,30 @@ void server::Init()
 void server::Shutdown()
 {
     lua::ShutdownLevel();
+    if (network_host != 0)
+        enet_host_destroy (network_host);
 }
 
-namespace 
+
+void server::InitNewGame()
 {
+    PrepareLevel();
 }
 
-void server::GameReset()
+bool server::NetworkStart()
 {
+    return true;
+}
+
+
+
+void server::PrepareLevel()
+{
+    state = sv_waiting_for_clients;
+
     server::NoCollisions = false;
 
+    server::LevelTime         = 0.0;
     server::ConserveLevel     = true;
     server::TwoPlayerGame     = false;
     server::SingleComputerGame= true;
@@ -203,14 +253,22 @@ void server::GameReset()
 
     move_counter = 0;
 
+    world::PrepareLevel ();
+    player::PrepareLevel();
+
     /* Restart the Lua environment so symbol definitions from
        different levels do not get in each other's way.*/
     lua::ShutdownLevel();
     lua_State *L = lua::InitLevel();
-    if (lua::Dofile(L, "init.lua") != 0) {
-        throw levels::XLevelLoading("While processing 'init.lua':\n"+lua::LastError(L));
+    if (lua::DoSysFile(L, "compat.lua") != lua::NO_LUAERROR) {
+        throw XLevelLoading("While processing 'compat.lua':\n"+lua::LastError(L));
+    }
+    if (lua::DoSysFile(L, "init.lua") != lua::NO_LUAERROR) {
+        throw XLevelLoading("While processing 'init.lua':\n"+lua::LastError(L));
+    }
+    if (lua::DoSysFile(L, "security.lua") != lua::NO_LUAERROR) {
+        throw XLevelLoading("While processing 'security.lua':\n"+lua::LastError(L));
     }
-
 }
 
 void server::RestartLevel() 
@@ -221,10 +279,14 @@ void server::RestartLevel()
     }
 }
 
+bool server::IsRestartingLevel() {
+    return state == sv_restart_level;
+}
+
 void server::Msg_RestartGame() 
 {
     if (state == sv_running || state == sv_finished) {
-        state = sv_restart_level;
+        state = sv_restart_game;
         current_state_dtime = 0;
     }
 }
@@ -256,9 +318,11 @@ void server::Tick (double dtime)
         gametick (dtime);
         break;
     case sv_restart_level:
+    case sv_restart_game:
         current_state_dtime += dtime;
         if (current_state_dtime >= 1.0) {
-            load_level(CurrentLevel);
+            lev::Index *ind = lev::Index::getCurrentIndex();
+            load_level(ind->getCurrent(), (state == sv_restart_level));
         } else {
             gametick(dtime);
         }
@@ -268,33 +332,29 @@ void server::Tick (double dtime)
         if (current_state_dtime <= 2.5)
             gametick(dtime);
         else {
-            client::Msg_AdvanceLevel (levels::advance_normal);
+            client::Msg_AdvanceLevel (lev::ADVANCE_NEXT_MODE);
             state = sv_waiting_for_clients;
         }
         break;
     }
 }
 
-void server::Msg_SetLevelPack (const std::string &name)
-{
-    levelpack = levels::FindLevelPack (name);
-    if (levelpack == 0) {
-        // XXX server error: invalid level pack
-    }
+void server::Msg_SetLevelPack (const std::string &name) {
+    lev::Index::setCurrentIndex(name);
 }
 
-void server::Msg_LoadLevel (unsigned ilevel)
-{
-    load_level(ilevel);
+void server::Msg_LoadLevel (lev::Proxy *levelProxy, bool isPreview) {
+    server::CreatingPreview = isPreview;
+    load_level(levelProxy, false);
 }
 
 
-void server::Msg_StartGame() 
+void server::Msg_StartGame()
 {
     if (state == sv_waiting_for_clients) {
         time_accu = 0;
         state = sv_running;
-    } 
+    }
     else {
         // Warning << "server: Received unexpected StartGame message.\n";
         // XXX discard message if not waiting for it?
@@ -302,115 +362,68 @@ void server::Msg_StartGame()
 }
 
 namespace enigma_server {
-    using levels::LevelInfo;
-    
-    class GlobalLevelIter {     // iterates repeatedly through ALL levels
-        int levelnr, packnr;
-        int levels, packs;
-
-    public:
-        GlobalLevelIter(LevelPack *lp, int ilevel)
-            : levelnr(ilevel),
-              packnr(0),
-              levels(lp->size()), 
-              packs(enigma_levels::LevelPacks.size())
-        {
-            while (packnr<packs && enigma_levels::LevelPacks[packnr] != lp)
-                ++packnr;
-        }
-
-        GlobalLevelIter& operator++() {
-            if (levelnr<(levels-1))
-                ++levelnr;
-            else {
-                levelnr  = 0;
-                packnr = (packnr+1)%packs;
-                levels = enigma_levels::LevelPacks[packnr]->size();
-            }
-            return *this;
-        }
-
-        LevelPack *getPack() const { return enigma_levels::LevelPacks[packnr]; }
-        int getLevel() const { return levelnr; }
-        const levels::LevelInfo& getInfo() const { return getPack()->get_info(getLevel()); }
-    };
-
-    void Msg_Jumpto(LevelPack *lp, int ilevel) {
-        CurrentLevelPack = lp;
-        CurrentLevel = ilevel; // XXX
-
-        Msg_SetLevelPack (lp->get_name());
-        Msg_LoadLevel (ilevel);
-
-        Msg_Command("restart");
-    }
 
     void Msg_Command_jumpto(const string& dest) {
         // global level jump
-        // e.g.:  dest = "7,33" -> jump to level #33 of 7th levelpack
-        // note: pack and level counter start at 1 (not at 0)
+        // e.g.:  dest = "Enigma IV,33" -> jump to level #33 of index "Enigma IV"
+        // note: level counter start at 1 (not at 0)
 
         size_t comma = dest.find_first_of(',');
         string error;
 
         if (comma != string::npos) {
-            int packnr = atoi(dest.c_str())-1;
-            int packs  = enigma_levels::LevelPacks.size();
+            std::string name = dest.substr(0, comma);
+            Log << "Jumpto '" << name <<"'\n";
             int ilevel = atoi(dest.c_str()+comma+1)-1;
-
-            if (packnr >= 0 && packnr < packs) {
-                LevelPack *lp = enigma_levels::LevelPacks[packnr];
-
-                if (ilevel >= 0 && static_cast<unsigned>(ilevel) < lp->size()) {
-                    Msg_Jumpto(lp, ilevel);
+            
+            if (lev::Index::setCurrentIndex(name)) {
+                
+                if (ilevel >= 0 && ilevel < lev::Index::getCurrentIndex()->size()) {
+                    lev::Index * curInd = lev::Index::getCurrentIndex();
+                    curInd->setCurrentPosition(ilevel);
+                    Msg_LoadLevel (curInd->getProxy(ilevel), false);
+                    Msg_Command("restart");
+                } else {
+                    error = ecl::strf("Illegal level %i (1-%i)", ilevel+1, lev::Index::getCurrentIndex()->size());
+                    // May be we want to reset the current index ?
                 }
-                else error = px::strf("Illegal level %i (1-%i)", ilevel+1, lp->size());
-            }
-            else error = px::strf("Illegal level pack %i (1-%i)", packnr+1, packs);
+            } else error = ecl::strf("Illegal level pack %s", name.c_str());
+        } else if (lev::Index::setCurrentIndex(dest)) {
+            lev::Index * curInd = lev::Index::getCurrentIndex();
+            Msg_LoadLevel (curInd->getCurrent(), false);
+            Msg_Command("restart");
+        } else {
+            error = "Syntax: jumpto pack[,level]";
         }
-        else error = "Syntax: jumpto pack,level";
 
         if (!error.empty()) client::Msg_ShowText(error, false, 2);
     }
 
     void Msg_Command_find(const string& text) {
-        struct LowerCaseString {
-            string low;
-            LowerCaseString(const string& s) : low(s) {
-                for (string::iterator i = low.begin(); i != low.end(); ++i) 
-                    *i = tolower(*i);
+        std::string indName = lev::Proxy::search(text);
+        if (!indName.empty()) {
+            lev::Index::setCurrentIndex(indName);
+            lev::Index * searchResult = lev::Index::getCurrentIndex();
+            searchResult->setCurrentPosition(0);
+            if (searchResult->size() == 1) {
+                // play unique level directly 
+                Msg_LoadLevel (searchResult->getProxy(0), false);
+                Msg_Command("restart");
+            } else {
+                // display multiple levels as pack
+                Msg_Command("abort");
             }
-            bool containedBy(LowerCaseString other) const {
-                return other.low.find(low) != string::npos;
-            }
-        };
-
-        LowerCaseString  lowtext(text);
-        GlobalLevelIter  liter(CurrentLevelPack, CurrentLevel);
-        const LevelInfo *start = &liter.getInfo();
-        ++liter;
-
-        for (const LevelInfo *info = &liter.getInfo();
-             info != start;
-             info = &(++liter).getInfo())
-        {
-            if (lowtext.containedBy(info->filename) ||
-                lowtext.containedBy(info->name) ||
-                lowtext.containedBy(info->author))
-            {
-                Msg_Jumpto(liter.getPack(), liter.getLevel());
-                info = start = 0;
-                break;
-            }
-        }
-
-        if (start)
+        } else {
             client::Msg_ShowText(string("Couldn't find '")+text+'\'', false, 2);
+        }
     }
 };
 
 void server::Msg_Command (const string &cmd)
 {
+    lev::Index *ind = lev::Index::getCurrentIndex();
+    lev::Proxy *curProxy = ind->getCurrent();
+    
     // ------------------------------ normal commands
     if (cmd == "invrotate") {
         player::RotateInventory();
@@ -420,13 +433,13 @@ void server::Msg_Command (const string &cmd)
     }
     else if (cmd == "restart") {
         player::Suicide();
-        server::RestartLevel();
+        server::Msg_RestartGame();
     }
     else if (cmd == "abort") {
         client::Msg_Command(cmd);
     }
-    
-    // ------------------------------ cheats    
+
+    // ------------------------------ cheats
     else if (cmd == "god") {
         SendMessage (player::GetMainActor (player::CurrentPlayer()),
                      "shield");
@@ -443,11 +456,10 @@ void server::Msg_Command (const string &cmd)
 
     // ------------------------------ quick options
     else if (cmd == "easy") {
-        if (options::GetInt("Difficulty") == DIFFICULTY_HARD) {
-            Level level (server::CurrentLevelPack, server::CurrentLevel);
-            if (level.get_info().has_easymode) {
+        if (app.state->getInt("Difficulty") == DIFFICULTY_HARD) {
+            if (curProxy->hasEasymode()) {
                 client::Msg_ShowText("Restarting in easy mode", false, 2);
-                options::SetOption("Difficulty", DIFFICULTY_EASY);
+                app.state->setProperty("Difficulty", DIFFICULTY_EASY);
                 server::Msg_Command("restart");
             }
             else
@@ -457,10 +469,9 @@ void server::Msg_Command (const string &cmd)
             client::Msg_ShowText("Already in easy mode.", false, 2);
     }
     else if (cmd == "noeasy") {
-        if (options::GetInt("Difficulty") == DIFFICULTY_EASY) {
-            Level level (server::CurrentLevelPack, server::CurrentLevel);
-            options::SetOption("Difficulty", DIFFICULTY_HARD);
-            if (level.get_info().has_easymode) {
+        if (app.state->getInt("Difficulty") == DIFFICULTY_EASY) {
+            app.state->setProperty("Difficulty", DIFFICULTY_HARD);
+            if (curProxy->hasEasymode()) {
                 client::Msg_ShowText("Restarting in normal mode", false, 2);
                 server::Msg_Command("restart");
             }
@@ -490,12 +501,11 @@ void server::Msg_Command (const string &cmd)
             client::Msg_ShowText("Already in easy-going mode.", false, 2);
     }
     else if (cmd == "info") {
-        const LevelInfo &info = CurrentLevelPack->get_info(CurrentLevel);
         string infotext       = 
-            px::strf("Level #%i of '", CurrentLevel+1) + CurrentLevelPack->get_name()
-            + "' (" + info.filename + ".lua)  -  \"" + info.name + "\" by " + info.author
-            + px::strf(" (rev=%i)", info.revision);
-
+            ecl::strf("Level #%i of '", ind->getCurrentLevel()) + ind->getName()
+            + "' (" + curProxy->getAbsLevelPath() + ")  -  \"" + curProxy->getTitle() + "\" by " + curProxy->getAuthor()
+            + ecl::strf(" (rev=%i)", curProxy->getReleaseVersion());
+            
         client::Msg_ShowText(infotext, true);
     }
     else if (cmd.substr(0, 5) == "find ") { // global level-jump
@@ -533,34 +543,19 @@ void server::Msg_Panic (bool onoff) {
         state = sv_running;
 }
 
-void server::Msg_MouseForce (const px::V2 &f) {
+void server::Msg_MouseForce (const ecl::V2 &f) {
     world::SetMouseForce (f);
 }
 
 
 void server::SetCompatibility(const char *version) {
-    static const char *versionName[GAMET_COUNT+1] = {
-        "enigma", // same indices as enum GameType
-        "oxyd1",
-        "per.oxyd",
-        "oxyd.extra",
-        "oxyd.magnum",
-        0
-    };
-
-    GameType type = GAMET_UNKNOWN;
-    for (int v = 0; v<GAMET_COUNT; ++v) {
-        if (0 == strcmp(version, versionName[v])) {
-            type = GameType(v);
-            break;
-        }
-    }
+    GameType type = GetGameType(version);
 
     if (type == GAMET_UNKNOWN) {
         fprintf(stderr, "Invalid compatibility mode '%s' (ignored. using enigma behavior)\n", version);
         fprintf(stderr, "Valid modes:");
         for (int v = 0; v<GAMET_COUNT; ++v)
-            fprintf(stderr, " %s", versionName[v]);
+            fprintf(stderr, " %s", GetGameTypeName((GameType)v).c_str());
         fprintf(stderr, "\n");
         type = GAMET_ENIGMA;
     }
@@ -571,8 +566,8 @@ void server::SetCompatibility(const char *version) {
 
 enigma::Difficulty server::GetDifficulty()
 {
-    int i= options::GetInt ("Difficulty");
-    if (i == DIFFICULTY_EASY)
+    int i= app.state->getInt ("Difficulty");
+    if (i == DIFFICULTY_EASY && !server::CreatingPreview)
         return DIFFICULTY_EASY;
     else
         return DIFFICULTY_HARD;
@@ -596,5 +591,5 @@ int server::GetMoveCounter()
 
 void server::Msg_ActivateItem()
 {
-    player::ActivateItem();
+    player::ActivateFirstItem();
 }
