@@ -13,15 +13,15 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: st_switches.cc,v 1.5 2004/03/15 20:15:34 dheck Exp $
  */
+
+#include "errors.hh"
 #include "laser.hh"
 #include "player.hh"
 #include "server.hh"
-
-#include <cassert>
+#include "Inventory.hh"
 
 #include "stones_internal.hh"
 
@@ -225,7 +225,7 @@ void CoinSlot::animcb() {
 
 void CoinSlot::tick(double dtime)
 {
-    assert(remaining_time > 0);
+    ASSERT(remaining_time > 0, XLevelRuntime, "CoinSlot: tick called, but no remaining time");
 
     remaining_time -= dtime;
     if (remaining_time <= 0)
@@ -234,7 +234,7 @@ void CoinSlot::tick(double dtime)
 
 void CoinSlot::actor_hit(const StoneContact &sc)
 {
-    if (player::Inventory *inv = player::GetInventory(sc.actor)) {
+    if (enigma::Inventory *inv = player::GetInventory(sc.actor)) {
         if (Item *it = inv->get_item (0)) {
             ItemID id = get_id(it);
             if (id == it_coin1 || id == it_coin2 || id == it_coin4) {
@@ -246,6 +246,7 @@ void CoinSlot::actor_hit(const StoneContact &sc)
                 remaining_time += coin_value;
 
                 inv->yield_first();
+                player::RedrawInventory (inv);
                 delete it;
             }
         }
@@ -284,7 +285,7 @@ namespace
         void init_model() {set_model(is_on() ? "st-key1" : "st-key0");}
         void actor_hit(const StoneContact &sc);
         const char *collision_sound() { return "metal"; }
-        bool check_matching_key (player::Inventory *inv);
+        bool check_matching_key (enigma::Inventory *inv);
     public:
         KeyStone(const char *kind="st-key", int keycode=0) 
         : OnOffStone(kind) 
@@ -310,19 +311,20 @@ namespace
     };
 }
 
-bool KeyStone::check_matching_key (player::Inventory *inv)
+bool KeyStone::check_matching_key (enigma::Inventory *inv)
 {
     Item *it = inv->get_item(0);
     int keycode, my_keycode = int_attrib ("keycode");
-    return (it->is_kind("it-key*") 
-            && it->int_attrib("keycode", &keycode) 
+    return (it
+            && it->is_kind("it-key*")
+            && it->int_attrib("keycode", &keycode)
             && my_keycode == keycode);
 }
 
-void KeyStone::actor_hit(const StoneContact &sc) 
+void KeyStone::actor_hit(const StoneContact &sc)
 {
-    player::Inventory *inv = player::GetInventory(sc.actor);
-    if (!inv) 
+    enigma::Inventory *inv = player::GetInventory(sc.actor);
+    if (!inv)
         return;
 
     bool toggle = false;
@@ -337,9 +339,10 @@ void KeyStone::actor_hit(const StoneContact &sc)
             }
         }
         else if (check_matching_key (inv)) {
-            DisposeObject(inv->yield_first());
+            DisposeObject (inv->yield_first());
             toggle = true;
         }
+        player::RedrawInventory (inv);
     }
     else {
         if (check_matching_key (inv))
@@ -408,7 +411,8 @@ namespace
             case EAST:  set_model("st-fourswitch-e"); break;
             case SOUTH: set_model("st-fourswitch-s"); break;
             case WEST:  set_model("st-fourswitch-w"); break;
-            case NODIR: assert(0);
+            case NODIR: ASSERT(0, XLevelRuntime,
+                "FourSwitch: no direction defined (found in init_model)");
             }
         }
 
@@ -416,10 +420,11 @@ namespace
             turn();
         }
 
-        void on_message (const Message &m)
+        virtual Value on_message (const Message &m)
         {
             if (m.message == "signal" || m.message == "trigger")
                 turn();
+            return Value();
         }
 
         const char *collision_sound() { return "metal"; }
@@ -515,7 +520,7 @@ void LaserTimeSwitchBase::change_state(State newstate) {
         PerformAction(this, !inverse());
         if (newstate == TOUCHED) {
             double delay = timer_delay();
-            assert(delay>0.0);
+            ASSERT(delay>0.0, XLevelRuntime, "LaserTimeSwitchBase: delay non-positive");
             GameTimer.set_alarm(this, delay, false);
         }
     }
@@ -526,7 +531,8 @@ void LaserTimeSwitchBase::change_state(State newstate) {
     }
     else {
         // it's not allowed to switch from LIGHTED to TOUCHED
-        assert(!(state == LIGHTED && newstate == TOUCHED));
+       ASSERT(!(state == LIGHTED && newstate == TOUCHED), XLevelRuntime,
+           "LaserTimeSwitchBase: trying to switch from lighted to touched");
     }
     state = newstate;
 }
@@ -594,7 +600,8 @@ const char *LaserTimeSwitch::get_inactive_model() const {
 
 double LaserTimeSwitch::timer_delay() const {
     double delay;
-    if (!double_attrib("delay", &delay)) assert(0);
+    if (!double_attrib("delay", &delay))
+        ASSERT(0, XLevelRuntime, "LaserTimeSwitch: delay not properly defined");
     return delay;
 }
 
@@ -639,25 +646,27 @@ namespace
     };
 }
 
-void FloppyStone::init_model() {
+void FloppyStone::init_model() 
+{
     set_model(is_on() ? "st-floppy1" : "st-floppy0");
 }
 
 void FloppyStone::actor_hit (const StoneContact &sc)
 {
-    if (player::Inventory *inv = player::GetInventory(sc.actor)) {
+    if (enigma::Inventory *inv = player::GetInventory(sc.actor)) {
         if (is_on()) {
             if (!inv->is_full()) {
-                inv->add_item(MakeItem("it-floppy"));
+                inv->add_item (MakeItem("it-floppy"));
                 set_on(false);
                 PerformAction(this, is_on());
             }
         }
-        else if (player::wielded_item_is(sc.actor, "it-floppy")) {
-            DisposeObject(inv->yield_first());
+        else if (player::WieldedItemIs (sc.actor, "it-floppy")) {
+            DisposeObject (inv->yield_first());
             set_on(true);
             PerformAction(this, is_on());
         }
+        player::RedrawInventory (inv);
     }
 }
 
