@@ -13,15 +13,16 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: stones.cc,v 1.29 2004/03/15 20:15:34 dheck Exp $
  */
+
+#include "errors.hh"
 #include "stones_internal.hh"
 #include "server.hh"
 #include "client.hh"
 #include "player.hh"
-#include <cassert>
+#include "Inventory.hh"
 
 using namespace std;
 using namespace world;
@@ -131,7 +132,6 @@ const char *Stone::get_kind() const
         return Object::get_kind();
 }
 
-
 StoneResponse Stone::collision_response(const StoneContact &) {
     return STONE_REBOUND;
 }
@@ -161,12 +161,11 @@ const char * Stone::collision_sound() {
 
    Note: This should be used by on_impulse() to perform a move.
 */
-bool Stone::move_stone(Direction dir) {
+bool Stone::move_stone(GridPos newPos, const char *soundevent) {
     GridPos p      = get_pos();
-    GridPos newPos = move(p, dir);
 
     if (!GetStone(newPos)) {
-        sound_event ("movesmall");
+        sound_event (soundevent);
 
         MoveStone(p, newPos);
         server::IncMoveCounter();
@@ -178,12 +177,40 @@ bool Stone::move_stone(Direction dir) {
     }
     return false;
 }
+bool Stone::move_stone(Direction dir) {
+    return move_stone(move(get_pos(), dir), "movesmall");
+}
 
 void Stone::on_move() {
     if (!is_floating()) 
         ShatterActorsInsideField (get_pos());
 }
 
+/* Multiplies velocity with the attribute-matrix
+   hit_distortion_[xx,xy,yx,yy] and factor hit_factor
+   If components are not set, use ((1,0),(0,1)) as
+   default matrix, resp. defaultfactor as hit_factor. */
+ecl::V2 Stone::distortedVelocity (ecl::V2 vel, double defaultfactor = 1.0) {
+    ecl::V2 newvel(0,0);    
+    if(const Value *xx = this->get_attrib("hit_distortion_xx"))
+        newvel[0] += to_double(*xx) * vel[0];
+    else
+        newvel[0] += vel[0];
+    if(const Value *xy = this->get_attrib("hit_distortion_xy"))
+        newvel[0] += to_double(*xy) * vel[1];
+    if(const Value *yx = this->get_attrib("hit_distortion_yx"))
+        newvel[1] += to_double(*yx) * vel[0];
+    if(const Value *yy = this->get_attrib("hit_distortion_yy"))
+        newvel[1] += to_double(*yy) * vel[1];
+    else
+        newvel[1] += vel[1];
+    if (const Value *factor = this->get_attrib("hit_factor"))
+        newvel *= to_double(*factor);
+    else
+        newvel *= defaultfactor;    
+    return newvel;
+}
+
 
 
 // *******************************************************************************
@@ -274,7 +301,8 @@ namespace
 
 void SpitterStone::animcb() {
     switch (state) {
-    case IDLE: assert(0); 
+    case IDLE:
+        ASSERT(0, XLevelRuntime, "SpitterStone: animcb called with inconsistent state"); 
     case LOADING: {
         Actor     *ball = MakeActor (ac_cannonball);
         ActorInfo *ai   = ball->get_actorinfo();
@@ -298,11 +326,12 @@ void SpitterStone::actor_hit (const StoneContact &sc)
     if (state != IDLE)
         return;
 
-    if (player::Inventory *inv = player::GetInventory(sc.actor)) {
+    if (enigma::Inventory *inv = player::GetInventory(sc.actor)) {
         int lifepos = inv->find("it-extralife");
         if (lifepos != -1) {
             delete inv->yield_item(lifepos);
-            ball_velocity = sc.actor->get_vel();
+            player::RedrawInventory (inv);
+            ball_velocity = distortedVelocity(sc.actor->get_vel(), 1.0);
             state = LOADING;
             set_anim ("st-spitter-loading");
         }
@@ -322,8 +351,10 @@ YieldedGridStone::YieldedGridStone(Stone *st)
 
 YieldedGridStone::~YieldedGridStone() 
 {
-    assert(!stone);
-    assert(!model);
+    ASSERT(!stone, XLevelRuntime,
+        "YieldedGridStone: destructor called though stone still exists");
+    ASSERT(!model, XLevelRuntime,
+        "YieldedGridStone: destructor called though model still exists");
 }
 
 void YieldedGridStone::set_stone(GridPos pos) 
@@ -396,7 +427,7 @@ namespace
 
         void actor_hit (const StoneContact &sc) {
             if (Actor *other = FindOtherMarble(sc.actor)) {
-                other->add_force (20*sc.actor->get_vel());
+                other->add_force (distortedVelocity(sc.actor->get_vel(), 20));
             }
         }
 
@@ -460,6 +491,7 @@ namespace
         CLONEOBJ(BreakingStone);
 
         void init_model() {
+            sound_event("stonedestroy");
             set_anim("st-breaking");
         }
 
@@ -507,10 +539,23 @@ namespace
 
         const char *collision_sound() {return "stone";}
 
-        void message (const string &msg, const Value &) {
+        virtual Value message (const string &msg, const Value &) {
             if (msg == "trigger" || msg == "signal") {
                 ReplaceStone(get_pos(), MakeStone("st-plain_hole"));
             }
+            return Value();
+        }
+        void actor_hit (const StoneContact &sc) {
+            if (player::WieldedItemIs (sc.actor, "it-pencil")) {
+                enigma::Inventory *inv = player::GetInventory(sc.actor);
+                if (inv && inv->size() > 0) {
+                    delete inv->yield_item(0);
+                    player::RedrawInventory(inv);
+                    sound_event("stonepaint");
+                    ReplaceStone(get_pos(), MakeStone("st-firebreak"));
+                }
+            } else
+                Stone::actor_hit(sc);
         }
     public:
         PlainStone() : Stone("st-plain") {}
@@ -519,10 +564,11 @@ namespace
     class PlainStone_Hollow : public Stone {
         CLONEOBJ(PlainStone_Hollow);
 
-        void message (const string &msg, const Value &) {
+        virtual Value message (const string &msg, const Value &) {
             if (msg == "trigger" || msg == "signal") {
                 ReplaceStone(get_pos(), MakeStone("st-plain"));
             }
+            return Value();
         }
 
         StoneResponse collision_response(const StoneContact &) 
@@ -561,12 +607,13 @@ namespace
         void on_laserhit (Direction) {
             break_me();
         }
-        void message (const string &msg, const Value &) {
+        virtual Value message (const string &msg, const Value &) {
             if (msg =="ignite" || msg == "expl" || msg == "bombstone")
                 break_me();
+            return Value();
         }
         void actor_hit (const StoneContact &sc) {
-            if (player::wielded_item_is(sc.actor, "it-hammer")) {
+            if (player::WieldedItemIs (sc.actor, "it-hammer")) {
                 break_me();
             }
         }
@@ -591,14 +638,15 @@ namespace
         }
 
         void actor_hit (const StoneContact &sc) {
-            if (player::wielded_item_is(sc.actor, "it-hammer")) {
+            if (player::WieldedItemIs (sc.actor, "it-hammer")) {
                 break_me();
             }
         }
 
-        void message (const string &msg, const Value &) {
+        virtual Value message (const string &msg, const Value &) {
             if (msg =="ignite" || msg == "expl" || msg == "bombstone")
                 break_me();
+            return Value();
         }
         const char *collision_sound() {return "metal";}
     public:
@@ -629,25 +677,41 @@ namespace
             sound_event ("stonedestroy");
             ReplaceStone(get_pos(), MakeStone ("st-plain_breaking"));
         }
-        void message (const string &msg, const Value &) {
+        virtual Value message (const string &msg, const Value &) {
             if (msg =="ignite" || msg == "expl" || msg == "bombstone")
                 break_me();
+            return Value();
         }
         void on_move() {
+            Stone::on_move();
             GridPos p = get_pos();
             if (Floor *fl = GetFloor (p)) {
-                if (fl->is_kind("fl-abyss"))
-                    ReplaceStone(p, MakeStone("st-plain_falling"));
+                if (fl->is_kind("fl-abyss")) {
+                    ReplaceStone (p, MakeStone("st-plain_falling"));
+                }
                 else if (fl->is_kind("fl-swamp") || fl->is_kind("fl-water")) {
                     sound_event ("drown");
                     client::Msg_Sparkle (p.center());
-                    KillStone(p);
+                    KillStone (p);
                 }
             }
         }
 
         bool is_movable () const { return true; }
 
+        void actor_hit (const StoneContact &sc) {
+            if (player::WieldedItemIs (sc.actor, "it-pencil")) {
+                enigma::Inventory *inv = player::GetInventory(sc.actor);
+                if (inv && inv->size() > 0) {
+                    delete inv->yield_item(0);
+                    player::RedrawInventory(inv);
+                    sound_event("stonepaint");
+                    ReplaceStone(get_pos(), MakeStone("st-firebreak_move"));
+                }
+            } else
+                Stone::actor_hit(sc);
+        }
+
     public:
         PlainStone_Movable() : Stone("st-plain_move") {
         }
@@ -655,12 +719,14 @@ namespace
 }
 
 
+/* -------------------- Black- and Whiteballs Stones -------------------- */
+
 namespace
 {
     class BlackBallsStone : public Stone {
         CLONEOBJ(BlackBallsStone);
 
-        void on_message (const Message &m)
+        virtual Value on_message (const Message &m)
         {
             GridPos p = get_pos();
             Actor *a = world::CurrentCollisionActor;
@@ -678,6 +744,7 @@ namespace
                     SendMessage (GetStone (move (p, SOUTH)), "signal", 1.0);
                 }
             }
+            return Value();
         }
     public:
         BlackBallsStone() : Stone ("st-blackballs") {
@@ -687,7 +754,7 @@ namespace
     class WhiteBallsStone : public Stone {
         CLONEOBJ(WhiteBallsStone);
 
-        void on_message (const Message &m)
+        virtual Value on_message (const Message &m)
         {
             GridPos p = get_pos();
             Actor *a = world::CurrentCollisionActor;
@@ -705,6 +772,7 @@ namespace
                     SendMessage (GetStone (move (p, SOUTH)), "signal", 1.0);
                 }
             }
+            return Value();
         }
 
     public:
@@ -735,9 +803,10 @@ namespace
 
 // --------------------------------------------------------------------------------
 
+extern void InitSwitches();
+
 void stones::Init()
 {
-    extern void InitSwitches();
 
     // Register(new ...);
 
