@@ -13,19 +13,19 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: stones_simple.cc,v 1.71 2004/03/18 18:09:23 dheck Exp $
  */
+
+#include "errors.hh"
 #include "laser.hh"
 #include "server.hh"
 #include "player.hh"
 #include "client.hh"
+#include "Inventory.hh"
 
 #include "stones_internal.hh"
 
-#include <cassert>
-
 using namespace std;
 using namespace world;
 using namespace stones;
@@ -115,7 +115,7 @@ namespace
             return traits->hollow || traits->glass;
         }
 
-        void on_message (const Message &m)
+        virtual Value on_message (const Message &m)
         {
             if (traits->hollow && m.message == "glasses") {
                 if (to_int(m.value)) {
@@ -131,6 +131,7 @@ namespace
                     }
                 }
             }
+            return Value();
         }
 
         const SimpleStoneTraits *traits; // owned by simple_stone_traits
@@ -223,7 +224,7 @@ namespace
         CLONEOBJ(EasyModeStone);
         DECL_TRAITS;
 
-        void message(const std::string &msg, const Value&) {
+        virtual Value message(const std::string &msg, const Value&) {
             if (msg == "init") {
                 if (server::GetDifficulty() == DIFFICULTY_EASY) {
                     SetFloor (get_pos(), MakeFloor ("fl-normal"));
@@ -232,6 +233,7 @@ namespace
                 }
                 KillStone (get_pos());
             }
+            return Value();
         }
     public:
         EasyModeStone() 
@@ -399,7 +401,10 @@ void SwapStone::on_impulse(const Impulse& impulse)
         GridPos oldp = get_pos();
         GridPos newp = move(oldp, impulse.dir);
 
-        if (!IsLevelBorder(newp)) {
+        // never swap beyond the world and for non enigma modes do not swap to
+        // border as well.
+        if (IsInsideLevel(newp) &&
+                (server::GameCompatibility == GAMET_ENIGMA || !IsLevelBorder(newp))) {
             Stone *other = GetStone(newp);
             if (other && other->is_removable()) {
                 SwapStone *newStone = new SwapStone;
@@ -550,9 +555,10 @@ namespace
         void animcb() {
             KillStone(get_pos());
         }
-        void message(const string &msg, const Value &) {
+        virtual Value message(const string &msg, const Value &) {
             if (msg =="ignite" || msg == "expl" || msg == "bombstone")
                 break_me();
+            return Value();
         }
 
         virtual string get_break_anim() const  {
@@ -591,7 +597,7 @@ namespace
         Stone_break(const char *kind) : BreakableStone(kind) { }
     private:
         bool may_be_broken_by(Actor *a) const {
-            return player::wielded_item_is(a, "it-hammer");
+            return player::WieldedItemIs (a, "it-hammer");
         }
     };
 
@@ -635,11 +641,12 @@ namespace
         Break_bolder() : BreakableStone("st-break_bolder") {}
     private:
         bool may_be_broken_by(Actor *a) const {
-            return player::wielded_item_is(a, "it-hammer");
+            return player::WieldedItemIs (a, "it-hammer");
         }
-        virtual void message(const string &msg, const Value &) {
+        virtual Value message(const string &msg, const Value &) {
             if (msg == "trigger")
                 break_me();
+            return Value();
         }
     };
 }
@@ -675,7 +682,7 @@ namespace
             return "st-rock3_break-anim";
         }
         bool may_be_broken_by(Actor *a) const {
-            return player::wielded_item_is(a, "it-hammer");
+            return player::WieldedItemIs (a, "it-hammer");
         }
 
         bool is_movable() const { return true; }
@@ -718,7 +725,7 @@ namespace
     private:
         bool may_be_broken_by(Actor *a) const {
             return a->get_attrib("whiteball") &&
-                player::wielded_item_is(a, "it-hammer");
+                player::WieldedItemIs (a, "it-hammer");
         }
     };
 }
@@ -748,7 +755,7 @@ namespace
     private:
         bool may_be_broken_by(Actor *a) const {
             return a->get_attrib("blackball") &&
-                player::wielded_item_is(a, "it-hammer");
+                player::WieldedItemIs (a, "it-hammer");
         }
     };
 }
@@ -778,7 +785,7 @@ namespace
         BrickMagic() : Stone("st-brick_magic") {}
     private:
         void actor_hit(const StoneContact &sc) {
-            if (player::wielded_item_is(sc.actor, "it-magicwand")) {
+            if (player::WieldedItemIs (sc.actor, "it-magicwand")) {
                 sound_event ("stonepaint");
                 ReplaceStone (get_pos(), MakeStone("st-glass"));
             }
@@ -813,7 +820,7 @@ namespace
 
         void actor_hit(const StoneContact &sc) {
             if( state == INVISIBLE) {
-                if (player::wielded_item_is(sc.actor, "it-brush")) {
+                if (player::WieldedItemIs (sc.actor, "it-brush")) {
                     sound_event ("stonepaint");
                     state = BRUSH;
                     if (server::GameCompatibility == GAMET_PEROXYD) {
@@ -857,14 +864,14 @@ namespace
         State state;
         void actor_hit(const StoneContact &sc) {
             if (state == INVISIBLE) {
-                if (player::wielded_item_is(sc.actor, "it-brush")) {
+                if (player::WieldedItemIs (sc.actor, "it-brush")) {
                     sound_event ("stonepaint");
                     state = BRUSH;
                     set_model("st-stone_break");
                 }
             }
             else if (state == BRUSH) {
-                if (player::wielded_item_is(sc.actor, "it-hammer")) {
+                if (player::WieldedItemIs (sc.actor, "it-hammer")) {
                     sound_event ("stonedestroy");
                     state = DESTROY;
                     set_anim("st-stone_break-anim");
@@ -907,7 +914,7 @@ namespace
 
         void actor_hit(const StoneContact &sc) {
             if (state == INVISIBLE) {
-                if (player::wielded_item_is(sc.actor, "it-magicwand")) {
+                if (player::WieldedItemIs (sc.actor, "it-magicwand")) {
                     sound_event ("stonepaint");
                     state = STONE;
                     set_model("st-greenbrown");
@@ -933,49 +940,62 @@ set_stone("st-wood", 10,10)
 \endverbatim
 
 Note: There are two flavors of st-wood which may be specified
-by using st-wood1 or st-wood2.
+by using st-wood1 or st-wood2, and a two related kinds: st-flrock,
+which creates the unburnable fl-rock and denies fire under it, and 
+the burnable st-hay, which leaves the burnable but stable fl-hay
+behind.
 
 \image html st-wood.png
 */
 namespace
 {
     class WoodenStone : public Stone {
-	CLONEOBJ(WoodenStone);
+        CLONEOBJ(WoodenStone);
     public:
-        WoodenStone(const char *kind) : Stone(kind) {}
+        WoodenStone(const char *kind, const char *floorkind_, bool blockfire_ = false) :
+            Stone(kind), floorkind(floorkind_), blockfire(blockfire_) {}
 
     private:
-        void fall() {
+        const char *floorkind;
+        bool blockfire;
+
+        void maybe_fall_or_stopfire() {
             GridPos p = get_pos();
             if (world::IsLevelBorder(p))
                 return;
-
             if (Floor *fl = GetFloor(p)) {
                 const string &k = fl->get_kind();
-                if (k == "fl-abyss" || k=="fl-water" || k=="fl-swamp") {
-                    SetFloor(p, MakeFloor(is_kind("st-wood1") ? "fl-stwood1" : "fl-stwood2"));
+                if(blockfire)
+                    SendMessage(fl, "stopfire");
+                if (k == "fl-abyss" || k == "fl-water" || k == "fl-swamp") {
+                    SetFloor(p, MakeFloor(floorkind));
                     KillStone(p);
                 }
             }
         }
 
-        void message (const string &msg, const Value &) {
-            if (msg == "fall")
-                fall();
+        virtual Value message (const string &msg, const Value &) {
+            if (msg == "fire" && !blockfire) {
+                KillStone(get_pos());
+                return Value(1.0);  // allow fire to spread
+            } else if (msg == "heat" && blockfire) {
+                return Value(1.0);  // block fire
+            } else if (msg == "fall")
+                maybe_fall_or_stopfire();
+            return Value();
         }
 
         // in oxyd1 only fall when moving
         void on_move() {
+            Stone::on_move();
             if (server::GameCompatibility == GAMET_OXYD1)
-                fall();
-            else
-                Stone::on_move();
+                maybe_fall_or_stopfire();
         }
 
         // other oxyds versions: fall everytime the floor changes
         void on_floor_change() {
             if (server::GameCompatibility != GAMET_OXYD1)
-                fall();
+                maybe_fall_or_stopfire();
         }
         bool is_movable () const { return true; }
     };
@@ -984,12 +1004,15 @@ namespace
       st-wood2. */
     class RandomWoodenStone : public Stone {
     public:
-	RandomWoodenStone() : Stone("st-wood") {}
+        RandomWoodenStone() : Stone("st-wood") {}
     private:
-	Stone *clone() {
-	    return new WoodenStone (IntegerRand(0, 1) ? "st-wood1" :  "st-wood2");
-	}
-	void dispose() {delete this;}
+        Stone *clone() {
+            if(IntegerRand(0,1) == 0)
+                return new WoodenStone("st-wood1", "fl-stwood1");
+            else
+                return new WoodenStone("st-wood2", "fl-stwood2");
+        }
+        void dispose() {delete this;}
     };
 }
 
@@ -1061,8 +1084,8 @@ namespace
         CLONEOBJ(ScissorsStone);
         DECL_TRAITS;
         void actor_hit(const StoneContact &sc) {
-            world::KillRubberBand (sc.actor, (Stone*)0);
-            world::KillRubberBand (sc.actor, (Actor*)0);
+            if (world::KillRubberBands (sc.actor))
+                world::PerformAction(this, false);
             sound_event("scissors");
             set_anim("st-scissors-snip");
         }
@@ -1104,16 +1127,26 @@ namespace
         void actor_hit(const StoneContact &sc) {
             double strength = 10.0;
             double length = 1.0;
-            double_attrib ("length", &length);
+            double minlength = 0.0;
             double_attrib ("strength", &strength);
+            double_attrib ("length", &length);
+            double_attrib ("minlength", &minlength);
 
             world::RubberBandData rbd;
             rbd.strength = strength;
             rbd.length = length;
+            rbd.minlength = minlength;
+
+            // The mode attribute "scissor" defines, if when touching an st-rubberband,
+            // other rubberbands to the actor will be cut of or not, true means they will. true is default.
+            enigma::Value const *scissorValue = get_attrib("scissor");
+            bool isScissor = (scissorValue == NULL)? true : to_bool(*scissorValue);
 
             if (!world::HasRubberBand (sc.actor, this)) {
                 sound_event ("rubberband");
-                world::KillRubberBand (sc.actor, (Stone*)0);
+                if (isScissor) {
+                    world::KillRubberBand (sc.actor, (Stone*)0);
+                }
                 world::AddRubberBand (sc.actor, this, rbd);
             }
 //             if (player::wielded_item_is (sc.actor, "it-magicwand"))
@@ -1122,7 +1155,7 @@ namespace
 
         void on_impulse (const Impulse& impulse) {
             Actor *a = dynamic_cast<Actor *> (impulse.sender);
-            if (a && player::wielded_item_is (a, "it-magicwand"))
+            if (a && player::WieldedItemIs (a, "it-magicwand"))
                 move_stone(impulse.dir);
         }
 
@@ -1187,6 +1220,10 @@ namespace
 
             // set_on(true);   DOESN'T WORK! calls init_model()
         }
+        
+        virtual ~TimerStone() {
+            GameTimer.remove_alarm (this);
+        }
     private:
         int m_signalvalue;
 
@@ -1217,7 +1254,7 @@ namespace
         void alarm() {
             if (is_on()) {
 //                 sound::PlaySound("st-timer");
-                PerformAction(this, m_signalvalue);
+                PerformAction(this, m_signalvalue != 0);
                 m_signalvalue = 1-m_signalvalue;
             }
         }
@@ -1253,17 +1290,19 @@ namespace
 
         enum State { IDLE, FARTING, BREAKING };
         State state;
+        bool rememberBreaking;  // set true if an explosion or break-message
+                                // or such occured while farting
 
         void change_state(State newstate);
         void animcb();
         void actor_hit(const StoneContact &sc);
-        void message(const string &m, const Value &val);
+        virtual Value message(const string &m, const Value &val);
 
         void on_laserhit(Direction) {
             change_state(BREAKING);
         }
     public:
-        FartStone() : state(IDLE) 
+        FartStone() : state(IDLE), rememberBreaking(false) 
         {}
     };
     DEF_TRAITS(FartStone, "st-fart", st_fart);
@@ -1278,6 +1317,8 @@ void FartStone::change_state(State newstate)
     case IDLE:
         state = IDLE;
         init_model();
+        if (rememberBreaking)
+            change_state(BREAKING);
         break;
     case FARTING:
     case BREAKING:
@@ -1292,7 +1333,8 @@ void FartStone::change_state(State newstate)
             else
                 set_anim("st-farting");
             state = newstate;
-        }
+        } else if (state == FARTING && newstate == BREAKING)
+            rememberBreaking = true;
         break;
     }
 }
@@ -1307,12 +1349,12 @@ void FartStone::animcb()
 
 void FartStone::actor_hit(const StoneContact &sc) 
 {
-    if (player::wielded_item_is(sc.actor, "it-hammer"))
+    if (player::WieldedItemIs (sc.actor, "it-hammer"))
         change_state(BREAKING);
     else
         change_state(FARTING);
 }
-void FartStone::message (const string &m, const Value &val) 
+Value FartStone::message (const string &m, const Value &val) 
 {
     if (m == "signal" && to_int(val) != 0)
         change_state(FARTING);
@@ -1320,6 +1362,7 @@ void FartStone::message (const string &m, const Value &val)
         change_state(FARTING);
     else if (m == "ignite" || m == "expl") 
         change_state(BREAKING);
+    return Value();
 }
 
 
@@ -1330,40 +1373,46 @@ namespace
 {
     /*! Steals one item from the player's inventory when hit. */
     class ThiefStone : public Stone {
-	CLONEOBJ(ThiefStone);
+        CLONEOBJ(ThiefStone);
         DECL_TRAITS;
 
-        enum State { IDLE, EMERGING, RETREATING } state;
+        enum State { IDLE, EMERGING, RETREATING, CAPTURED } state;
         Actor *m_affected_actor;
+        Item * bag;
     public:
-	ThiefStone();
+        ThiefStone();
+        virtual ~ThiefStone();
     private:
-	void steal_from_player();
+        void steal_from_player();
 
         void actor_hit(const StoneContact &sc);
         // even a slight touch should steal from the actor: 
         void actor_touch(const StoneContact &sc) { actor_hit(sc); }
         void animcb();
+        virtual Value message(const string &msg, const Value &v);        
 
-        const char *collision_sound() {
-            return "cloth";
-        }
+        const char *collision_sound() { return "cloth"; }
+        int affected_player;
     };
     DEF_TRAITS(ThiefStone, "st-thief", st_thief);
 }
 
 ThiefStone::ThiefStone() 
-: state(IDLE),
-  m_affected_actor (0)
-{
+: state(IDLE), m_affected_actor (0), affected_player (-1), bag(NULL) {}
+
+ThiefStone::~ThiefStone() {
+    if (bag != NULL)
+        delete bag;
 }
 
-
 void ThiefStone::actor_hit(const StoneContact &sc) {
-    if (state==IDLE) {
+    ActorID id = get_id(sc.actor);
+    if (state == IDLE) {
         set_anim("st-thief-emerge");
         state = EMERGING;
         m_affected_actor = sc.actor;
+        affected_player = -1;
+        m_affected_actor->int_attrib("player", &affected_player);
     }
 }
 
@@ -1378,23 +1427,51 @@ void ThiefStone::animcb() {
         state = IDLE;
         init_model();
         break;
+    case CAPTURED:
+        KillStone(get_pos());
+        break;
     default:
-        assert(0);
+        ASSERT(0, XLevelRuntime, "ThiefStone: animcb called with inconsistent state");
     }
 }
 
 void ThiefStone::steal_from_player() 
 {
-    if (m_affected_actor && !m_affected_actor->has_shield()) {
-        player::Inventory *inv = player::GetInventory(m_affected_actor);
+    // the actor that hit the thief may no longer exist!
+    if (m_affected_actor && affected_player >= 0 &&
+            player::HasActor(affected_player, m_affected_actor) && 
+            !m_affected_actor->has_shield()) {
+        enigma::Inventory *inv = player::GetInventory(m_affected_actor);
         if (inv && inv->size() > 0) {
-            int i = IntegerRand (0, inv->size()-1);
-            delete inv->yield_item(i);
+            if (bag == NULL)
+                bag = world::MakeItem(it_bag);
+            int i = IntegerRand (0, int (inv->size()-1));
+            dynamic_cast<ItemHolder *>(bag)->add_item(inv->yield_item(i));
+            player::RedrawInventory (inv);
             sound_event("thief");
         }
     }
 }
 
+Value ThiefStone::message(const string &msg, const Value &v) {
+    if(msg == "capture" && state == IDLE) {
+        state = CAPTURED;
+        Item * it =  world::GetItem(get_pos());
+        
+        // add items on grid pos that can be picked up to our bag
+        if (it != NULL && !(it->get_traits().flags & itf_static) && bag != NULL) {
+            dynamic_cast<ItemHolder *>(bag)->add_item(world::YieldItem(get_pos()));
+        }
+        // drop bag if pos is not occupied by a static item
+        if (world::GetItem(get_pos()) == NULL)
+            world::SetItem(get_pos(), bag);
+        bag = NULL;
+        set_anim(string(get_kind()) + "-captured");
+        return Value(1);
+    } else
+        return Stone::message(msg, v);
+}
+
 // -------------------------
 //      ActorImpulseBase
 // -------------------------
@@ -1417,7 +1494,7 @@ namespace
                 double_attrib("force", &forcefac);
 
                 V2 vec = normalize(sc.actor->get_pos() - get_pos().center());
-                sc.actor->add_force (vec * forcefac);
+                sc.actor->add_force (distortedVelocity(vec, forcefac));                
 
                 sound_event("bumper");
                 set_anim("st-actorimpulse-anim");
@@ -1448,7 +1525,7 @@ namespace
 
         int m_signalidx;
 
-        void message (const string &msg, const Value &) {
+        virtual Value message (const string &msg, const Value &) {
             if (msg == "signal") {
                 world::EmitSignalByIndex (this, m_signalidx, 0);
                 m_signalidx += 1;
@@ -1459,6 +1536,7 @@ namespace
             } else if (msg == "init") {
                 world::EmitSignalByIndex (this, m_signalidx, 1);
             }
+            return Value();
         }
 
     public:
@@ -1474,7 +1552,7 @@ namespace
         ActorImpulseStoneInvisible() : ActorImpulseBase("st-actorimpulse_invisible") {}
 
         void actor_hit(const StoneContact& sc) {
-            if (player::wielded_item_is(sc.actor, "it-brush")) {
+            if (player::WieldedItemIs (sc.actor, "it-brush")) {
                 Stone *st = MakeStone("st-actorimpulse");
                 SetStone(get_pos(), st);
                 st->actor_hit(sc);
@@ -1548,12 +1626,13 @@ namespace
             }
         }
 
-        void on_message (const Message &m) {
+        virtual Value on_message (const Message &m) {
             if (m.message == "signal" || m.message == "trigger") {
                 // toggle between black and white stone
                 m_type = (m_type + 4) % 8;
                 init_model();
             }
+            return Value();
         }
 
         bool is_floating() const { return true; }
@@ -1631,8 +1710,8 @@ namespace
         YinYangStone3() : YinYangStone("st-yinyang3") {}
     private:
         void actor_hit(const StoneContact &sc) {
-            if (player::wielded_item_is(sc.actor, "it-magicwand") ||
-                player::wielded_item_is(sc.actor, "it-brush"))
+            if (player::WieldedItemIs (sc.actor, "it-magicwand") ||
+                player::WieldedItemIs (sc.actor, "it-brush"))
             {
                 if      (sc.actor->get_attrib("blackball")) 
                     turn_white("st-white4");
@@ -1653,37 +1732,55 @@ namespace
         CLONEOBJ(BombStone);
         const char *collision_sound() {return "stone";}
     public:
-        BombStone() : Stone("st-bombs"),state(IDLE) {}
+        BombStone(const char* kind_, const char* itemkind_) :
+            Stone(kind_), state(IDLE), itemkind(itemkind_) {}
     private:
         enum State { IDLE, BREAK };
         State state;
-        void actor_hit(const StoneContact &sc);
-        void change_state (State newstate) {
-            if (state == IDLE && newstate==BREAK) {
-                state = newstate;
-                sound_event ("stonedestroy");
-                set_anim("st-bombs-anim");
-            }
-        }
-        void animcb() {
-            assert(state == BREAK);
-            GridPos p = get_pos();
-            SendExplosionEffect(p, EXPLOSION_BOMBSTONE);
-            KillStone(p);
-            SetItem(p, it_explosion1);
-        }
-        void BombStone::message(const string &msg, const Value &) {
-            if (msg =="expl" || msg =="bombstone")
-                change_state(BREAK);
-        }
+        const char* itemkind;
+        void actor_hit (const StoneContact &sc);
+        void change_state (State newstate);
+        void animcb();
+        virtual Value message (const string &msg, const Value &);
     };
 }
 
-void BombStone::actor_hit(const StoneContact &sc) {
-    if (player::Inventory *inv = player::GetInventory(sc.actor)) {
+void BombStone::change_state (State newstate) 
+{
+    if (state == IDLE && newstate==BREAK) {
+        string model = get_kind();
+        state = newstate;
+        sound_event ("stonedestroy");
+        set_anim(model + "-anim");
+    }
+}
+
+void BombStone::animcb() 
+{
+    ASSERT(state == BREAK, XLevelRuntime, "BombStone: animcb called with inconsistent state");
+    GridPos p = get_pos();
+    SendExplosionEffect(p, EXPLOSION_BOMBSTONE);
+    KillStone(p);
+    if(Item *it = GetItem(get_pos())) {
+        SendMessage(it, "ignite");
+    } else
+        SetItem(p, it_explosion1);
+}
+
+Value BombStone::message(const string &msg, const Value &) 
+{
+    if (msg =="expl" || msg =="bombstone")
+        change_state(BREAK);
+    return Value();
+}
+
+void BombStone::actor_hit(const StoneContact &sc) 
+{
+    if (enigma::Inventory *inv = player::GetInventory(sc.actor)) {
         if (!inv->is_full()) {
-            Item *it = MakeItem("it-blackbomb");
+            Item *it = MakeItem(itemkind);
             inv->add_item(it);
+            player::RedrawInventory (inv);
         }
     }
 }
@@ -1774,7 +1871,7 @@ namespace
             set_visible_model();
         }
 
-        void message(const string& msg, const Value &val) {
+        virtual Value message(const string& msg, const Value &val) {
             if (msg == "glasses") {
                 if (to_int(val)) {
                     if (!visible) {
@@ -1789,6 +1886,7 @@ namespace
                     }
                 }
             }
+            return Value();
         }
     public:
         DeathStoneInvisible() : visible(false) {}
@@ -1847,10 +1945,11 @@ namespace
             explode();
         }
 
-        void message(const string &msg, const Value &) {
+        virtual Value message(const string &msg, const Value &) {
             if (msg == "expl") {
                 explode();
             }
+            return Value();
         }
 
         bool is_sticky(const Actor *) const 
@@ -1886,7 +1985,7 @@ namespace
     private:
         bool is_floating() const { return true; }
 
-        void message(const string &msg, const Value &val) {
+        virtual Value message(const string &msg, const Value &val) {
             if (msg == "signal") {
                 int ival = to_int (val);
                 if (ival > 0)
@@ -1898,6 +1997,7 @@ namespace
                 lighten();
             else if (msg == "darken")
                 darken();
+            return Value();
         }
     };
     class DiscoLight : public DiscoStone {
@@ -1968,7 +2068,7 @@ namespace
         void actor_hit(const StoneContact &sc)
         {
             if (subtype != MAX_SUBTYPE) {
-                if (player::wielded_item_is (sc.actor, "it-sword")) {
+                if (player::WieldedItemIs (sc.actor, "it-sword")) {
                     subtype += 1;
                     if (subtype == MAX_SUBTYPE) {
                         client::Msg_ShowText ("All right, we'll call it a draw", false, 4.0);
@@ -1980,7 +2080,7 @@ namespace
             }
         }
         void init_model() {
-            set_model(px::strf("st-knight%d", subtype));
+            set_model(ecl::strf("st-knight%d", subtype));
         }
         bool is_floating() const { return subtype == MAX_SUBTYPE; }
     public:
@@ -1989,6 +2089,104 @@ namespace
     DEF_TRAITS(Knight, "st-knight", st_knight);
 }
 
+
+/* -------------------- Polarization Switch stone -------------------- */
+namespace
+{
+    class PolarSwitchStone : public OnOffStone {
+        CLONEOBJ(PolarSwitchStone);
+        DECL_TRAITS;
+    public:
+        PolarSwitchStone() : OnOffStone("st-polarswitch") {}
+    private:
+        void actor_hit(const StoneContact &sc) { set_on(!is_on()); }
+        void init_model() { set_model(is_on() ? "st-glass1" : "st-glass2"); }
+        bool is_transparent(Direction) const { return this->is_on(); }
+        void notify_onoff(bool) { lasers::MaybeRecalcLight(this->get_pos()); }
+    };
+    DEF_TRAITS(PolarSwitchStone, "st-polarswitch", st_polarswitch);
+}
+
+
+/* -------------------- Fire breakable stones -------------------- */
+
+/* These stones mimic the behaviour of the plain-looking stones in
+   Oxyd. */
+namespace
+{
+    class Stone_firebreak : public Stone {
+        CLONEOBJ(Stone_firebreak);
+
+        const char *collision_sound() {return "stone";}
+
+        void break_me() {
+            sound_event("stonedestroy");
+            ReplaceStone(get_pos(), MakeStone("st-plain_breaking"));
+        }
+
+        virtual Value message (const string &msg, const Value &) {
+            if (msg =="heat" || msg == "fire") {
+                break_me();
+                return Value(1.0);
+            }
+            return Value();
+        }
+
+        void actor_hit(const StoneContact &sc) {
+            if (player::WieldedItemIs(sc.actor, "it-brush")) {
+                sound_event("stonepaint");
+                ReplaceStone(get_pos(), MakeStone("st-plain"));
+            } else
+                Stone::actor_hit(sc);
+        }
+
+    public:
+        Stone_firebreak() : Stone("st-firebreak") { }
+    };
+
+    class Stone_movefirebreak : public Stone {
+        CLONEOBJ(Stone_movefirebreak);
+
+        void break_me() {
+            sound_event("stonedestroy");
+            ReplaceStone(get_pos(), MakeStone("st-plain_breaking"));
+        }
+
+        virtual Value message (const string &msg, const Value &) {
+            if (msg =="fire")
+                break_me();
+            return Value();
+        }
+
+        void actor_hit(const StoneContact &sc) {
+            if (player::WieldedItemIs(sc.actor, "it-brush")) {
+                sound_event("stonepaint");
+                ReplaceStone(get_pos(), MakeStone("st-plain_move"));
+            } else
+                Stone::actor_hit(sc);
+        }
+
+        void on_move() {
+            GridPos p = get_pos();
+            if (Floor *fl = GetFloor (p)) {
+                if (fl->is_kind("fl-abyss")) {
+                    ReplaceStone (p, MakeStone("st-plain_falling"));
+                }
+                else if (fl->is_kind("fl-swamp") || fl->is_kind("fl-water")) {
+                    sound_event ("drown");
+                    client::Msg_Sparkle (p.center());
+                    KillStone (p);
+                }
+            }
+        }
+
+        bool is_movable () const { return true; }
+
+    public:
+        Stone_movefirebreak() : Stone("st-firebreak_move") { }
+    };
+}
+
 
 /* -------------------- Functions -------------------- */
 
@@ -1996,14 +2194,14 @@ void world::DefineSimpleStone(const std::string &kind,
                               const std::string &sound,
                               int hollow, int glass)
 {
-    Register(new SimpleStone(kind, sound, hollow, glass));
+    Register(new SimpleStone(kind, sound, hollow != 0, glass != 0));
 }
 
 void world::DefineSimpleStoneMovable(const std::string &kind, 
                                      const std::string &sound, 
                                      int glass)
 {
-    Register(new SimpleStoneMovable(kind, sound, glass));
+    Register(new SimpleStoneMovable(kind, sound, glass != 0));
 }
 
 void stones::Init_simple()
@@ -2014,7 +2212,9 @@ void stones::Init_simple()
     BlackWhiteStone::setup();
 
     Register(new BlockStone);
-    Register(new BombStone);
+    Register(new BombStone("st-bombs", "it-blackbomb"));
+    //Register(new BombStone("st-dynamite", "it-dynamite"));
+    //Register(new BombStone("st-whitebombs", "it-whitebomb"));
     Register(new BrakeStone);
 
     Register(new Break_acblack);
@@ -2056,8 +2256,10 @@ void stones::Init_simple()
     Register(new Window);
 
     Register(new RandomWoodenStone); // random flavor
-    Register(new WoodenStone("st-wood1")); // horizontal planks
-    Register(new WoodenStone("st-wood2")); // vertical planks
+    Register(new WoodenStone("st-wood1", "fl-stwood1")); // horizontal planks
+    Register(new WoodenStone("st-wood2", "fl-stwood2")); // vertical planks
+    Register(new WoodenStone("st-flrock", "fl-rock", true));
+    Register(new WoodenStone("st-flhay", "fl-hay"));
     Register(new WoodenStone_Growing);
     Register(new GreenbrownStone_Growing);
     Register(new VolcanoStone_Growing);
@@ -2065,4 +2267,9 @@ void stones::Init_simple()
     Register(new YinYangStone1);
     Register(new YinYangStone2);
     Register(new YinYangStone3);
+
+    Register(new PolarSwitchStone);
+
+    Register(new Stone_firebreak);
+    Register(new Stone_movefirebreak);
 }
