@@ -13,10 +13,10 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: world.cc,v 1.90 2004/05/22 13:04:24 dheck Exp $
  */
+#include "errors.hh"
 #include "laser.hh"
 #include "player.hh"
 #include "sound.hh"
@@ -32,7 +32,6 @@
 #include <functional>
 #include <map>
 #include <numeric>
-#include <cassert>
 
 
 
@@ -42,7 +41,7 @@
 
 using namespace std;
 using namespace world;
-using namespace px;
+using namespace ecl;
 
 #include "world_internal.hh"
 
@@ -59,7 +58,7 @@ namespace {
       stone. But it's annoying when there are two adjacent stones and
       a marble rebounds from any one (or even both) corners, because
       it changes the direction of the marble quite unpredictably.
-       
+
       This function modifies the contact information for two adjacent
       stones in such a way that the two collisions are treated as a
       single collision with a flat surface. */
@@ -91,7 +90,7 @@ namespace {
             }
         }
     }
-    
+
 /*! Find an already existing contact point in the ContactList that is
   similar to the second argument. */
     bool has_nearby_contact (const ContactList &cl, const Contact &c)
@@ -99,7 +98,8 @@ namespace {
         double posdelta = 0.2;
         double normaldelta = 0.1;
         for (size_t i=0; i<cl.size(); ++i) {
-            if (length (cl[i].pos - c.pos) < posdelta && length (cl[i].normal - c.normal) < normaldelta)
+            if (length (cl[i].pos - c.pos) < posdelta
+                && length (cl[i].normal - c.normal) < normaldelta)
                 return true;
         }
         return false;
@@ -144,7 +144,7 @@ namespace
     {
         Object *src = s->source;
         Object *dst = GetObject(s->destloc);
-        
+
 #if defined(VERBOSE_MESSAGES)
         src->warning("emit_from: msg='%s'", // dest=%i/%i obj=%p",
                      s->message.c_str()
@@ -238,16 +238,16 @@ RubberBand::RubberBand (Actor *a1, Actor *a2, const RubberBandData &d)
   model(display::AddRubber(get_p1(),get_p2())),
   data (d)
 {
-    assert(actor);
-    assert(d.length >= 0);
+    ASSERT(actor, XLevelRuntime, "RubberBand: no actor defined");
+    ASSERT(d.length >= 0, XLevelRuntime, "RubberBand: length negative");
 }
 
 RubberBand::RubberBand (Actor *a1, Stone *st, const RubberBandData &d)
 : actor(a1), actor2(0), stone(st), model(0),
   data (d)
 {
-    assert(actor);
-    assert(d.length >= 0);
+    ASSERT(actor, XLevelRuntime, "RubberBand: no actor defined");
+    ASSERT(d.length >= 0, XLevelRuntime, "RubberBand: length negative");
     model = display::AddRubber(get_p1(), get_p2());
 }
 
@@ -258,7 +258,7 @@ RubberBand::~RubberBand() {
 void RubberBand::apply_forces ()
 {
     V2 v = get_p2()-get_p1();
-    double vv = px::length(v);
+    double vv = ecl::length(v);
 
     if (vv > data.length) {
         V2 force = v * data.strength*(vv-data.length)/vv;
@@ -378,7 +378,7 @@ namespace
     auto_ptr<World> level;
 }
 
-util::Timer    world::GameTimer;
+enigma::Timer  world::GameTimer;
 bool           world::TrackMessages;
 Actor         *world::CurrentCollisionActor = 0;
 
@@ -458,7 +458,7 @@ void World::remove (ForceField *ff)
 
 Object *World::get_named (const string &name)
 {
-    px::Dict<Object*>::iterator found = m_objnames.find(name);
+    ecl::Dict<Object*>::iterator found = m_objnames.find(name);
     if (found != m_objnames.end()) 
         return found->second;
     Log << "Did not find named object: " << name << '\n';
@@ -473,7 +473,7 @@ void World::name_object (Object *obj, const std::string &name)
 
 void World::unname (Object *obj)
 {
-    assert(obj);
+    ASSERT(obj, XLevelRuntime, "unname: no object given");
     string name;
     if (obj->string_attrib("name", &name)) {
         m_objnames.remove(name);
@@ -553,6 +553,10 @@ void World::scramble_puzzles()
 
 /* -------------------- Force calculation -------------------- */
 
+#ifndef M_PI
+#define M_PI 3.1415926535
+#endif
+
 void World::add_mouseforce (Actor *a, Floor *floor, V2 &mforce)
 {
     if (a->get_controllers() != 0) {
@@ -642,12 +646,12 @@ V2 World::get_global_force (Actor *a)
 /* -------------------- Collision handling -------------------- */
 
 struct Ball {
-    px::V2 c;                   // center
+    ecl::V2 c;                   // center
     double r;                   // radius
 };
 
 struct Oblong {
-    px::V2 c;                   // center
+    ecl::V2 c;                   // center
     double w;                   // width
     double h;                   // height
     double erad;                // edge radius
@@ -711,7 +715,7 @@ void World::find_contact_with_stone (Actor *a, GridPos p, StoneContact &c)
             c.normal        = V2(0,-1);
             dist            = y-ay;
         }
-	c.is_contact = (dist-r < contact_e);
+        c.is_contact = (dist-r < contact_e);
     }
     // Closest feature == west or east face of the stone?
     else if (ay>y+erad && ay<y+1-erad) {
@@ -761,7 +765,7 @@ void World::find_contact_with_stone (Actor *a, GridPos p, StoneContact &c)
 void World::find_stone_contacts (Actor *a, StoneContactList &cl)
 {
     ActorInfo &ai = *a->get_actorinfo();
-    px::Rect r(round_down<int>(ai.pos[0]-0.5), round_down<int>(ai.pos[1]-0.5), 1, 1);
+    ecl::Rect r(round_down<int>(ai.pos[0]-0.5), round_down<int>(ai.pos[1]-0.5), 1, 1);
 
     static StoneContact contacts[2][2];
 
@@ -821,7 +825,8 @@ void World::handle_stone_contact (StoneContact &sc)
 
                     if (!slow_collision) {
                         client::Msg_Sparkle (sc.contact_point);
-                        double volume = std::max (1.0, length(ai.vel)/2);
+                        double volume = std::max (0.25, length(ai.vel)/8);
+                        volume = std::min (1.0, volume);
                         sound::SoundEvent (sc.sound.c_str(), sc.contact_point, volume);
                     }
                 }
@@ -840,10 +845,10 @@ void World::handle_stone_contact (StoneContact &sc)
 namespace {
     struct ActorEntry {
         double pos;
-        int idx;
+        size_t idx;
 
         ActorEntry () { pos = 0; idx = 0; }
-        ActorEntry (double pos_, int idx_) { pos = pos_; idx = idx_; }
+        ActorEntry (double pos_, size_t idx_) { pos = pos_; idx = idx_; }
 
         bool operator < (const ActorEntry &x) const {
             return pos < x.pos;
@@ -861,12 +866,12 @@ void World::handle_actor_contacts () {
     }
     sort (xlist.begin(), xlist.end());
     for (size_t i=0; i<nactors; ++i) {
-        int a1 = xlist[i].idx;
+        size_t a1 = xlist[i].idx;
         ActorInfo &ai1 = *actorlist[a1]->get_actorinfo();
         double r1 = ai1.radius;
         double x1 = ai1.pos[0];
         for (size_t j=i+1; j<nactors; ++j) {
-            int a2 = xlist[j].idx;
+            size_t a2 = xlist[j].idx;
             if (xlist[j].pos - x1 < r1 + get_radius(actorlist[a2])) 
                 handle_actor_contact (a1, a2);
             else
@@ -875,7 +880,7 @@ void World::handle_actor_contacts () {
     }
 }
 
-void World::handle_actor_contact (unsigned i, unsigned j)
+void World::handle_actor_contact (size_t i, size_t j)
 {
     Actor *actor1 = actorlist[i];
     ActorInfo &a1 = *actor1->get_actorinfo();
@@ -915,6 +920,7 @@ void World::handle_actor_contact (unsigned i, unsigned j)
 
             if (!has_nearby_contact (a1.contacts, contact)) {
                 double volume = length (force) * ActorTimeStep;
+                volume = std::min(1.0, volume);
                 if (volume > 0.4)
                     sound::SoundEvent ("ballcollision", contact.pos, volume);
             }
@@ -927,7 +933,6 @@ void World::handle_contacts (unsigned actoridx)
     Actor *actor1 = actorlist[actoridx];
     ActorInfo &a1 = *actor1->get_actorinfo();
 
-
     if (a1.ignore_contacts)     // used by the cannonball for example
         return;
 
@@ -936,9 +941,6 @@ void World::handle_contacts (unsigned actoridx)
     find_stone_contacts(actor1, cl);
     for (StoneContactList::iterator i=cl.begin(); i != cl.end(); ++i)
         handle_stone_contact (*i);
-
-//     // Handle contacts with other actors
-//     handle_actor_contacts (actoridx);
 }
 
 /* -------------------- Actor Motion -------------------- */
@@ -997,7 +999,6 @@ void World::move_actors (double dtime)
         for_each (m_rubberbands.begin(), m_rubberbands.end(), 
                   mem_fun(&RubberBand::apply_forces));
 
-
         rest_time -= dt;
     }
 }
@@ -1080,13 +1081,11 @@ void world::Resize (int w, int h)
     display::NewWorld(w, h);
 }
 
-void world::PrepareLevel (int w, int h)
+void world::PrepareLevel ()
 {
     GameTimer.clear();
     CurrentCollisionActor = 0;
-    server::GameReset();
-    player::NewWorld();
-    Resize (w, h);
+    Resize (20, 13);
 }
 
 bool world::InitWorld()
@@ -1109,16 +1108,18 @@ bool world::InitWorld()
         if (a->int_attrib("player", &iplayer)) {
             player::AddActor(iplayer,a);
             if (iplayer == 0) seen_player0 = true;
+        } else {
+            player::AddUnassignedActor(a);
         }
     }
 
     level->changed_stones.clear();
 
     if (!seen_player0) 
-        throw enigma_levels::XLevelLoading("Error: No player 0 defined!");
+        throw XLevelLoading("Error: No player 0 defined!");
 
     world::BroadcastMessage("init", Value(),
-                            GridLayerBits(GRID_ITEMS_BIT | GRID_STONES_BIT));
+        GridLayerBits(GRID_ITEMS_BIT | GRID_STONES_BIT | GRID_FLOOR_BIT));
 
     server::InitMoveCounter();
     STATUSBAR->show_move_counter (server::ShowMoves);
@@ -1209,13 +1210,30 @@ void world::AddRubberBand (Actor *a, Stone *st, const RubberBandData &d)
 void world::AddRubberBand (Actor *a, Actor *a2, const RubberBandData &d)
 {
     RubberBandData rbd (d);
-    rbd.length = px::Max (d.length, get_radius(a) + get_radius(a2));
+    rbd.length = ecl::Max (d.length, get_radius(a) + get_radius(a2));
     level->m_rubberbands.push_back(new RubberBand (a, a2, rbd));
 }
 
+bool world::KillRubberBands (Actor *a)
+{
+    bool didKill = false;
+    for (unsigned i=0; i<level->m_rubberbands.size(); ) {
+        RubberBand &r = *level->m_rubberbands[i];
+        if (r.get_actor() == a || r.get_actor2() == a) {
+            delete &r;
+            level->m_rubberbands.erase(level->m_rubberbands.begin()+i);
+            didKill = true;
+            continue;       // don't increment i
+        }
+        ++i;
+    }
+    return didKill;
+}
+
+
 void world::KillRubberBand (Actor *a, Stone *st)
 {
-    assert(a);
+    ASSERT(a, XLevelRuntime, "KillRubberBand: no actor attached");
     for (unsigned i=0; i<level->m_rubberbands.size(); ) {
         RubberBand &r = *level->m_rubberbands[i];
         if (r.get_actor() == a && r.get_stone() != 0)
@@ -1230,7 +1248,7 @@ void world::KillRubberBand (Actor *a, Stone *st)
 
 void world::KillRubberBand (Actor *a, Actor *a2)
 {
-    assert (a);
+    ASSERT(a, XLevelRuntime, "KillRubberBand: no actor attached");
     for (unsigned i=0; i<level->m_rubberbands.size(); ) {
         RubberBand &r = *level->m_rubberbands[i];
         if (r.get_actor() == a && r.get_actor2() != 0)
@@ -1260,11 +1278,11 @@ void world::GiveRubberBands (Stone *st, vector<Rubber_Band_Info> &rubs) {
    for (unsigned i=0; i<level->m_rubberbands.size(); ) {
         RubberBand &r = *level->m_rubberbands[i];
         if (r.get_stone() == st) {
-	    Rubber_Band_Info rbi;
-	    rbi.act = r.get_actor();
-	    rbi.data = r.get_data();
-	    rubs.push_back(rbi);
-	}
+            Rubber_Band_Info rbi;
+            rbi.act = r.get_actor();
+            rbi.data = r.get_data();
+            rubs.push_back(rbi);
+        }
         ++i;
     }
 }
@@ -1334,25 +1352,28 @@ bool world::GetSignalTargetPos (Object *src, GridPos &pos, int signalidx)
 }
 
 
-void world::SendMessage(Object *o, const std::string &msg) 
+Value world::SendMessage(Object *o, const std::string &msg) 
 {
-    SendMessage (o, Message (msg, Value()));
+    return SendMessage (o, Message (msg, Value()));
 }
 
-void world::SendMessage(Object *o, const std::string &msg, const Value& value)
+Value world::SendMessage(Object *o, const std::string &msg, const Value& value)
 {
-    SendMessage (o, Message (msg, value));
+    return SendMessage (o, Message (msg, value));
 }
 
-void world::SendMessage (Object *o, const Message &m)
+Value world::SendMessage (Object *o, const Message &m)
 {
     if (o) {
         if (TrackMessages)
             o->warning("will be sent message '%s' (with Value)", m.message.c_str());
-        o->on_message (m);
+        return o->on_message(m);
     }
-    else if (TrackMessages)
+    else if (TrackMessages) {
         fprintf(stderr, "Sending message '%s' to NULL-object\n", m.message.c_str());
+        return Value();
+    }
+    return Value();
 }
 
 
@@ -1362,9 +1383,9 @@ void world::BroadcastMessage (const std::string& msg,
 {
     int  width     = level->w;
     int  height    = level->h;
-    bool to_floors = grids & GRID_FLOOR_BIT;
-    bool to_items  = grids & GRID_ITEMS_BIT;
-    bool to_stones = grids & GRID_STONES_BIT;
+    bool to_floors = (grids & GRID_FLOOR_BIT) != 0;
+    bool to_items  = (grids & GRID_ITEMS_BIT) != 0;
+    bool to_stones = (grids & GRID_STONES_BIT) != 0;
 
     for (int y = 0; y<height; ++y) {
         for (int x = 0; x<width; ++x) {
@@ -1391,8 +1412,8 @@ void world::PerformAction (Object *o, bool onoff)
 #endif // VERBOSE_MESSAGES
 
     if (action == "callback") {
-        if (lua::CallFunc(lua::LevelState(), target.c_str(), Value(onoff)) != 0) {
-            throw enigma_levels::XLevelRuntime(string("callback '")+target+"' failed:\n"+lua::LastError(lua::LevelState()));
+        if (lua::CallFunc(lua::LevelState(), target.c_str(), Value(onoff), o) != 0) {
+            throw XLevelRuntime(string("callback '")+target+"' failed:\n"+lua::LastError(lua::LevelState()));
         }
     }
     else if (action == "signal") {
@@ -1415,8 +1436,8 @@ namespace
 {
     void explosion (GridPos p, ItemID explosion_item)
     {
-        Stone *stone = GetStone(p);
-        SendMessage(stone, "expl");
+        if (Stone *stone = GetStone(p))
+            SendMessage(stone, "expl");
         if (Item  *item  = GetItem(p)) {
             if (has_flags(item, itf_indestructible))
                 SendMessage(item, "expl");
@@ -1425,6 +1446,8 @@ namespace
         }
         else
             SetItem(p, explosion_item);
+        if (Floor *floor = GetFloor(p))
+            SendMessage(floor, "expl");
     }
 }
 
@@ -1433,42 +1456,48 @@ void world::SendExplosionEffect(GridPos center, ExplosionType type)
     const int AFFECTED_FIELDS       = 8;
 
     for (int a = 0; a<AFFECTED_FIELDS; ++a) {
-	GridPos  dest = get_neighbour (center, a+1);
-	Item    *item            = GetItem(dest);
-	Stone   *stone           = GetStone(dest);
-	bool     direct_neighbor = a<4;
+        GridPos  dest            = get_neighbour (center, a+1);
+        Item    *item            = GetItem (dest);
+        Stone   *stone           = GetStone (dest);
+        Floor   *floor           = GetFloor (dest);
+        bool     direct_neighbor = a<4;
 
-	switch (type) {
-	case EXPLOSION_DYNAMITE:
-	    if (stone) SendMessage(stone, "ignite");
-	    if (item) SendMessage(item, "ignite");
-	    break;
+        switch (type) {
+        case EXPLOSION_DYNAMITE:
+            if (stone) SendMessage(stone, "ignite");
+            if (item) SendMessage(item, "ignite");
+            if (floor) SendMessage(floor, "ignite");
+            break;
 
-	case EXPLOSION_BLACKBOMB:
-	    if (direct_neighbor) {
+        case EXPLOSION_BLACKBOMB:
+            if (direct_neighbor) {
                 explosion (dest, it_explosion1);
-            } 
-            else {
-                SendMessage(stone, "ignite");
-                SendMessage(item, "ignite");
+            } else {
+                // Note: should not ignite in non-enigma-mode!
+                if (stone) SendMessage(stone, "ignite");
+                if (item) SendMessage(item, "ignite");
+                if (floor) SendMessage(floor, "ignite");
             }
             break;
 
-	case EXPLOSION_WHITEBOMB:
-            explosion (dest, it_explosion3);
-	    break;
+        case EXPLOSION_WHITEBOMB:
+            // Note: at least in oxyd1 only direct neighbors
+            // explode, and the others not even ignite
+            explosion (dest, it_explosion3);            
+            break;
 
-	case EXPLOSION_BOMBSTONE:
-	    if (direct_neighbor) {
-		if (stone) SendMessage(stone, "bombstone");
-		if (item) SendMessage(item, "bombstone");
-	    }
-	    break;
+        case EXPLOSION_BOMBSTONE:
+            if (direct_neighbor) {
+                if (stone) SendMessage(stone, "bombstone");
+                if (item) SendMessage(item, "bombstone");
+                if (floor) SendMessage(floor, "bombstone");
+            }
+            break;
 
         case EXPLOSION_SPITTER:
-
+            // TODO: spitter explosions
             break;
-	}
+        }
     }
 }
 
@@ -1490,15 +1519,18 @@ const Field *world::GetField (GridPos p)
 
 /* -------------------- Floor manipulation -------------------- */
 
-void world::KillFloor(GridPos p) {
+void world::KillFloor(GridPos p) 
+{
     level->fl_layer.kill(p);
 }
 
-Floor *world::GetFloor(GridPos p) {
+Floor *world::GetFloor(GridPos p) 
+{
     return level->fl_layer.get(p);
 }
 
-void world::SetFloor(GridPos p, Floor* fl) {
+void world::SetFloor(GridPos p, Floor* fl) 
+{
     level->fl_layer.set(p,fl);
     if (!level->preparing_level)
         if (Stone *st = GetStone(p))
@@ -1692,7 +1724,7 @@ void world::ReleaseActor(Actor *a)
     a->get_actorinfo()->grabbed = false;
 }
 
-bool world::GetActorsInRange (px::V2 center, double range,
+bool world::GetActorsInRange (ecl::V2 center, double range,
                               vector<Actor*>& actors)
 {
     ActorList &al = level->actorlist;
@@ -1769,7 +1801,7 @@ void world::Shutdown()
 /* -------------------- Object repository -------------------- */
 namespace
 {
-    class ObjectRepos : public px::Nocopy {
+    class ObjectRepos : public ecl::Nocopy {
     public:
         ObjectRepos();
         ~ObjectRepos();
@@ -1793,7 +1825,7 @@ ObjectRepos::ObjectRepos() {
 
 ObjectRepos::~ObjectRepos()
 {
-    px::delete_map(objmap.begin(), objmap.end());
+    ecl::delete_map(objmap.begin(), objmap.end());
 }
 
 
@@ -1855,6 +1887,7 @@ namespace
     ObjectRepos *repos;
     vector<Actor *> actor_repos(ac_COUNT);
     vector<Stone *> stone_repos(st_COUNT);
+    vector<Item *> item_repos(it_COUNT);
 }
 
 void world::Register (const string &kind, Object *obj) {
@@ -1886,7 +1919,8 @@ void world::RegisterStone (Stone *stone)
 {
     Register(static_cast<Object*>(stone));
     StoneID id = get_id(stone);
-    assert (id != st_INVALID);
+    ASSERT (id != st_INVALID, XLevelRuntime,
+        "RegisterStone: trying to register with invalid ID");
     stone_repos[id] = stone;
 }
 
@@ -1894,7 +1928,8 @@ void world::RegisterActor (Actor *actor)
 {
     Register(static_cast<Object*>(actor));
     ActorID id = get_id(actor);
-    assert (id != ac_INVALID);
+    ASSERT (id != ac_INVALID, XLevelRuntime,
+        "RegisterActor: trying to register with invalid ID");
     actor_repos[id] = actor;
 }
 
@@ -1944,7 +1979,7 @@ Actor *world::MakeActor (ActorID id)
     if (Actor *ac = actor_repos[id])
         return ac->clone();
     else
-        assert (0 || "internal error: built-in actor not defined!?");
+        ASSERT(0, XLevelRuntime, "MakeActor: no actor for ID defined");
     return 0;
 }
 
@@ -1953,7 +1988,7 @@ Stone *world::MakeStone (StoneID id)
     if (Stone *st = stone_repos[id])
         return st->clone();
     else
-        assert (0 || "internal error: built-in stone not defined!?");
+        ASSERT(0, XLevelRuntime, "MakeStone: no stone for ID defined");
     return 0;
 }
 
@@ -1965,21 +2000,20 @@ void world::DisposeObject(Object *o) {
     }
 }
 
-void world::DefineSimpleFloor(const std::string &kind,
-                              double friction, double mousefactor)
+void world::DefineSimpleFloor(const std::string &kind, double friction,
+                              double mousefactor, bool burnable,
+                              const std::string &firetransform)
 {
-    Register(new Floor(kind.c_str(), friction, mousefactor));
+    Register(new Floor(kind.c_str(), friction, mousefactor,
+             flf_default, burnable ? flft_burnable : flft_default,
+             firetransform.c_str(), ""));
 }
 
 void world::DumpObjectInfo() {
     repos->dump_info();
 }
 
-
-namespace
-{
-    vector<Item *> item_repos(it_COUNT);
-}
+/* ------------------- Item repository ------------------- */
 
 void world::Register (const string &kind, Item *obj) 
 {
@@ -1991,7 +2025,8 @@ void world::RegisterItem (Item *item)
 {
     Register(static_cast<Object*>(item));
     ItemID id = get_id(item);
-    assert (id != it_INVALID);
+    ASSERT(id != it_INVALID, XLevelRuntime,
+        "RegisterItem: trying to register with invalid ID");
     item_repos[id] = item;
 }
 
@@ -2000,7 +2035,7 @@ Item *world::MakeItem (ItemID id)
     if (Item *it = item_repos[id])
         return it->clone();
     else
-        assert (0 || "internal error: built-in item not defined!?");
+        ASSERT(0, XLevelRuntime, "MakeItem: no item for ID defined");
     return 0;
 }
 
